Package: altmeta
Type: Package
Title: Alternative Meta-Analysis Methods
Version: 4.3.1
Date: 2026-04-29
Authors@R: 
  c(person(given = "Lifeng", family = "Lin",
    role = c("aut", "cre"), email = "lifenglin@arizona.edu",
    comment = c(ORCID = "0000-0002-3562-9816")),
    person(given = "Yaqi", family = "Jing", role = c("ctb")),
    person(given = "Kristine J.", family = "Rosenberger", role = c("ctb")),
    person(given = "Linyu", family = "Shi", role = c("ctb")),
    person(given = "Yipeng", family = "Wang", role = c("ctb")),
    person(given = "Xing", family = "Xing", role = c("ctb"),
    comment = c(ORCID = "0000-0001-9394-8957")),
    person(given = "Zhiyuan", family = "Yu", role = c("ctb")),
    person(given = "Haitao", family = "Chu",
    role = c("aut"), email = "chux0051@umn.edu",
    comment = c(ORCID = "0000-0003-0932-598X")))
Maintainer: Lifeng Lin <lifenglin@arizona.edu>
Depends: R (>= 3.5.0)
Imports: rjags (>= 4-6), coda, graphics, grDevices, lme4, Matrix,
        metafor (>= 3.0-2), methods, stats, utils
SystemRequirements: JAGS 4.x.y (http://mcmc-jags.sourceforge.net)
Description: Provides alternative statistical methods for meta-analysis, including:
 - bivariate generalized linear mixed models for synthesizing odds ratios, relative risks,
   and risk differences
   (Chu et al., 2012 <doi:10.1177/0962280210393712>)
 - tests and measures for between-study heterogeneity
   (Lin et al., 2017 <doi:10.1111/biom.12543>;
    Wang et al., 2022 <doi:10.1002/sim.9261>;
    Yu et al., 2025 <doi:10.1186/s12874-025-02719-7>);
 - measures, tests, and visualization tools for publication bias or small-study effects
   (Lin and Chu, 2018 <doi:10.1111/biom.12817>;
    Lin, 2019 <doi:10.1002/jrsm.1340>;
    Lin, 2020 <doi:10.1177/0962280220910172>;
    Shi et al., 2020 <doi:10.1002/jrsm.1415>);
 - meta-analysis of combining standardized mean differences and odds ratios
   (Jing et al., 2023 <doi:10.1080/10543406.2022.2105345>);
 - meta-analysis of diagnostic tests for synthesizing sensitivities, specificities, etc.
   (Reitsma et al., 2005 <doi:10.1016/j.jclinepi.2005.02.022>;
    Chu and Cole, 2006 <doi:10.1016/j.jclinepi.2006.06.011>);
 - meta-analysis methods for synthesizing proportions
   (Lin and Chu, 2020 <doi:10.1097/ede.0000000000001232>);
 - models for multivariate meta-analysis, measures of inconsistency degrees of freedom
   in Bayesian network meta-analysis, and predictive P-score
   (Lin and Chu, 2018 <doi:10.1002/jrsm.1293>;
    Lin, 2020 <doi:10.1080/10543406.2020.1852247>;
    Rosenberger et al., 2021 <doi:10.1186/s12874-021-01397-5>).
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2026-04-29 23:49:58 UTC; lifen
Author: Lifeng Lin [aut, cre] (ORCID: <https://orcid.org/0000-0002-3562-9816>),
  Yaqi Jing [ctb],
  Kristine J. Rosenberger [ctb],
  Linyu Shi [ctb],
  Yipeng Wang [ctb],
  Xing Xing [ctb] (ORCID: <https://orcid.org/0000-0001-9394-8957>),
  Zhiyuan Yu [ctb],
  Haitao Chu [aut] (ORCID: <https://orcid.org/0000-0003-0932-598X>)
Repository: CRAN
Date/Publication: 2026-04-30 08:30:36 UTC
Built: R 4.6.0; ; 2026-04-30 11:23:45 UTC; unix

S3method(plot, meta.dt)
S3method(plot, metaoutliers)
S3method(print, meta.dt)
S3method(print, metahet.hybrid)

export(maprop.glmm, maprop.twostep, meta.biv, meta.dt, meta.or.smd, meta.pen, metahet, metahet.hybrid, metaoutliers, metapb, mvma, mvma.bayesian, mvma.hybrid, mvma.hybrid.bayesian, nma.icdf, nma.predrank, pb.bayesian.binary, pb.hybrid.binary, pb.hybrid.generic, ssfunnel)

import("graphics", "rjags", "stats")

importFrom("coda", "gelman.diag")
importFrom("grDevices", "adjustcolor", "dev.off", "png")
importFrom("lme4", "glmer")
importFrom("Matrix", "rankMatrix")
importFrom("metafor", "rma", "rma.mv")
importFrom("methods", "is")
importFrom("utils", "write.table")

pb.bay <- function(y, s2 = NA, sig.level = 0.1, n00, n01, n10, n11, het = "mul", 
                   sd.prior = "unif", n.adapt = 1000, n.chains = 3, 
                   n.burnin = 5000, n.iter = 10000, thin = 2, upp.het = 2, 
                   phi = 0.5, coda = FALSE, traceplot = FALSE){
  out <- NULL
  if(is.element("mul", het)){
    
    if(is.element("unif", sd.prior)){
      modelstring <- function(o){
        out <- "
        model{
        for(i in 1:N){
        y[i] ~ dnorm(alpha + beta*se[i], prec[i])
        prec[i] <- pow(kappa*se[i], -2)
        se[i] <- sqrt(1/(n0[i]*p01[i]*(1-p01[i])) +  1/(n1[i]*p11[i]*(1-p11[i])))
        n01[i] ~ dbin(p01[i], n0[i])
        n11[i] ~ dbin(p11[i], n1[i])
        logit(p01[i]) <- mu[i]
        logit(p11[i]) <- mu[i] + delta[i]
        delta[i] ~ dnorm(d, inv.tau2)
        mu[i] ~ dnorm(0, 0.0001)
        }
        alpha ~ dnorm(0, 0.0001)
        beta ~ dnorm(0, 0.0001)
        kappa ~ dunif(0, upp.kappa)
        d ~ dnorm(0, 0.0001)
        inv.tau2 <- pow(tau, -2)
        tau ~ dunif(0, upp.tau)
        }"
  
        return(out)
      }
      
      jags.dat <- list(N = length(y), y = y,
                       n0 = n00 + n01, n1 = n10 + n11, n01 = n01, n11 = n11,
                       upp.kappa = upp.het, upp.tau = upp.het)
      inits <- NULL
      rng.seeds <- sample(1000000, n.chains)
      for(i in 1:n.chains){
        inits[[i]] <- list(alpha = rnorm(1), beta = rnorm(1), d = rnorm(1),
                           kappa = runif(1, 0.1, 1.9), tau = runif(1, 0.1, 1.9),
                           .RNG.name = "base::Wichmann-Hill", .RNG.seed = rng.seeds[i])
      }
  }
    
    if(is.element("hn", sd.prior)){
      modelstring <- function(o){
        out <- "
        model{
        for(i in 1:N){
        y[i] ~ dnorm(alpha + beta*se[i], prec[i])
        prec[i] <- pow(kappa*se[i], -2)
        se[i] <- sqrt(1/(n0[i]*p01[i]*(1-p01[i])) +  1/(n1[i]*p11[i]*(1-p11[i])))
        n01[i] ~ dbin(p01[i], n0[i])
        n11[i] ~ dbin(p11[i], n1[i])
        logit(p01[i]) <- mu[i]
        logit(p11[i]) <- mu[i] + delta[i]
        delta[i] ~ dnorm(d, inv.tau2)
        mu[i] ~ dnorm(0, 0.0001)
        }
        alpha ~ dnorm(0, 0.0001)
        beta ~ dnorm(0, 0.0001)
        kappa ~ dnorm(0, inv.phi2)I(0,)
        d ~ dnorm(0, 0.0001)
        inv.tau2 <- pow(tau, -2)
        tau ~ dnorm(0, inv.phi2)I(0,)
        }"
  
        return(out)
      }
      
      jags.dat <- list(N = length(y), y = y,
                       n0 = n00 + n01, n1 = n10 + n11, n01 = n01, n11 = n11,
                       inv.phi2 = phi^(-2))
      inits <- NULL
      rng.seeds <- sample(1000000, n.chains)
      for(i in 1:n.chains){
        inits[[i]] <- list(alpha = rnorm(1), beta = rnorm(1), d = rnorm(1),
                           kappa = runif(1, 0.1, 1.9), tau = runif(1, 0.1, 1.9),
                           .RNG.name = "base::Wichmann-Hill", .RNG.seed = rng.seeds[i])
      }
}
    }
  
  
  if(is.element("add", het)){
    if(is.element("unif", sd.prior)){
      modelstring <- function(o){
        out <- "
        model{
        for(i in 1:N){
        y[i] ~ dnorm(alpha + beta*se[i], prec[i])
        prec[i] <- 1/(pow(gamma, 2) + pow(se[i], 2))
        se[i] <- sqrt(1/(n0[i]*p01[i]*(1-p01[i])) +  1/(n1[i]*p11[i]*(1-p11[i])))
        n01[i] ~ dbin(p01[i], n0[i])
        n11[i] ~ dbin(p11[i], n1[i])
        logit(p01[i]) <- mu[i]
        logit(p11[i]) <- mu[i] + delta[i]
        delta[i] ~ dnorm(d, inv.tau2)
        mu[i] ~ dnorm(0, 0.0001)
        }
        alpha ~ dnorm(0, 0.0001)
        beta ~ dnorm(0, 0.0001)
        gamma ~ dunif(0, upp.gamma)
        d ~ dnorm(0, 0.0001)
        inv.tau2 <- pow(tau, -2)
        tau ~ dunif(0, upp.tau)
        }"
  
        return(out)
      }
      
      jags.dat <- list(N = length(y), y = y,
                       n0 = n00 + n01, n1 = n10 + n11, n01 = n01, n11 = n11,
                       upp.gamma = upp.het, upp.tau = upp.het)
      inits <- NULL
      rng.seeds <- sample(1000000, n.chains)
      for(i in 1:n.chains){
        inits[[i]] <- list(alpha = rnorm(1), beta = rnorm(1), d = rnorm(1),
                           gamma = runif(1, 0.1, 1.9), tau = runif(1, 0.1, 1.9),
                           .RNG.name = "base::Wichmann-Hill", .RNG.seed = rng.seeds[i])
      }
  }
    
    if(is.element("hn", sd.prior)){
      modelstring <- function(o){
        out <- "
        model{
        for(i in 1:N){
        y[i] ~ dnorm(alpha + beta*se[i], prec[i])
        prec[i] <- 1/(pow(gamma, 2) + pow(se[i], 2))
        se[i] <- sqrt(1/(n0[i]*p01[i]*(1-p01[i])) +  1/(n1[i]*p11[i]*(1-p11[i])))
        n01[i] ~ dbin(p01[i], n0[i])
        n11[i] ~ dbin(p11[i], n1[i])
        logit(p01[i]) <- mu[i]
        logit(p11[i]) <- mu[i] + delta[i]
        delta[i] ~ dnorm(d, inv.tau2)
        mu[i] ~ dnorm(0, 0.0001)
        }
        alpha ~ dnorm(0, 0.0001)
        beta ~ dnorm(0, 0.0001)
        gamma ~ dnorm(0, inv.phi2)I(0,)
        d ~ dnorm(0, 0.0001)
        inv.tau2 <- pow(tau, -2)
        tau ~ dnorm(0, inv.phi2)I(0,)
        }"
  
        return(out)
      }
      
      jags.dat <- list(N = length(y), y = y,
                       n0 = n00 + n01, n1 = n10 + n11, n01 = n01, n11 = n11,
                       inv.phi2 = phi^(-2))
      inits <- NULL
      rng.seeds <- sample(1000000, n.chains)
      for(i in 1:n.chains){
        inits[[i]] <- list(alpha = rnorm(1), beta = rnorm(1), d = rnorm(1),
                           gamma = runif(1, 0.1, 1.9), tau = runif(1, 0.1, 1.9),
                           .RNG.name = "base::Wichmann-Hill", .RNG.seed = rng.seeds[i])
      }
      }
    }
  
  jags.m <- jags.model(file = textConnection(modelstring()),
                       data = jags.dat, inits = inits, n.chains = n.chains, n.adapt = n.adapt)
  update(jags.m, n.iter = n.burnin)
  params <- c("beta")
  samps <- coda.samples(model = jags.m, variable.names = params,
                        n.iter = n.iter, thin = thin)
  quants <- quantile(unlist(samps),
                     probs = c(sig.level/2, 1 - sig.level/2, 0.5))
  
  if(traceplot){
    par(mfrow = c(n.chains,1))
    for(i in 1:n.chains){
      temp <- as.vector(samps[[i]])
      tp <- plot(temp, type = "l", col = "red", ylab = paste("beta[", i, "]", sep = ""), 
                 xlab = "Iterations", main = paste("Chain",i))
    }  
    print(tp)
  }
  
  out$est.bay <- as.numeric(quants[3])
  out$ci.bay <- quants[c(1, 2)]
  
  temp <- name <- NULL
  if(coda){
    for(i in 1:n.chains){
      temp <- cbind(temp, as.vector(samps[[i]]))
      name <- c(name, paste("beta[", i, "]", sep = ""))
    }
    beta <- data.frame(temp)
    names(beta) <- name
    out$samps.bay <- beta
  }
  
  return(out)
  }

nma.predrank <- function(sid, tid, r, n, data, n.adapt = 1000, n.chains = 3, n.burnin = 2000,
                         n.iter = 5000, n.thin = 2, lowerbetter = TRUE, pred = TRUE,
                         pred.samples = FALSE, trace = FALSE){
  if(missing(sid)) stop("need to specify study id from 1 to NS for each study.")
  if(missing(tid)) stop("need to specify treatment id from 1 to NT for each arm in each study.")
  if(missing(r)) stop("need to specify numer of responses for each arm in each study.")
  if(missing(n)) stop("need to specify number of subjects for each arm in each study.")
  if(!missing(data)){
    sid <- eval(substitute(sid), data, parent.frame())
    tid <- eval(substitute(tid), data, parent.frame())
    r <- eval(substitute(r), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if((length(sid) != length(tid)) | (length(sid) != length(r)) | (length(sid) != length(n))){
    stop("the input data have different dimensions.")
  }
  if(max(sid) < 2) stop("less than two studies in the network meta-analysis.")
  
  BayesianNMAModel <- function(o){
    out <- "
    model{
    for(i in 1:NS){
    w[i,1] <- 0
    delta[i,t[i,1]] <- 0
    mu[i] ~ dnorm(0, 0.0001)  #vague priors for trial baselines
    for(k in 1:na[i]){
    r[i,k] ~ dbin(p[i,t[i,k]], n[i,k])  #binomial likelihood
    logit(p[i,t[i,k]]) <- mu[i] + delta[i,t[i,k]]  #model
    }
    for(k in 2:na[i]){
    # trial-specific log OR
    delta[i,t[i,k]] ~ dnorm(md[i,t[i,k]], taud[i,t[i,k]])
    md[i,t[i,k]] <- d[t[i,k]] - d[t[i,1]] + sw[i,k]	#mean of LOR
    taud[i,t[i,k]] <- tau*2*(k - 1)/k  #precision of LOR
    # adjustment of multi-arm trials
    w[i,k] <- delta[i,t[i,k]] - d[t[i,k]] + d[t[i,1]]
    sw[i,k] <- sum(w[i,1:(k-1)])/(k - 1)
    }
    }
    
    d[1] <- 0
    for(k in 2:NT){
    d[k] ~ dnorm(0, 0.0001)
    }
    
    tau <- pow(sigma, -2)
    sigma ~ dunif(0, 5)  #heterogeneity standard deviation
    
    # pairwise ORs
    for(h in 1:(NT - 1)){
    for(k in (h + 1):NT){
    lor[h,k] <- d[k] - d[h]
    }
    }
    
    rank <- rank(d)
    }"
return(out)
  }
  
  dat <- data.frame(sid = sid, tid = tid, r = r, n = n)
  NS <- max(dat$sid)
  NT <- max(dat$tid) 
  na <- as.numeric(table(dat$sid)) 
  r <- n <- t <- matrix(NA, nrow = NS, ncol = max(na))
  for(i in 1:NS){
    r[i, 1:na[i]] <- dat$r[dat$sid == i]
    n[i, 1:na[i]] <- dat$n[dat$sid == i]
    t[i, 1:na[i]] <- dat$tid[dat$sid == i]
  }
  
  jags.dat <- list(NS = NS, NT = NT, na = na, r = r, n = n, t = t)
  jags.inits <- list(
    list(mu = rep(0, NS), d = c(NA, rep(0, NT - 1)), sigma = 0.5),
    list(mu = rep(0.5, NS), d = c(NA, rep(-0.5, NT - 1)), sigma = 0.3),
    list(mu = rep(-0.5, NS), d = c(NA, rep(0.5, NT - 1)), sigma = 0.8))
  jags.m <- jags.model(file = textConnection(BayesianNMAModel()),
                       data = jags.dat, inits = jags.inits,
                       n.chains = n.chains, n.adapt = n.adapt)
  update(jags.m, n.iter = n.burnin)
  params <- c("d","sigma")
  coda <- coda.samples(model = jags.m, variable.names = params,
                       n.iter = n.iter, thin = n.thin)
  
  all.post <- NULL
  for(i in 1:n.chains){
    all.post <- rbind(all.post, coda[[i]])
  }
  all.post <- t(all.post)
  
  iter.post <- dim(all.post)[2]
  d.post <- all.post[paste0("d[", 1:NT, "]"),]
  dim <- ncol(d.post)
  all.trts <- 1:NT
  
  P <- rep(list(NULL), NT)
  for(k in 1:NT){
    x <- (matrix(rep(d.post[k,], NT - 1), byrow = TRUE, nrow = NT - 1) 
          - d.post[-k,])
    x <- t(x)
    y <- matrix(NA, nrow = dim, ncol = NT-1)
    yavg <- matrix(NA, nrow = dim, ncol = 1)
    for(p in 1:dim){
      for(q in 1:(NT-1))
        if(!lowerbetter){
          if (x[p,q] > 0) {y[p,q] = 1} 
          else(y[p,q] = 0)
        }else {
          if (x[p,q] < 0) {y[p,q] = 1} 
          else(y[p,q] = 0)
        }
      yavg[p] = mean(y[p,])
    }
    P[[k]] <- yavg
  }
  
  P.score <-  matrix(NA, nrow = NT, ncol = 4)
  colnames(P.score) <-
    c("Mean", "Median", "95% CrI LB", "95% CrI UB")
  for(k in 1:NT){
    P.score[k,1] <- mean(P[[k]])
    P.score[k, 2:4] <- as.numeric(quantile(P[[k]],
                                           probs = c(0.5, 0.025, 0.975)))
  }
  
  if(pred){
    if(lowerbetter) d.post <- -d.post
    sigma.post <- all.post["sigma",]
    P.pred <- rep(list(NULL), NT)
    for(k in 1:NT){
      x <-
        (matrix(rep(d.post[k,], NT - 1), byrow = TRUE, nrow = NT - 1) -
           d.post[-k,])/
        matrix(rep(sigma.post, NT - 1), byrow = TRUE, nrow = NT - 1)
      x <- t(x)
      out <- pnorm(x, mean = rep(0, NT - 1), sd = 1)
      yavg <- matrix(NA, nrow = dim, ncol = 1)
      for(p in 1:dim){
        yavg[p] = mean(out[p,])
      }
      P.pred[[k]] <- yavg
    }
    
    P.score.pred <- matrix(NA, nrow = NT, ncol = 4)
    colnames(P.score.pred) <-
      c("Mean", "Median", "95% CrI LB", "95% CrI UB")
    for(k in 1:NT){
      P.score.pred[k,1] <- mean(P.pred[[k]])
      P.score.pred[k, 2:4] <- as.numeric(quantile(P.pred[[k]],
                                                  probs = c(0.5, 0.025, 0.975)))
    }
    
    out <- list(P.score = P.score, P.score.pred = P.score.pred)
    if(pred.samples) out$P.pred = P.pred
  }else{
    out <- list(P.score = P.score)
  }
  
  if(trace) out$coda <- coda
  
  class(out) <- "nma.predrank"
  return(out)
}

tau2.m.solver <- function(w, Qm){
	f <- function(tau2){
		out <- sum(sqrt(1 + w*tau2)) - Qm*sqrt(3.14159/2)
		return(out)
	}
	f <- Vectorize(f)
	n <- length(w)
	tau2.upp <- sum(1/w)*(Qm^2/n*2/3.14159 - 1)
	tau2.upp <- max(c(tau2.upp, 0.01))
	f.low <- f(0)
	f.upp <- f(tau2.upp)
	if(f.low*f.upp > 0){
		tau2.m <- 0
	}else{
		tau2.m <- uniroot(f, interval = c(0, tau2.upp))
		tau2.m <- tau2.m$root
	}
	return(tau2.m)
}

pb.rank <- function(y, s2){
  theta.hat <- sum(y/s2)/sum(1/s2)
  s2.star <- s2 - 1/sum(1/s2)
  y.star <- (y - theta.hat)/sqrt(s2.star)
  if(all(abs(diff(y.star[!is.na(y.star)])) < 1e-5) | all(abs(diff(s2.star[!is.na(s2.star)])) < 1e-5)){
    out <- list(pval = 1, stat = 0)
  }else{
    out <- cor.test(x = y.star, y = s2, alternative = "two.sided", method = "kendall", exact = FALSE)
    out <- list(pval = out$p.value, stat = as.numeric(out$statistic))
  }
  return(out)
}

pb.reg <- function(y, s2, n = NA, n00 = NA, n01 = NA, n10 = NA, n11 = NA){
  x.reg <- sqrt(s2)
  y.reg <- y
  if(all(abs(diff(x.reg[!is.na(x.reg)])) < 1e-5) | all(abs(diff(y.reg[!is.na(y.reg)])) < 1e-5)){
    out <- list(pval = 1, stat = 0, coef = c(0, 0), std.res = y.reg/sqrt(s2))
  }else{
    w.reg <- 1/s2
    X <- cbind(1, x.reg)
    Y <- as.matrix(y.reg)
    W <- diag(w.reg)
    XWX <- t(X) %*% W %*% X
    if(as.numeric(rankMatrix(XWX)) < dim(XWX)[1]){
      out <- list(pval = 1, stat = 0, coef = c(0, 0), std.res = y.reg/sqrt(s2))
    }else{
      inv <- solve(XWX)
      coef <- inv %*% t(X) %*% W %*% Y
      var.coef <- inv
      pval <- as.numeric(2*pnorm(-abs(coef[2, 1])/sqrt(var.coef[2, 2])))
      res <- Y - X %*% coef
      std.res <- res/sqrt(s2)
      out <- list(pval = pval, stat = as.numeric(coef[2, 1]/sqrt(var.coef[2, 2])), coef = as.numeric(coef), std.res = std.res)
    }
  }
  return(out)
}

metapb <- function(y, s2, data, model = "RE", n.resam = 1000){
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
  }
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  n <- length(y)
  w <- 1/s2
  mu.bar <- sum(w*y)/sum(w)
  Q <- sum(w*(y - mu.bar)^2)
  p.Q <- 1 - pchisq(Q, df = n - 1)
  I2 <- (Q - n + 1)/Q
  tau2 <- (Q - n + 1)/(sum(w) - sum(w^2)/sum(w))
  tau2 <- max(c(0, tau2))

  if(missing(model)){
    if(p.Q >= 0.05){
      model <- "FE"
    }else{
      model <- "RE"
    }
  }

  if(!is.element(model, c("FE", "RE"))){
    stop("wrong input for the argument model.")
  }

  if(model == "FE"){
    y.reg <- y/sqrt(s2)
    x.reg <- 1/sqrt(s2)
    mu.est <- sum(y/s2)/sum(1/s2)
  }else{
    y.reg <- y/sqrt(s2 + tau2)
    x.reg <- 1/sqrt(s2 + tau2)
    mu.est <- sum(y/(s2 + tau2))/sum(1/(s2 + tau2))
  }
  reg <- lm(y.reg ~ x.reg)
  reg.coef <- summary(reg)$coef
  reg.int <- reg.coef["(Intercept)", "Estimate"]
  reg.int.se <- reg.coef["(Intercept)", "Std. Error"]
  reg.int.ci.low <- reg.coef["(Intercept)", "Estimate"] -
    qt(0.975, df = n - 2)*reg.coef["(Intercept)", "Std. Error"]
  reg.int.ci.upp <- reg.coef["(Intercept)", "Estimate"] +
    qt(0.975, df = n - 2)*reg.coef["(Intercept)", "Std. Error"]
  reg.int.ci <- c("2.5%" = reg.int.ci.low, "97.5%" = reg.int.ci.upp)
  reg.pval <- reg.coef["(Intercept)", "Pr(>|t|)"]

  std.dev <- as.numeric(summary(reg)$residuals)
  cm2 <- var(std.dev)
  cm3 <- mean((std.dev - mean(std.dev))^3)
  cm4 <- mean((std.dev - mean(std.dev))^4)
  cm5 <- mean((std.dev - mean(std.dev))^5)
  cm6 <- mean((std.dev - mean(std.dev))^6)
  skewness <- cm3/(cm2^(1.5))

  var.skew <- 9 + 35/4*cm3^2/cm2^3 -6*cm4/cm2^2 + cm6/cm2^3 +
    9/4*cm3^2*cm4/cm2^5 - 3*cm3*cm5/cm2^4
  skewness.ci.low <- skewness - 1.96*sqrt(var.skew/n)
  skewness.ci.upp <- skewness + 1.96*sqrt(var.skew/n)
  skewness.ci <- c("2.5%" = skewness.ci.low, "97.5%" = skewness.ci.upp)
  var0.skew <- 6
  skewness.pval <- 2*(1 - pnorm(sqrt(n/var0.skew)*abs(skewness)))

  combined.pval <- 1 - (1 - min(c(reg.pval, skewness.pval)))^2

  reg.int.star <- skewness.star <- numeric(n.resam)
  for(i in 1:n.resam){
    if(model == "FE"){
      y.star <- rnorm(n = n, mean = mu.est, sd = sqrt(s2))
      y.reg.star <- y.star/sqrt(s2)
      x.reg.star <- 1/sqrt(s2)
    }else{
      y.star <- rnorm(n = n, mean = mu.est, sd = sqrt(s2 + tau2))
      y.reg.star <- y.star/sqrt(s2 + tau2)
      x.reg.star <- 1/sqrt(s2 + tau2)
    }
    reg.star <- lm(y.reg.star ~ x.reg.star)
    reg.coef.star <- summary(reg.star)$coef
    reg.int.star[i] <- reg.coef.star["(Intercept)", "Estimate"]
    std.dev.star <- as.numeric(summary(reg.star)$residuals)
    cm2.star <- var(std.dev.star)
    cm3.star <- mean((std.dev.star - mean(std.dev.star))^3)
    cm4.star <- mean((std.dev.star - mean(std.dev.star))^4)
    cm5.star <- mean((std.dev.star - mean(std.dev.star))^5)
    cm6.star <- mean((std.dev.star - mean(std.dev.star))^6)
    skewness.star[i] <- cm3.star/(cm2.star^(1.5))
  }

  reg.pval.resam <- (sum(abs(reg.int.star) > abs(reg.int)) + 1)/(n.resam + 1)
  skewness.pval.resam <-
    (sum(abs(skewness.star) > abs(skewness)) + 1)/(n.resam + 1)
  combined.pval.resam <-
    1 - (1 - min(c(reg.pval.resam, skewness.pval.resam)))^2

  reg.int.replace <- skewness.replace <- numeric(n.resam)
  for(i in 1:n.resam){
    temp <- sample(n, replace = TRUE)
    y.replace <- y[temp]
    s2.replace <- s2[temp]
    w.replace <- 1/s2.replace
    mu.bar.replace <- sum(w.replace*y.replace)/sum(w.replace)
    Q.replace <- sum(w.replace*(y.replace - mu.bar.replace)^2)
    p.Q.replace <- 1 - pchisq(Q.replace, df = n - 1)
    tau2.replace <- (Q.replace - n + 1)/(sum(w.replace) - sum(w.replace^2)/sum(w.replace))
    tau2.replace <- max(c(0, tau2.replace))
    if(model == "FE"){
      y.reg.replace <- y.replace/sqrt(s2.replace)
      x.reg.replace <- 1/sqrt(s2.replace)
    }else{
      y.reg.replace <- y.replace/sqrt(s2.replace + tau2.replace)
      x.reg.replace <- 1/sqrt(s2.replace + tau2.replace)
    }
    reg.replace <- lm(y.reg.replace ~ x.reg.replace)
    reg.coef.replace <- summary(reg.replace)$coef
    reg.int.replace[i] <- reg.coef.replace["(Intercept)", "Estimate"]
    std.dev.replace <- as.numeric(summary(reg.replace)$residuals)
    cm2.replace <- var(std.dev.replace)
    cm3.replace <- mean((std.dev.replace - mean(std.dev.replace))^3)
    cm4.replace <- mean((std.dev.replace - mean(std.dev.replace))^4)
    cm5.replace <- mean((std.dev.replace - mean(std.dev.replace))^5)
    cm6.replace <- mean((std.dev.replace - mean(std.dev.replace))^6)
    skewness.replace[i] <- cm3.replace/(cm2.replace^(1.5))
  }
  reg.int.ci.resam <- quantile(reg.int.replace, probs = c(0.025, 0.975))
  skewness.ci.resam <- quantile(skewness.replace, probs = c(0.025, 0.975))

  out <- NULL
  out$n <- n
  out$p.Q <- p.Q
  out$I2 <- I2
  out$tau2 <- tau2
  out$model <- model
  out$std.dev <- std.dev
  out$reg.int <- reg.int
  out$reg.int.ci <- reg.int.ci
  out$reg.int.ci.resam <- reg.int.ci.resam
  out$reg.pval <- reg.pval
  out$reg.pval.resam <- reg.pval.resam
  out$skewness <- skewness
  out$skewness.ci <- skewness.ci
  out$skewness.ci.resam <- skewness.ci.resam
  out$skewness.pval <- skewness.pval
  out$skewness.pval.resam <- skewness.pval.resam
  out$combined.pval <- combined.pval
  out$combined.pval.resam <- combined.pval.resam
  return(out)
}

mvma <- function(ys, covs, data, method = "reml", tol = 1E-10){
  if(missing(ys)) stop("the argument ys is missing.")
  if(missing(covs)) stop("the argument covs is missing.")
  if(!missing(data)){
    ys <- eval(substitute(ys), data, parent.frame())
    covs <- eval(substitute(covs), data, parent.frame())
  }
  if(method != "fe" & method != "ml" & method != "reml") stop("method must be fe, ml, or reml.")
  if(method == "fe") out <- mvma.fe(ys = ys, covs = covs, tol = tol)
  if(method != "fe") out <- mvma.re(ys = ys, covs = covs, method = method, tol = tol)
  return(out)
}

ssfunnel <- function(y, s2, n, data, type, alpha = c(0.1, 0.05, 0.01, 0.001), log.ss = FALSE, sigma, p0, xlim, ylim, xlab, ylab,
  cols.contour, col.mostsig, cex.pts, lwd.contour, pch, x.legend, y.legend, cex.legend, bg.legend, ...){
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if(missing(y)) stop("the observed effect sizes are required.")
  if(missing(s2)) stop("the within-study variances are required.")
  if(missing(n)) stop("the sample sizes are required.")
  if(missing(type)) stop("the data type needs to be specified.")
  if(!is.element(type, c("md", "smd", "lor", "lrr", "rd"))) stop("the data type can be MD, SMD, LOR, LRR, or RD.")
  if(!is.null(alpha)){
    alpha <- sort(alpha, decreasing = TRUE)
    if(all(alpha != 0)) alpha <- c(alpha, 0)
  }
  if(any(alpha < 0) | any(alpha > 1)) stop("the significance levels should be between 0 and 1.")
  if(missing(p0)){
    p0 <- NA

  }else{
    if(p0 < 0 | p0 > 1) stop("p0 must be between 0 and 1.")
  }
  if(missing(sigma) & type == "md") stop("the standard deviation sigma must be specified for mean difference.")
  if(missing(xlim)) xlim <- c(-max(abs(y)), max(abs(y)))
  if(missing(ylim)) ylim <- c(1, max(n))

  if(missing(xlab)){
    if(type == "md") xlab <- "Mean difference"
    if(type == "smd") xlab <- "Standardized mean difference"
    if(type == "lor") xlab <- "Log odds ratio"
    if(type == "lrr") xlab <- "Log relative risk"
    if(type == "rd") xlab <- "Risk difference"
  }
  if(missing(ylab)) ylab <- "Sample size"

  pval <- 2*pnorm(-abs(y)/sqrt(s2))
  cols <- rep(NA, length(y))
  if(missing(cols.contour)){
    if(length(alpha) > 0){
      cols.contour <- rep(NA, length(alpha))
      cols[pval >= alpha[1]] <- cols.contour[1] <- "gray80"
    }else{
      cols <- rep("black", length(y))
    }
    if(length(alpha) > 1) cols[pval < alpha[1] & pval >= alpha[2]] <- cols.contour[2] <- "gray50"
    if(length(alpha) > 2){
      for(i in 3:length(alpha)){
        cols[pval < alpha[i - 1] & pval >= alpha[i]] <- cols.contour[i] <- paste0("gray", 10*max(c(0, 5 - (i - 2))))
      }
    }
  }else{
    if(missing(col.mostsig)) col.mostsig <- "gray20"
    cols.contour <- c(cols.contour, col.mostsig)
    if(length(alpha) != length(cols.contour)) stop("the length of cols.contour is not equal to the length of alpha.")
    if(length(alpha) > 0){
      cols[pval >= alpha[1]] <- cols.contour[1]
    }else{
      cols <- rep("black", length(y))
    }
    if(length(alpha) > 1) cols[pval < alpha[1] & pval >= alpha[2]] <- cols.contour[2]
    if(length(alpha) > 2){
      for(i in 3:length(alpha)){
        cols[pval < alpha[i - 1] & pval >= alpha[i]] <- cols.contour[i]
      }
    }
  }
  cols.contour <- cols.contour[-length(cols.contour)]
  alpha <- alpha[-length(alpha)]

  if(missing(cex.pts)) cex.pts <- 1
  if(missing(lwd.contour)) lwd.contour <- 1
  if(missing(pch)) pch <- 19

  if(type == "md") ssfunnel.md(y, s2, n, alpha, log.ss, sigma, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...)
  if(type == "smd") ssfunnel.smd(y, s2, n, alpha, log.ss, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...)
  if(type == "lor") ssfunnel.lor(y, s2, n, alpha, log.ss, p0, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...)
  if(type == "lrr") ssfunnel.lrr(y, s2, n, alpha, log.ss, p0, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...)
  if(type == "rd") ssfunnel.rd(y, s2, n, alpha, log.ss, p0, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...)

  if(missing(x.legend)) x.legend <- "topleft"
  if(missing(y.legend)) y.legned <- NULL
  if(missing(cex.legend)) cex.legend <- 1
  if(missing(bg.legend)) bg.legend <- "white"

  alpha.legend <- alpha
  legend(x = x.legend, y = y.legend, ncol = 1,
    legend = sapply(1:length(alpha), function(i) as.expression(substitute(alpha == A, list(A = alpha.legend[i])))),
    col = cols.contour, lwd = lwd.contour, title = NULL, cex = cex.legend, bg = bg.legend)
}

pb.inv.n <- function(y, s2 = NA, n = NA, n00, n01, n10, n11){
  n <- n00 + n01 + n10 + n11
  nSum0 <- n00 + n10
  nSum1 <- n01 + n11
  x.reg <- 1/n
  y.reg <- y
  w.reg <- nSum0*nSum1/n
  if(all(abs(diff(x.reg[!is.na(x.reg)])) < 1e-5) | all(abs(diff(y.reg[!is.na(y.reg)])) < 1e-5)){
    out <- list(pval = 1, stat = 0, coef = c(0, 0))
  }else{
    out <- lm(y.reg ~ x.reg, weights = w.reg)
    out <- summary(out)$coefficients
    pval <- out["x.reg", "Pr(>|t|)"]
    coef <- as.numeric(out[,1])
    out <- list(pval = pval, stat = as.numeric(out["x.reg", "t value"]), coef = coef)
  }
  return(out)
}

pb.hybrid.generic <- function(y, s2, n, data, methods, iter.resam = 1000, theo.pval = TRUE){
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
    if(!missing(n)){
      n <- eval(substitute(n), data, parent.frame())
    }
  }
  if(missing(methods)){
    methods <- c("rank", "reg", "reg.het", "skew", "skew.het", "inv.sqrt.n", "trimfill")
  }
  if(!all(is.element(methods, c("rank", "reg", "reg.het", "skew", "skew.het", "inv.sqrt.n", "trimfill")))){
    stop("incorrect input for methods.")
  }
  if(missing(n) & is.element("inv.sqrt.n", methods)){
    stop("n must be specified if 'inv.sqrt.n' is included.")
  }

  pval.rank <- pval.reg <- pval.reg.het <- pval.skew <- pval.skew.het <- pval.inv.sqrt.n <- pval.trimfill <- NA
  pval.rank.theo <- pval.reg.theo <- pval.reg.het.theo <- pval.skew.theo <- pval.skew.het.theo <- pval.inv.sqrt.n.theo <- pval.trimfill.theo <- NA
  if(is.element("rank", methods)){
    rank <- pb.rank(y, s2)
    stat.rank <- rank$stat
    if(theo.pval) pval.rank.theo <- rank$pval
  }
  if(is.element("reg", methods)){
    reg <- pb.reg(y, s2)
    stat.reg <- reg$stat
    if(theo.pval) pval.reg.theo <- reg$pval
  }
  if(is.element("reg.het", methods)){
    reg.het <- pb.reg.het(y, s2)
    stat.reg.het <- reg.het$stat
    if(theo.pval) pval.reg.het.theo <- reg.het$pval
  }
  if(is.element("skew", methods)){
    skew <- pb.skew(y, s2)
    stat.skew <- skew$stat
    if(theo.pval) pval.skew.theo <- skew$pval
  }
  if(is.element("skew.het", methods)){
    skew.het <- pb.skew.het(y, s2)
    stat.skew.het <- skew.het$stat
    if(theo.pval) pval.skew.het.theo <- skew.het$pval
  }
  if(is.element("inv.sqrt.n", methods)){
    inv.sqrt.n <- pb.inv.sqrt.n(y, s2, n)
    stat.inv.sqrt.n <- inv.sqrt.n$stat
    if(theo.pval) pval.inv.sqrt.n.theo <- inv.sqrt.n$pval
  }
  if(is.element("trimfill", methods)){
    options(warn = -1)
    rma <- rma(yi = y, vi = s2, method = "DL")
    trimfill <- pb.trimfill(rma, estimator = "R0")
    stat.trimfill <- trimfill$k0
    if(theo.pval) pval.trimfill.theo <- trimfill$pval
    options(warn = 0)
  }

  N <- length(y)
  w <- 1/s2
  theta.hat <- sum(w*y)/sum(w)
  Q <- sum(w*(y - theta.hat)^2)
  tau2.hat <- (Q - N + 1)/(sum(w) - sum(w^2)/sum(w))
  tau2.hat <- max(c(0, tau2.hat))
  w <- 1/(s2 + tau2.hat)
  theta.hat <- sum(w*y)/sum(w)
  if(all(is.na(n))) n <- rep(NA, N)

  stat.rank.resam <- stat.reg.resam <- stat.reg.het.resam <- stat.skew.resam <- stat.skew.het.resam <-
    stat.inv.sqrt.n.resam <- stat.trimfill.resam <- stat.hybrid.resam <- stat.hybrid.theo.resam <- rep(NA, iter.resam)
  pval.rank.theo.resam <- pval.reg.theo.resam <- pval.reg.het.theo.resam <- pval.skew.theo.resam <- pval.skew.het.theo.resam <-
    pval.inv.sqrt.n.theo.resam <- pval.trimfill.theo.resam <- rep(NA, iter.resam)
  for(i in 1:iter.resam){
    idx <- sample(1:N, replace = TRUE)
    s2.resam <- s2[idx]
    n.resam <- n[idx]
    y.resam <- rnorm(n = N, mean = theta.hat, sd = sqrt(s2.resam + tau2.hat))
    if(is.element("rank", methods)){
      rank.resam <- pb.rank(y.resam, s2.resam)
      stat.rank.resam[i] <- rank.resam$stat
      if(theo.pval) pval.rank.theo.resam[i] <- rank.resam$pval
    }
    if(is.element("reg", methods)){
      reg.resam <- pb.reg(y.resam, s2.resam)
      stat.reg.resam[i] <- reg.resam$stat
      if(theo.pval) pval.reg.theo.resam[i] <- reg.resam$pval
    }
    if(is.element("reg.het", methods)){
      reg.het.resam <- pb.reg.het(y.resam, s2.resam)
      stat.reg.het.resam[i] <- reg.het.resam$stat
      if(theo.pval) pval.reg.het.theo.resam[i] <- reg.het.resam$pval
    }
    if(is.element("skew", methods)){
      skew.resam <- pb.skew(y.resam, s2.resam)
      stat.skew.resam[i] <- skew.resam$stat
      if(theo.pval) pval.skew.theo.resam[i] <- skew.resam$pval
    }
    if(is.element("skew.het", methods)){
      skew.het.resam <- pb.skew.het(y.resam, s2.resam)
      stat.skew.het.resam[i] <- skew.het.resam$stat
      if(theo.pval) pval.skew.het.theo.resam[i] <- skew.het.resam$pval
    }
    if(is.element("inv.sqrt.n", methods)){
      inv.sqrt.n.resam <- pb.inv.sqrt.n(y.resam, s2.resam, n.resam)
      stat.inv.sqrt.n.resam[i] <- inv.sqrt.n.resam$stat
      if(theo.pval) pval.inv.sqrt.n.theo.resam[i] <- inv.sqrt.n.resam$pval
    }
    if(is.element("trimfill", methods)){
      options(warn = -1)
      rma.resam <- rma(yi = y.resam, vi = s2.resam, method = "DL")
      trimfill.resam <- pb.trimfill(rma.resam, estimator = "R0")
      stat.trimfill.resam[i] <- trimfill.resam$k0
      if(theo.pval) pval.trimfill.theo.resam[i] <- trimfill.resam$pval
      options(warn = 0)
    }
  }

  if(is.element("rank", methods)) pval.rank <- (sum(abs(stat.rank.resam) >= abs(stat.rank)) + 1)/(iter.resam + 1)
  if(is.element("reg", methods)) pval.reg <- (sum(abs(stat.reg.resam) >= abs(stat.reg)) + 1)/(iter.resam + 1)
  if(is.element("reg.het", methods)) pval.reg.het <- (sum(abs(stat.reg.het.resam) >= abs(stat.reg.het)) + 1)/(iter.resam + 1)
  if(is.element("skew", methods)) pval.skew <- (sum(abs(stat.skew.resam) >= abs(stat.skew)) + 1)/(iter.resam + 1)
  if(is.element("skew.het", methods)) pval.skew.het <- (sum(abs(stat.skew.het.resam) >= abs(stat.skew.het)) + 1)/(iter.resam + 1)
  if(is.element("inv.sqrt.n", methods)) pval.inv.sqrt.n <- (sum(abs(stat.inv.sqrt.n.resam) >= abs(stat.inv.sqrt.n)) + 1)/(iter.resam + 1)
  if(is.element("trimfill", methods)) pval.trimfill <- (sum(abs(stat.trimfill.resam) >= abs(stat.trimfill)) + 1)/(iter.resam + 1)

  if(length(methods) == 1){
    if(!theo.pval) out <- get(paste0("pval.", methods))
    if(theo.pval){
      out <- list(get(paste0("pval.", methods)), get(paste0("pval.", methods, ".theo")))
      names(out) <- c(paste0("pval.", methods), paste0("pval.", methods, ".theo"))
    }
  }

  if(length(methods) > 1){
    stat.hybrid <- min(c(pval.rank, pval.reg, pval.reg.het, pval.skew, pval.skew.het, pval.inv.sqrt.n, pval.trimfill), na.rm = TRUE)
    if(theo.pval) stat.hybrid.theo <- min(c(pval.rank.theo, pval.reg.theo, pval.reg.het.theo, pval.skew.theo, pval.skew.het.theo, pval.inv.sqrt.n.theo, pval.trimfill.theo), na.rm = TRUE)
    pval.rank.resam <- pval.reg.resam <- pval.reg.het.resam <- pval.skew.resam <- pval.skew.het.resam <-
      pval.inv.sqrt.n.resam <- pval.trimfill.resam <- rep(NA, iter.resam)
    for(i in 1:iter.resam){
      if(theo.pval) stat.hybrid.theo.resam[i] <- min(c(pval.rank.theo.resam[i], pval.reg.theo.resam[i], pval.reg.het.theo.resam[i], pval.skew.theo.resam[i], pval.skew.het.theo.resam[i], pval.inv.sqrt.n.theo.resam[i], pval.trimfill.theo.resam[i]), na.rm = TRUE)
      if(is.element("rank", methods)) pval.rank.resam[i] <- (sum(abs(stat.rank.resam[-i]) >= abs(stat.rank.resam[i])) + 1)/iter.resam
      if(is.element("reg", methods)) pval.reg.resam[i] <- (sum(abs(stat.reg.resam[-i]) >= abs(stat.reg.resam[i])) + 1)/iter.resam
      if(is.element("reg.het", methods)) pval.reg.het.resam[i] <- (sum(abs(stat.reg.het.resam[-i]) >= abs(stat.reg.het.resam[i])) + 1)/iter.resam
      if(is.element("skew", methods)) pval.skew.resam[i] <- (sum(abs(stat.skew.resam[-i]) >= abs(stat.skew.resam[i])) + 1)/iter.resam
      if(is.element("skew.het", methods)) pval.skew.het.resam[i] <- (sum(abs(stat.skew.het.resam[-i]) >= abs(stat.skew.het.resam[i])) + 1)/iter.resam
      if(is.element("inv.sqrt.n", methods)) pval.inv.sqrt.n.resam[i] <- (sum(abs(stat.inv.sqrt.n.resam[-i]) >= abs(stat.inv.sqrt.n.resam[i])) + 1)/iter.resam
      if(is.element("trimfill", methods)) pval.trimfill.resam[i] <- (sum(abs(stat.trimfill.resam[-i]) >= abs(stat.trimfill.resam[i])) + 1)/iter.resam
      stat.hybrid.resam[i] <- min(c(pval.rank.resam[i], pval.reg.resam[i], pval.reg.het.resam[i], pval.skew.resam[i], pval.skew.het.resam[i], pval.inv.sqrt.n.resam[i], pval.trimfill.resam[i]), na.rm = TRUE)
    }
    pval.hybrid <- (sum(stat.hybrid.resam <= stat.hybrid) + 1)/(iter.resam + 1)
    if(!theo.pval){
      out <- list(pval.rank = pval.rank, pval.reg = pval.reg, pval.reg.het = pval.reg.het, pval.skew = pval.skew, pval.skew.het = pval.skew.het,
        pval.inv.sqrt.n = pval.inv.sqrt.n, pval.trimfill = pval.trimfill, pval.hybrid = pval.hybrid)
    }
    if(theo.pval){
      pval.hybrid.theo <- (sum(stat.hybrid.theo.resam <= stat.hybrid.theo) + 1)/(iter.resam + 1)
      out <- list(pval.rank = pval.rank, pval.rank.theo = pval.rank.theo, pval.reg = pval.reg, pval.reg.theo = pval.reg.theo, pval.reg.het = pval.reg.het, pval.reg.het.theo = pval.reg.het.theo,
        pval.skew = pval.skew, pval.skew.theo = pval.skew.theo, pval.skew.het = pval.skew.het, pval.skew.het.theo = pval.skew.het.theo, pval.inv.sqrt.n = pval.inv.sqrt.n, pval.inv.sqrt.n.theo = pval.inv.sqrt.n.theo,
        pval.trimfill = pval.trimfill, pval.trimfill.theo = pval.trimfill.theo, pval.hybrid = pval.hybrid, pval.hybrid.theo = pval.hybrid.theo)
    }
    na.test <- which(is.na(out))
    if(length(na.test) > 0) out <- out[-na.test]
  }

  return(out)
}

check.counts <- function(n00, n01, n10, n11){
  idx.double.zero <- which(n00 + n10 == 0 | n01 + n11 == 0)
  if(length(idx.double.zero) > 0){
    if(length(n00) - length(idx.double.zero) >= 5){
      n00 <- n00[-idx.double.zero]
      n01 <- n01[-idx.double.zero]
      n10 <- n10[-idx.double.zero]
      n11 <- n11[-idx.double.zero]
    }else{
      # avoid removing too many double-zero studies
      n00[idx.double.zero] <- n00[idx.double.zero] + 0.01
      n01[idx.double.zero] <- n01[idx.double.zero] + 0.01
      n10[idx.double.zero] <- n10[idx.double.zero] + 0.01
      n11[idx.double.zero] <- n11[idx.double.zero] + 0.01
    }
  }

  idx.zero <- which(n00 == 0 | n01 == 0 | n10 == 0 | n11 == 0)
  if(length(idx.zero) > 0){
    n00[idx.zero] <- n00[idx.zero] + 0.5
    n01[idx.zero] <- n01[idx.zero] + 0.5
    n10[idx.zero] <- n10[idx.zero] + 0.5
    n11[idx.zero] <- n11[idx.zero] + 0.5
  }
  out <- list(n00 = n00, n01 = n01, n10 = n10, n11 = n11)
  return(out)
}

metahet <- function(y, s2, data, n.resam = 1000){
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
  }
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  n <- length(y)

  out0 <- metahet.base(y = y, s2 = s2)

  out.resam.param <- lapply(1:n.resam, function(o){
    y.temp <- rnorm(n, mean = out0$weighted.mean, sd = sqrt(s2))
    out.temp <- metahet.base(y.temp, s2)
    return(out.temp)
  })
  out.resam.param <- unlist(out.resam.param)

  out.resam.replace <- lapply(1:n.resam, function(o){
    temp <- sample(n, replace = TRUE)
    y.temp <- y[temp]
    s2.temp <- s2[temp]
    out.temp <- metahet.base(y.temp, s2.temp)
    return(out.temp)
  })
  out.resam.replace <- unlist(out.resam.replace)

  # the conventional methods
  Q <- out0$Q
  Q.resam.param <- out.resam.param[names(out.resam.param) == "Q"]
  names(Q.resam.param) <- NULL
  p.Q <- (sum(Q.resam.param > Q) + 1)/(n.resam + 1)
  p.Q.theo <- 1 - pchisq(Q, n - 1)

  Q.resam.replace <- out.resam.replace[names(out.resam.replace) == "Q"]
  names(Q.resam.replace) <- NULL
  ci.Q <- quantile(Q.resam.replace, probs = c(0.025, 0.975))

  tau2.DL <- out0$tau2.DL
  tau2.resam.replace <- out.resam.replace[names(out.resam.replace) == "tau2.DL"]
  names(tau2.resam.replace) <- NULL
  ci.tau2.DL <- quantile(tau2.resam.replace, probs = c(0.025, 0.975))

  H <- out0$H
  H.resam.replace <- out.resam.replace[names(out.resam.replace) == "H"]
  names(H.resam.replace) <- NULL
  ci.H <- quantile(H.resam.replace, probs = c(0.025, 0.975))

  I2 <- out0$I2
  I2.resam.replace <- out.resam.replace[names(out.resam.replace) == "I2"]
  names(I2.resam.replace) <- NULL
  ci.I2 <- quantile(I2.resam.replace, probs = c(0.025, 0.975))

  # absolute deviation based on weighted mean
  Qr <- out0$Qr
  Qr.resam.param <- out.resam.param[names(out.resam.param) == "Qr"]
  names(Qr.resam.param) <- NULL
  p.Qr <- (sum(Qr.resam.param > Qr) + 1)/(n.resam + 1)

  Qr.resam.replace <- out.resam.replace[names(out.resam.replace) == "Qr"]
  names(Qr.resam.replace) <- NULL
  ci.Qr <- quantile(Qr.resam.replace, probs = c(0.025, 0.975))

  tau2.r <- out0$tau2.r
  tau2.r.resam.replace <- out.resam.replace[names(out.resam.replace) == "tau2.r"]
  names(tau2.r.resam.replace) <- NULL
  ci.tau2.r <- quantile(tau2.r.resam.replace, probs = c(0.025, 0.975))

  Hr <- out0$Hr
  Hr.resam.replace <- out.resam.replace[names(out.resam.replace) == "Hr"]
  names(Hr.resam.replace) <- NULL
  ci.Hr <- quantile(Hr.resam.replace, probs = c(0.025, 0.975))

  Ir2 <- out0$Ir2
  Ir2.resam.replace <- out.resam.replace[names(out.resam.replace) == "Ir2"]
  names(Ir2.resam.replace) <- NULL
  ci.Ir2 <- quantile(Ir2.resam.replace, probs = c(0.025, 0.975))

  # absolute deviation based on weighted median
  Qm <- out0$Qm
  Qm.resam.param <- out.resam.param[names(out.resam.param) == "Qm"]
  names(Qm.resam.param) <- NULL
  p.Qm <- (sum(Qm.resam.param > Qm) + 1)/(n.resam + 1)

  Qm.resam.replace <- out.resam.replace[names(out.resam.replace) == "Qm"]
  names(Qm.resam.replace) <- NULL
  ci.Qm <- quantile(Qm.resam.replace, probs = c(0.025, 0.975))

  tau2.m <- out0$tau2.m
  tau2.m.resam.replace <- out.resam.replace[names(out.resam.replace) == "tau2.m"]
  names(tau2.m.resam.replace) <- NULL
  ci.tau2.m <- quantile(tau2.m.resam.replace, probs = c(0.025, 0.975))

  Hm <- out0$Hm
  Hm.resam.replace <- out.resam.replace[names(out.resam.replace) == "Hm"]
  names(Hm.resam.replace) <- NULL
  ci.Hm <- quantile(Hm.resam.replace, probs = c(0.025, 0.975))

  Im2 <- out0$Im2
  Im2.resam.replace <- out.resam.replace[names(out.resam.replace) == "Im2"]
  names(Im2.resam.replace) <- NULL
  ci.Im2 <- quantile(Im2.resam.replace, probs = c(0.025, 0.975))

  # output
  out <- NULL

  out$p.Q <- p.Q
  out$p.Q.theo <- p.Q.theo
  out$p.Qr <- p.Qr
  out$p.Qm <- p.Qm

  out$Q <- Q
  out$ci.Q <- ci.Q
  out$tau2.DL <- tau2.DL
  out$ci.tau2.DL <- ci.tau2.DL
  out$H <- H
  out$ci.H <- ci.H
  out$I2 <- I2
  out$ci.I2 <- ci.I2

  out$Qr <- Qr
  out$ci.Qr <- ci.Qr
  out$tau2.r <- tau2.r
  out$ci.tau2.r <- ci.tau2.r
  out$Hr <- Hr
  out$ci.Hr <- ci.Hr
  out$Ir2 <- Ir2
  out$ci.Ir2 <- ci.Ir2

  out$Qm <- Qm
  out$ci.Qm <- ci.Qm
  out$tau2.m <- tau2.m
  out$ci.tau2.m <- ci.tau2.m
  out$Hm <- Hm
  out$ci.Hm <- ci.Hm
  out$Im2 <- Im2
  out$ci.Im2 <- ci.Im2

  return(out)
}

ssfunnel.lor <- function(y, s2, n, alpha, log.ss, p0, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...){
  plot.default(y, n, xlim = xlim, ylim = ylim,
    type = "n", log = ifelse(log.ss, "y", ""),
    xlab = xlab, ylab = ylab, ...)
  points(y, n, pch = pch, col = cols, cex = cex.pts)
  if(length(alpha) > 0){
    for(i in 1:length(alpha)){
      contour <- function(x){
        if(is.na(p0)){
          qnorm(alpha[i]/2)^2*4*(2 + exp(-x/2) + exp(x/2))/x^2
        }else{
          qnorm(alpha[i]/2)^2*(1 + (1 - p0)*exp(-x/2) + p0*exp(x/2))^2/(x^2*p0*(1 - p0))
        }
      }
      contour <- Vectorize(contour)
      curve(contour, from = min(c(-max(abs(y)), xlim[1])) - 1, to = max(c(max(abs(y)), xlim[2])) + 1, add = TRUE, col = cols.contour[i], lwd = lwd.contour)
    }
  }
}

pb.count <- function(y = NA, s2 = NA, n = NA, n00, n01, n10, n11){
  n <- n00 + n01 + n10 + n11
  n1Sum <- n10 + n11
  nSum1 <- n01 + n11
  or.mh <- sum(n00/n*n11)/sum(n01/n*n10)
  N <- length(n00)
  n11.e <- n11.v <- rep(NA, N)
  for(i in 1:N){
    obj <- hypergeometric(n1Sum[i], nSum1[i], n[i], or.mh)
    n11.e[i] <- obj$mean()
    n11.v[i] <- obj$var()
  }
  xx <- (n11 - n11.e)/sqrt(n11.v)
  yy <- 1/n11.v
  if(all(abs(diff(xx[!is.na(xx)])) < 1e-5) | all(abs(diff(yy[!is.na(yy)])) < 1e-5)){
    out <- list(pval = 1, stat = 0)
  }else{
    out <- cor.test(x = xx, y = yy, alternative = "two.sided", method = "kendall", exact = FALSE)
    out <- list(pval = out$p.value, stat = as.numeric(out$statistic))
  }
  return(out)
}

pb.bayesian.binary <- function(n00, n01, n10, n11, p01 = NULL, p11 = NULL, data, sig.level = 0.1, 
                               method = "bay", het = "mul", sd.prior = "unif", n.adapt = 1000, 
                               n.chains = 3, n.burnin = 5000, n.iter = 10000, thin = 2, upp.het = 2, 
                               phi = 0.5, coda = FALSE, traceplot = FALSE, seed = 1234){
  if(method != "bay" & method != "reg.bay" & method != "smoothed.bay") 
    stop("method must be bay, reg.bay or smoothed.bay")
  if(missing(n01) | missing(n11)) stop("the counts of events are required.")
  if(missing(n00) | missing(n10)) stop("the counts of non-events are required.")
  if(!missing(data)){
    n00 <- eval(substitute(n00), data, parent.frame())
    n01 <- eval(substitute(n01), data, parent.frame())
    n10 <- eval(substitute(n10), data, parent.frame())
    n11 <- eval(substitute(n11), data, parent.frame())
    if(!is.null(p01)) p01 <- eval(substitute(p01), data, parent.frame())
    if(!is.null(p11)) p01 <- eval(substitute(p11), data, parent.frame())
  }
  if(!is.null(p01) & !is.null(p11)){
    if(p01 > 1 | p11 > 1 | any(n00 < 0) | any(n01 < 0) | any(n10 < 0) | any(n11 < 0)) 
      stop("the event rates are invalid.")}
  
  n0 <- n00 + n01
  n1 <- n10 + n11
  n0_star <- n0
  n1_star <- n1
  n00_star <- n00
  n01_star <- n01
  n10_star <- n10
  n11_star <- n11
  identify.zero <- which(n00 == 0 | n01 == 0 | n10 == 0 | n11 == 0)
  if (length(identify.zero)>0){
    n00_star[identify.zero] <- n00_star[identify.zero] + 0.5
    n01_star[identify.zero] <- n01_star[identify.zero] + 0.5
    n10_star[identify.zero] <- n10_star[identify.zero] + 0.5
    n11_star[identify.zero] <- n11_star[identify.zero] + 0.5
    n0_star[identify.zero] <- n0_star[identify.zero] + 1
    n1_star[identify.zero] <- n1_star[identify.zero] + 1
    y <- log(n11_star/n10_star) - log(n01_star/n00_star)
    s2 <- 1/n11_star + 1/n10_star + 1/n01_star + 1/n00_star
    p01.sm <- mean(n01_star/n0_star)
    p11.sm <- mean(n11_star/n1_star)
  }else{
    y <- log(n11/n10) - log(n01/n00)
    s2 <- 1/n11 + 1/n10 + 1/n01 + 1/n00
    p01.sm <- mean(n01/n0)
    p11.sm <- mean(n11/n1)
  }
  s2.smoothed <- 1/(n0*p01.sm*(1 - p01.sm)) + 1/(n1*p11.sm*(1 - p11.sm))
  
  out <- NULL
  if(is.element("bay", method)){
    set.seed(seed)
    out <- pb.bay(y = y, s2 = s2, sig.level = sig.level, n00 = n00, n01 = n01, n10 = n10, n11 = n11, 
                      het = het, sd.prior = sd.prior, n.adapt = n.adapt, n.chains = n.chains, 
                      n.burnin = n.burnin, n.iter = n.iter, thin = thin, upp.het = upp.het, 
                      phi = phi, coda = coda, traceplot = traceplot)
  }
  
  if(is.element("reg.bay", method)){
    if(!is.null(p01) & !is.null(p11)){
      s2.true <- 1/(n0*p01*(1 - p01)) + 1/(n1*p11*(1 - p11))
      set.seed(seed)
      out <- pb.reg.bay(y = y, s2 = s2.true, sig.level = sig.level, n00 = n00, n01 = n01, n10 = n10, n11 = n11, 
                        het = het, sd.prior = sd.prior, n.adapt = n.adapt, n.chains = n.chains, 
                        n.burnin = n.burnin, n.iter = n.iter, thin = thin, upp.het = upp.het, 
                        phi = phi, coda = coda, traceplot = traceplot)
    }else{
      set.seed(seed)
      out <- pb.reg.bay(y = y, s2 = s2, sig.level = sig.level, n00 = n00, n01 = n01, n10 = n10, n11 = n11, 
                        het = het, sd.prior = sd.prior, n.adapt = n.adapt, n.chains = n.chains, 
                        n.burnin = n.burnin, n.iter = n.iter, thin = thin, upp.het = upp.het, 
                        phi = phi, coda = coda, traceplot = traceplot)
    }
  }
  
  if(is.element("smoothed.bay", method)){
    set.seed(seed)
    out.temp <- pb.reg.bay(y = y, s2 = s2.smoothed, sig.level = sig.level, n00 = n00, n01 = n01, n10 = n10, n11 = n11, 
                           het = het, sd.prior = sd.prior, n.adapt = n.adapt, n.chains = n.chains, 
                           n.burnin = n.burnin, n.iter = n.iter, thin = thin, upp.het = upp.het, 
                           phi = phi, coda = coda, traceplot = traceplot)
    out$est.smoothed.bay <- out.temp$est.reg.bay
    out$ci.smoothed.bay <- out.temp$ci.reg.bay
    out$samps.smoothed.bay <- out.temp$samp.reg.bay
  }
  
  return(out)
}

metahet.base <- function(y, s2){
	if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
	n <- length(y)

	w <- 1/s2
	mu.bar <- sum(w*y)/sum(w)

	out <- NULL

	out$weighted.mean <- mu.bar

	# the conventional methods
	Q <- sum(w*(y - mu.bar)^2)
	H <- sqrt(Q/(n - 1))
	I2 <- (Q - n + 1)/Q
	tau2.DL <- (Q - n + 1)/(sum(w) - sum(w^2)/sum(w))
	tau2.DL <- max(c(0, tau2.DL))
	out$Q <- Q
	out$H <- H
	out$I2 <- I2
	out$tau2.DL <- tau2.DL

	# absolute deviation based on weighted mean
	Qr <- sum(sqrt(w)*abs(y - mu.bar))
	Hr <- sqrt((3.14159*Qr^2)/(2*n*(n - 1)))
	Ir2 <- (Qr^2 - 2*n*(n - 1)/3.14159)/(Qr^2)
	tau2.r <- tau2.r.solver(w, Qr)
	out$Qr <- Qr
	out$Hr <- Hr
	out$Ir2 <- Ir2
	out$tau2.r <- tau2.r

	# absolute deviation based on weighted median

	expit <- function(x) {ifelse(x >= 0, 1/(1 + exp(-x/0.0001)), exp(x/0.0001)/(1 + exp(x/0.0001)))}
	psi <- function(x) {sum(w*(expit(x - y) - 0.5))}
	mu.med <- uniroot(psi, c(min(y) - 0.001, max(y) + 0.001))$root
	out$weighted.median <- mu.med
	Qm <- sum(sqrt(w)*abs(y - mu.med))
	Hm <- sqrt(3.14159/2)*Qm/n
	Im2 <- (Qm^2 - 2*n^2/3.14159)/Qm^2
	tau2.m <- tau2.m.solver(w, Qm)
	out$Qm <- Qm
	out$Hm <- Hm
	out$Im2 <- Im2
	out$tau2.m <- tau2.m

	return(out)
}

## Calculating ICDF: la, Lu--Ades; fe, fixed-effects; re, random-effects
nma.icdf <- function(sid, tid, r, n, data, type = c("la", "fe", "re"),
  n.adapt = 1000, n.chains = 3, n.burnin = 5000, n.iter = 20000, n.thin = 2,
  traceplot = FALSE, nma.name = NULL, seed = 1234){
  if(missing(sid)) stop("please specify study ID.")
  if(missing(tid)) stop("please specify treatment ID.")
  if(missing(r)) stop("please specify event count.")
  if(missing(n)) stop("please specify sample size.")
  if(!missing(data)){
    sid <- eval(substitute(sid), data, parent.frame())
    tid <- eval(substitute(tid), data, parent.frame())
    r <- eval(substitute(r), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if(all(!is.element(type, c("la", "fe", "re")))) stop("the type of ICDF is wrongly specified.")
  if(any(!is.element(type, c("la", "fe", "re")))){
    type <- type[is.element(type, c("la", "fe", "re"))]
  }

  n.trt <- length(unique(data$tid))
  study.idx <- unique(data$sid)
  n.study <- length(study.idx)
  out <- NULL
  out$nstudy.trtarm <- table(table(data$sid))
  out$nstudy.multi <- sum(table(data$sid) > 2)
  multi.trtarm <- NULL
  multi.idx <- which(table(data$sid) > 2)
  if(out$nstudy.multi > 0){
    for(i in 1:out$nstudy.multi){
      multi.trtarm[[i]] <- sort(data$tid[data$sid == multi.idx[i]])
    }
  }
  if(is.null(multi.trtarm)){
    multi.trtarm <- NA
  }
  out$multi.trtarm <- multi.trtarm

  if(is.element("la", type)){
    if(out$nstudy.multi > 0){
      out$icdf.la <- NA
    }else{
      T <- 0
      for(h in 1:(n.trt - 1)){
        for(k in (h + 1):n.trt){
          T.temp <- 0
          for(i in 1:n.study){
            dat.temp <- data[data$sid == i,]
            if(dim(dat.temp)[1] == 2){
              trt1.temp <- min(dat.temp$tid)
              trt2.temp <- max(dat.temp$tid)
              if(trt1.temp == h & trt2.temp == k){
                T.temp <- 1
              }
            }
          }
          if(T.temp == 1) T <- T + 1
        }
      }
      out$icdf.la <- T - n.trt + 1
    }
  }

  out <- c(out, list(icdf.fe = NA, icdf.re = NA))

  if(is.element("fe", type) | is.element("re", type)){
    na <- as.numeric(table(data$sid))
    t <- r <- n <- matrix(NA, n.study, max(na))
    for(i in 1:n.study){
      dat.temp <- data[data$sid == study.idx[i],]
      ord <- order(dat.temp$tid)
      dat.temp <- dat.temp[ord,]
      t[i, 1:na[i]] <- dat.temp$tid
      r[i, 1:na[i]] <- dat.temp$r
      n[i, 1:na[i]] <- dat.temp$n
    }
    dat.jags <- list(NT = n.trt, NS = n.study, r = r, n = n, t = t, na = na)
  }

  if(is.element("fe", type)){
    set.seed(seed)
    inits.fe <- NULL
    for(i in 1:n.chains){
      inits.fe[[i]] <- list(mu = rep(rnorm(1), n.study),
        .RNG.name = "base::Wichmann-Hill", .RNG.seed = sample(123456789, 1))
    }
    params.fe <- c("lor", "totresdev")

    # FE NMA with evidence consistency
    set.seed(seed)
    jags.nma.fe.c <- jags.model(file = textConnection(nma.fe.c()),
      data = dat.jags, inits = inits.fe, n.chains = n.chains, n.adapt = n.adapt)
    update(jags.nma.fe.c, n.iter = n.burnin)
    coda.nma.fe.c <- coda.samples(model = jags.nma.fe.c,
      variable.names = params.fe, n.iter = n.iter, thin = n.thin)
    smry.nma.fe.c <- summary(coda.nma.fe.c)
    out.nma.fe.c <- smry.nma.fe.c$quantiles[grep("lor",
      rownames(smry.nma.fe.c$quantiles)), c("2.5%", "50%", "97.5%")]
    pD.nma.fe.c <- dic.samples(jags.nma.fe.c, n.iter = n.iter, thin = n.thin)

    # FE NMA with evidence inconsistency
    set.seed(seed)
    jags.nma.fe.ic <- jags.model(file = textConnection(nma.fe.ic()),
      data = dat.jags, inits = inits.fe, n.chains = n.chains, n.adapt = n.adapt)
    update(jags.nma.fe.ic, n.iter = n.burnin)
    coda.nma.fe.ic <- coda.samples(model = jags.nma.fe.ic,
      variable.names = params.fe, n.iter = n.iter, thin = n.thin)
    smry.nma.fe.ic <- summary(coda.nma.fe.ic)
    out.nma.fe.ic <- smry.nma.fe.ic$quantiles[grep("lor",
      rownames(smry.nma.fe.ic$quantiles)), c("2.5%", "50%", "97.5%")]
    pD.nma.fe.ic <- dic.samples(jags.nma.fe.ic, n.iter = n.iter, thin = n.thin)

    icdf.fe <- sum(pD.nma.fe.ic$penalty) - sum(pD.nma.fe.c$penalty)
    out$icdf.fe <- icdf.fe

    if(traceplot){
      # trace plots for FE NMA with evidence consistency
      for(i in 1:(n.trt - 1)){
        for(j in (i + 1):n.trt){
          filename <- paste0("traceplot_NMA_FE_C_",
            "lor[", i , ",", j, "]", ".png")
          if(!is.null(nma.name)){
            filename <- paste0(nma.name, "_", filename)
          }
          png(filename, res = 600, height = 8.5, width = 11, units = "in")
          par(mfrow = c(length(coda.nma.fe.c), 1))
          for(k in 1:length(coda.nma.fe.c)){
            temp <- as.vector(coda.nma.fe.c[[k]][,paste0("lor[", i, ",", j, "]")])
            plot(temp, type = "l", col = "red", xlab = "Iteration",
              ylab = paste0("lor[", i, ",", j, "]"), main = paste("Chain", k))
          }
          dev.off()
        }
      }

      # trace plots for FE NMA with evidence inconsistency
      for(i in 1:(n.trt - 1)){
        for(j in (i + 1):n.trt){
          filename <- paste0("traceplot_NMA_FE_IC_",
            "lor[", i , ",", j, "]", ".png")
          if(!is.null(nma.name)){
            filename <- paste0(nma.name, "_", filename)
          }
          png(filename, res = 600, height = 8.5, width = 11, units = "in")
          par(mfrow = c(length(coda.nma.fe.ic), 1))
          for(k in 1:length(coda.nma.fe.ic)){
            temp <- as.vector(coda.nma.fe.ic[[k]][,paste0("lor[", i, ",", j, "]")])
            plot(temp, type = "l", col = "red", xlab = "Iteration",
              ylab = paste0("lor[", i, ",", j, "]"), main = paste("Chain", k))
          }
          dev.off()
        }
      }
    }
  }

  if(is.element("re", type)){
    set.seed(seed)
    inits.re <- NULL
    for(i in 1:n.chains){
      inits.re[[i]] <- list(mu = rep(rnorm(1), n.study), tau = runif(1, 0, 2),
            .RNG.name = "base::Wichmann-Hill", .RNG.seed = sample(123456789, 1))
    }
    params.re <- c("lor", "tau", "totresdev")

    # RE NMA with evidence consistency
    set.seed(seed)
    jags.nma.re.c <- jags.model(file = textConnection(nma.re.c()),
      data = dat.jags, inits = inits.re, n.chains = n.chains, n.adapt = n.adapt)
    update(jags.nma.re.c, n.iter = n.burnin)
    coda.nma.re.c <- coda.samples(model = jags.nma.re.c,
      variable.names = params.re, n.iter = n.iter, thin = n.thin)
    smry.nma.re.c <- summary(coda.nma.re.c)
    out.nma.re.c <- smry.nma.re.c$quantiles[grep("lor",
      rownames(smry.nma.re.c$quantiles)), c("2.5%", "50%", "97.5%")]
    pD.nma.re.c <- dic.samples(jags.nma.re.c, n.iter = n.iter, thin = n.thin)

    # RE NMA with evidence inconsistency
    set.seed(seed)
    jags.nma.re.ic <- jags.model(file = textConnection(nma.re.ic()),
      data = dat.jags, inits = inits.re, n.chains = n.chains, n.adapt = n.adapt)
    update(jags.nma.re.ic, n.iter = n.burnin)
    coda.nma.re.ic <- coda.samples(model = jags.nma.re.ic,
      variable.names = params.re, n.iter = n.iter, thin = n.thin)
    smry.nma.re.ic <- summary(coda.nma.re.ic)
    out.nma.re.ic <- smry.nma.re.ic$quantiles[grep("lor",
      rownames(smry.nma.re.ic$quantiles)), c("2.5%", "50%", "97.5%")]
    pD.nma.re.ic <- dic.samples(jags.nma.re.ic, n.iter = n.iter, thin = n.thin)

    icdf.re <- sum(pD.nma.re.ic$penalty) - sum(pD.nma.re.c$penalty)
    out$icdf.re <- icdf.re

    if(traceplot){
      # trace plots for RE NMA with evidence consistency
      for(i in 1:(n.trt - 1)){
        for(j in (i + 1):n.trt){
          filename <- paste0("traceplot_NMA_RE_C_",
            "lor[", i , ",", j, "]", ".png")
          if(!is.null(nma.name)){
            filename <- paste0(nma.name, "_", filename)
          }
          png(filename, res = 600, height = 8.5, width = 11, units = "in")
          par(mfrow = c(length(coda.nma.re.c), 1))
          for(k in 1:length(coda.nma.re.c)){
            temp <- as.vector(coda.nma.re.c[[k]][,paste0("lor[", i, ",", j, "]")])
            plot(temp, type = "l", col = "red", xlab = "Iteration",
              ylab = paste0("lor[", i, ",", j, "]"), main = paste("Chain", k))
          }
          dev.off()
        }
      }

      # trace plots for RE NMA with evidence inconsistency
      for(i in 1:(n.trt - 1)){
        for(j in (i + 1):n.trt){
          filename <- paste0("traceplot_NMA_RE_IC_",
             "lor[", i , ",", j, "]", ".png")
          if(!is.null(nma.name)){
            filename <- paste0(nma.name, "_", filename)
          }
          png(filename, res = 600, height = 8.5, width = 11, units = "in")
          par(mfrow = c(length(coda.nma.re.ic), 1))
          for(k in 1:length(coda.nma.re.ic)){
            temp <- as.vector(coda.nma.re.ic[[k]][,paste0("lor[", i, ",", j, "]")])
            plot(temp, type = "l", col = "red", xlab = "Iteration",
              ylab = paste0("lor[", i, ",", j, "]"), main = paste("Chain", k))
          }
          dev.off()
        }
      }
    }
  }

  return(out)
}

meta.or.smd <- function(y1, sd1, n1, y0, sd0, n0, r1, m1, r0, m0, id.bin, data,
  n.adapt = 1000, n.chains = 3, n.burnin = 5000, n.iter = 20000, n.thin = 2,
  seed = 1234){

  if(missing(y1) | missing(y0)) stop("please specify sample means for the continuous outcome.")
  if(missing(sd1) | missing(sd0)) stop("please specify sample standard deviations for the continuous outcome.")
  if(missing(n1) | missing(n0)) stop("lease specify sample sizes for the continuous outcome.")
  if(missing(r1) | missing(r0)) stop("please specify event counts for the binary outcome.")
  if(missing(m1) | missing(m0)) stop("please specify sample sizes for the binary outcome.")
  if(!missing(data)){
    y1 <- eval(substitute(y1), data, parent.frame())
    sd1 <- eval(substitute(sd1), data, parent.frame())
    n1 <- eval(substitute(n1), data, parent.frame())
    y0 <- eval(substitute(y0), data, parent.frame())
    sd0 <- eval(substitute(sd0), data, parent.frame())
    n0 <- eval(substitute(n0), data, parent.frame())
    r1 <- eval(substitute(r1), data, parent.frame())
    m1 <- eval(substitute(m1), data, parent.frame())
    r0 <- eval(substitute(r0), data, parent.frame())
    m0 <- eval(substitute(m0), data, parent.frame())
    id.bin <- eval(substitute(id.bin), data, parent.frame())
  }

  ## Data
  jags.data <- list(
    M = length(which(data$id.bin == 1)), 
    m0 = data$m0[which(data$id.bin == 1)], 
    r0 = data$r0[which(data$id.bin == 1)], 
    m1 = data$m1[which(data$id.bin == 1)], 
    r1 = data$r1[which(data$id.bin == 1)],
    N = length(which(data$id.bin == 0)), 
    n0 = data$n0[which(data$id.bin == 0)], 
    y0 = data$y0[which(data$id.bin == 0)], 
    sd.sq0 = data$sd0[which(data$id.bin == 0)]^2, 
    n1 = data$n1[which(data$id.bin == 0)], 
    y1 = data$y1[which(data$id.bin == 0)], 
    sd.sq1 = data$sd1[which(data$id.bin == 0)]^2,
    idx.bin = which(data$id.bin == 1), 
    idx.cont = which(data$id.bin == 0))
  
  ## MCMC
  set.seed(seed)
  inits <- NULL
  rng.seeds <- sample(1000000, 3)
  for(i in 1:n.chains){
    inits[[i]] <- list(d = rnorm(1), tau = runif(1, 0, 2),
      .RNG.name = "base::Wichmann-Hill", .RNG.seed = sample(123456789, 1))
  }

  set.seed(seed)
  jags.m <- jags.model(file = textConnection(meta.or.smd.model()),
                       data = jags.data, inits = inits, n.chains = n.chains, 
                       n.adapt = n.adapt)
  update(jags.m, n.burnin)  ## burn_in
  params <- c("d", "tau", "theta")
  samps <- coda.samples(model = jags.m, variable.names = params,
    n.iter = n.iter, thin = n.thin)
  bayes.result <- summary(samps)  ## Bayesian summary
  
  ## Bayesian results
  bayes.smd.dt <- structure(
    list(median = bayes.result$quantiles[,"50%"], 
      lci = bayes.result$quantiles[,"2.5%"],
      uci = bayes.result$quantiles[,"97.5%"]), 
    .Names = c("median", "lci", "uci"),
    row.names = row.names(bayes.result$quantiles),
    class = "data.frame")
  
  return(bayes.smd.dt) # return the overall SMD, between-study SD, and individual studies' SMDs
}

## adapted from the package "meta"
hypergeometric <- function(n1, m1, N, psi) {
  n2 <- N - n1
  
  if(n1<0 | n2<0 | m1<0 | m1 > N | psi <= 0)
    stop("wrong argument in hypergeometric")
  
  mode.compute <- function() {
    a <- psi - 1
    b <- -((m1 + n1 + 2) * psi + n2 - m1)     
    c <- psi * (n1 + 1) * (m1 + 1)
    q <- b + sign(b) * sqrt(b * b - 4 * a * c)
    q <- -q / 2
    
    mode <- trunc(c / q) 
    if(uu >= mode && mode >= ll) return(mode)
    else return(trunc(q / a))      
  }       
  
  r.function <- function(i) (n1 - i + 1) * (m1 - i + 1) / i / (n2 - m1 + i) * psi
  
  mean <- function() sum(prob[(ll:uu) + shift] * (ll:uu)) 
  var <-  function() sum(prob[(ll:uu) + shift] * (ll:uu)^2) - mean()^2          
  
  d <- function(x) return(prob[x + shift])
  
  p <- function(x, lower.tail = TRUE) {   
    if(lower.tail) return(sum(prob[ll:(x + shift)]))
    else return(sum(prob[(x + shift):uu]))
  }
  
  sample.low.to.high <- function(lower.end, ran) { 
    for(i in lower.end:uu) {                                
      if(ran <= prob[i + shift]) return(i)
      ran <- ran - prob[i + shift]
    }                                
  }
  
  sample.high.to.low <- function(upper.end, ran) {           
    for(i in upper.end:ll) {                              
      if(ran <= prob[i + shift]) return(i)
      ran <- ran - prob[i + shift]
    } 
  }  

  r <- function() {
    ran <- runif(1) 
    
    if(mode == ll) return(sample.low.to.high(ll, ran))            
    if(mode == uu) return(sample.high.to.low(uu, ran))                                         
    
    if(ran < prob[mode + shift]) return(mode)             
    ran <- ran - prob[mode + shift]
    
    lower <- mode - 1                                                                            
    upper <- mode + 1
    
    repeat{                                     
      if(prob[upper + shift] >= prob[lower + shift]) {              
        if(ran < prob[upper + shift]) return(upper)
        ran <- ran - prob[upper + shift]
        if(upper == uu) return(sample.high.to.low(lower, ran))
        upper <- upper + 1                            
      }
      
      else {
        if(ran < prob[lower + shift]) return(lower)
        ran <- ran - prob[lower + shift]
        if(lower == ll) return(sample.low.to.high(upper, ran))
        lower <- lower - 1                   
      }      
    } 
  }

  ll <- max(0, m1 - n2)
  uu <- min(n1, m1)              
  mode <- mode.compute()  
  
  prob <- array(1, uu - ll + 1)
  
  shift <- 1 - ll 
  if(mode < uu) #note the shift of location
    {  
      r1 <- r.function((mode + 1):uu)       
      prob[(mode + 1 + shift):(uu + shift)] <- cumprod(r1)       
    }
  
  if(mode > ll) {
    r1 <- 1 / r.function(mode:(ll + 1))
    prob[(mode - 1 + shift):(ll + shift)] <- cumprod(r1)
  }
  
  prob <- prob / sum(prob) 
  
  return(list(mean = mean, var = var, d = d, p = p, r = r))
}

tau2.r.solver <- function(w, Qr){
	f <- function(tau2){
		out <- sum(sqrt(1 - w/sum(w) + tau2*(w - 2*w^2/sum(w) + w*sum(w^2)/(sum(w))^2))) - Qr*sqrt(3.14159/2)
		return(out)
	}
	f <- Vectorize(f)
	tau.upp <- Qr*sqrt(3.14159/2)/sum(sqrt(w - 2*w^2/sum(w) + w*sum(w^2)/(sum(w))^2))
	tau2.upp <- tau.upp^2
	f.low <- f(0)
	f.upp <- f(tau2.upp)
	if(f.low*f.upp > 0){
		tau2.r <- 0
	}else{
		tau2.r <- uniroot(f, interval = c(0, tau2.upp))
		tau2.r <- tau2.r$root
	}
	return(tau2.r)
}

pb.hybrid.binary <- function(n00, n01, n10, n11, data, methods, iter.resam = 1000, theo.pval = TRUE){
  if(!missing(data)){
    n00 <- eval(substitute(n00), data, parent.frame())
    n01 <- eval(substitute(n01), data, parent.frame())
    n10 <- eval(substitute(n10), data, parent.frame())
    n11 <- eval(substitute(n11), data, parent.frame())
  }
  if(missing(methods)){
    methods <- c("rank", "reg", "reg.het", "skew", "skew.het", "inv.sqrt.n", "trimfill",
      "n", "inv.n", "as.rank", "as.reg", "as.reg.het", "smoothed", "smoothed.het", "score", "count")
  }
  if(!all(is.element(methods, c("rank", "reg", "reg.het", "skew", "skew.het", "inv.sqrt.n", "trimfill",
    "n", "inv.n", "as.rank", "as.reg", "as.reg.het", "smoothed", "smoothed.het", "score", "count")))){
    stop("incorrect input for methods.")
  }

  pval.rank <- pval.reg <- pval.reg.het <- pval.skew <- pval.skew.het <- pval.inv.sqrt.n <- pval.trimfill <-
    pval.n <- pval.inv.n <- pval.as.rank <- pval.as.reg <- pval.as.reg.het <-
    pval.smoothed <- pval.smoothed.het <- pval.score <- pval.count <- NA
  pval.rank.theo <- pval.reg.theo <- pval.reg.het.theo <- pval.skew.theo <- pval.skew.het.theo <- pval.inv.sqrt.n.theo <- pval.trimfill.theo <-
    pval.n.theo <- pval.inv.n.theo <- pval.as.rank.theo <- pval.as.reg.theo <- pval.as.reg.het.theo <-
    pval.smoothed.theo <- pval.smoothed.het.theo <- pval.score.theo <- pval.count.theo <- NA

  n00.ori <- n00
  n01.ori <- n01
  n10.ori <- n10
  n11.ori <- n11

  counts <- check.counts(n00, n01, n10, n11)
  n00 <- counts$n00
  n01 <- counts$n01
  n10 <- counts$n10
  n11 <- counts$n11

  y <- log(n11/n10*n00/n01)
  s2 <- 1/n00 + 1/n01 + 1/n10 + 1/n11
  n <- n00 + n01 + n10 + n11
  pi0 <- n01/(n00 + n01)
  n0Sum <- n00 + n01
  n1Sum <- n10 + n11

  if(is.element("rank", methods)){
    rank <- pb.rank(y, s2)
    stat.rank <- rank$stat
    if(theo.pval) pval.rank.theo <- rank$pval
  }
  if(is.element("reg", methods)){
    reg <- pb.reg(y, s2)
    stat.reg <- reg$stat
    if(theo.pval) pval.reg.theo <- reg$pval
  }
  if(is.element("reg.het", methods)){
    reg.het <- pb.reg.het(y, s2)
    stat.reg.het <- reg.het$stat
    if(theo.pval) pval.reg.het.theo <- reg.het$pval
  }
  if(is.element("skew", methods)){
    skew <- pb.skew(y, s2)
    stat.skew <- skew$stat
    if(theo.pval) pval.skew.theo <- skew$pval
  }
  if(is.element("skew.het", methods)){
    skew.het <- pb.skew.het(y, s2)
    stat.skew.het <- skew.het$stat
    if(theo.pval) pval.skew.het.theo <- skew.het$pval
  }
  if(is.element("inv.sqrt.n", methods)){
    inv.sqrt.n <- pb.inv.sqrt.n(y, s2, n)
    stat.inv.sqrt.n <- inv.sqrt.n$stat
    if(theo.pval) pval.inv.sqrt.n.theo <- inv.sqrt.n$pval
  }
  if(is.element("trimfill", methods)){
    options(warn = -1)
    rma <- rma(yi = y, vi = s2, method = "DL")
    trimfill <- pb.trimfill(rma, estimator = "R0")
    stat.trimfill <- trimfill$k0
    if(theo.pval) pval.trimfill.theo <- trimfill$pval
    options(warn = 0)
  }
  if(is.element("n", methods)){
    pbn <- pb.n(y = y, n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.n <- pbn$stat
    if(theo.pval) pval.n.theo <- pbn$pval
  }
  if(is.element("inv.n", methods)){
    inv.n <- pb.inv.n(y = y, n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.inv.n <- inv.n$stat
    if(theo.pval) pval.inv.n.theo <- inv.n$pval
  }
  if(is.element("as.rank", methods)){
    as.rank <- pb.as.rank(n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.as.rank <- as.rank$stat
    if(theo.pval) pval.as.rank.theo <- as.rank$pval
  }
  if(is.element("as.reg", methods)){
    as.reg <- pb.as.reg(n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.as.reg <- as.reg$stat
    if(theo.pval) pval.as.reg.theo <- as.reg$pval
  }
  if(is.element("as.reg.het", methods)){
    as.reg.het <- pb.as.reg.het(n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.as.reg.het <- as.reg.het$stat
    if(theo.pval) pval.as.reg.het.theo <- as.reg.het$pval
  }
  if(is.element("smoothed", methods)){
    smoothed <- pb.smoothed(y = y, n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.smoothed <- smoothed$stat
    if(theo.pval) pval.smoothed.theo <- smoothed$pval
  }
  if(is.element("smoothed.het", methods)){
    smoothed.het <- pb.smoothed.het(y = y, n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.smoothed.het <- smoothed.het$stat
    if(theo.pval) pval.smoothed.het.theo <- smoothed.het$pval
  }
  if(is.element("score", methods)){
    score <- pb.score(n00 = n00, n01 = n01, n10 = n10, n11 = n11)
    stat.score <- score$stat
    if(theo.pval) pval.score.theo <- score$pval
  }
  if(is.element("count", methods)){
    count <- pb.count(n00 = n00.ori, n01 = n01.ori, n10 = n10.ori, n11 = n11.ori)
    stat.count <- count$stat
    if(theo.pval) pval.count.theo <- count$pval
  }

  N <- length(y)
  w <- 1/s2
  theta.hat <- sum(w*y)/sum(w)
  Q <- sum(w*(y - theta.hat)^2)
  tau2.hat <- (Q - N + 1)/(sum(w) - sum(w^2)/sum(w))
  tau2.hat <- max(c(0, tau2.hat))
  w <- 1/(s2 + tau2.hat)
  theta.hat <- sum(w*y)/sum(w)
  if(all(is.na(n))) n <- rep(NA, N)

  stat.rank.resam <- stat.reg.resam <- stat.reg.het.resam <- stat.skew.resam <- stat.skew.het.resam <-
    stat.inv.sqrt.n.resam <- stat.trimfill.resam <-
    stat.n.resam <- stat.inv.n.resam <- stat.as.rank.resam <- stat.as.reg.resam <- stat.as.reg.het.resam <-
    stat.smoothed.resam <- stat.smoothed.het.resam <- stat.score.resam <- stat.count.resam <- stat.hybrid.resam <- stat.hybrid.theo.resam <- rep(NA, iter.resam)
  pval.rank.theo.resam <- pval.reg.theo.resam <- pval.reg.het.theo.resam <- pval.skew.theo.resam <- pval.skew.het.theo.resam <- pval.inv.sqrt.n.theo.resam <- pval.trimfill.theo.resam <-
    pval.n.theo.resam <- pval.inv.n.theo.resam <- pval.as.rank.theo.resam <- pval.as.reg.theo.resam <- pval.as.reg.het.theo.resam <-
    pval.smoothed.theo.resam <- pval.smoothed.het.theo.resam <- pval.score.theo.resam <- pval.count.theo.resam <- rep(NA, iter.resam)
  for(i in 1:iter.resam){
    idx <- sample(1:N, replace = TRUE)
    n0Sum.resam <- n0Sum[idx]
    n1Sum.resam <- n1Sum[idx]
    pi0.resam <- pi0[idx]
    s2.resam <- s2[idx]
    theta.resam <- rnorm(n = N, mean = theta.hat, sd = sqrt(s2.resam + tau2.hat))
    n00.resam <- n01.resam <- n10.resam <- n11.resam <- rep(NA, N)
    for(j in 1:N){
      counts.resam <- find.counts(n0. = n0Sum.resam[j], n1. = n1Sum.resam[j], lor = theta.resam[j], lor.var = s2.resam[j], p0.ori = pi0.resam[j])
      n00.resam[j] <- counts.resam$n00
      n01.resam[j] <- counts.resam$n01
      n10.resam[j] <- counts.resam$n10
      n11.resam[j] <- counts.resam$n11
    }
    n00.ori.resam <- n00.resam
    n00.ori.resam <- round(n00.ori.resam)
    n01.ori.resam <- n01.resam
    n01.ori.resam <- round(n01.ori.resam)
    n10.ori.resam <- n10.resam
    n10.ori.resam <- round(n10.ori.resam)
    n11.ori.resam <- n11.resam
    n11.ori.resam <- round(n11.ori.resam)
    counts.resam <- check.counts(n00.resam, n01.resam, n10.resam, n11.resam)
    n00.resam <- counts.resam$n00
    n01.resam <- counts.resam$n01
    n10.resam <- counts.resam$n10
    n11.resam <- counts.resam$n11
    y.resam <- log(n11.resam/n10.resam*n00.resam/n01.resam)
    s2.resam <- 1/n00.resam + 1/n01.resam + 1/n10.resam + 1/n11.resam
    n.resam <- n00.resam + n01.resam + n10.resam + n11.resam

    if(is.element("rank", methods)){
      rank.resam <- pb.rank(y.resam, s2.resam)
      stat.rank.resam[i] <- rank.resam$stat
      if(theo.pval) pval.rank.theo.resam[i] <- rank.resam$pval
    }
    if(is.element("reg", methods)){
      reg.resam <- pb.reg(y.resam, s2.resam)
      stat.reg.resam[i] <- reg.resam$stat
      if(theo.pval) pval.reg.theo.resam[i] <- reg.resam$pval
    }
    if(is.element("reg.het", methods)){
      reg.het.resam <- pb.reg.het(y.resam, s2.resam)
      stat.reg.het.resam[i] <- reg.het.resam$stat
      if(theo.pval) pval.reg.het.theo.resam[i] <- reg.het.resam$pval
    }
    if(is.element("skew", methods)){
      skew.resam <- pb.skew(y.resam, s2.resam)
      stat.skew.resam[i] <- skew.resam$stat
      if(theo.pval) pval.skew.theo.resam[i] <- skew.resam$pval
    }
    if(is.element("skew.het", methods)){
      skew.het.resam <- pb.skew.het(y.resam, s2.resam)
      stat.skew.het.resam[i] <- skew.het.resam$stat
      if(theo.pval) pval.skew.het.theo.resam[i] <- skew.het.resam$pval
    }
    if(is.element("inv.sqrt.n", methods)){
      inv.sqrt.n.resam <- pb.inv.sqrt.n(y.resam, s2.resam, n.resam)
      stat.inv.sqrt.n.resam[i] <- inv.sqrt.n.resam$stat
      if(theo.pval) pval.inv.sqrt.n.theo.resam[i] <- inv.sqrt.n.resam$pval
    }
    if(is.element("trimfill", methods)){
      options(warn = -1)
      rma.resam <- rma(yi = y.resam, vi = s2.resam, method = "DL")
      trimfill.resam <- pb.trimfill(rma.resam, estimator = "R0")
      stat.trimfill.resam[i] <- trimfill.resam$k0
      if(theo.pval) pval.trimfill.theo.resam[i] <- trimfill.resam$pval
      options(warn = 0)
    }
    if(is.element("n", methods)){
      pbn.resam <- pb.n(y = y.resam, n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.n.resam[i] <- pbn.resam$stat
      if(theo.pval) pval.n.theo.resam[i] <- pbn.resam$pval
    }
    if(is.element("inv.n", methods)){
      inv.n.resam <- pb.inv.n(y = y.resam, n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.inv.n.resam[i] <- inv.n.resam$stat
      if(theo.pval) pval.inv.n.theo.resam[i] <- inv.n.resam$pval
    }
    if(is.element("as.rank", methods)){
      as.rank.resam <- pb.as.rank(n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.as.rank.resam[i] <- as.rank.resam$stat
      if(theo.pval) pval.as.rank.theo.resam[i] <- as.rank.resam$pval
    }
    if(is.element("as.reg", methods)){
      as.reg.resam <- pb.as.reg(n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.as.reg.resam[i] <- as.reg.resam$stat
      if(theo.pval) pval.as.reg.theo.resam[i] <- as.reg.resam$pval
    }
    if(is.element("as.reg.het", methods)){
      as.reg.het.resam <- pb.as.reg.het(n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.as.reg.het.resam[i] <- as.reg.het.resam$stat
      if(theo.pval) pval.as.reg.het.theo.resam[i] <- as.reg.het.resam$pval
    }
    if(is.element("smoothed", methods)){
      smoothed.resam <- pb.smoothed(y = y.resam, n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.smoothed.resam[i] <- smoothed.resam$stat
      if(theo.pval) pval.smoothed.theo.resam[i] <- smoothed.resam$pval
    }
    if(is.element("smoothed.het", methods)){
      smoothed.het.resam <- pb.smoothed.het(y = y.resam, n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.smoothed.het.resam[i] <- smoothed.het.resam$stat
      if(theo.pval) pval.smoothed.het.theo.resam[i] <- smoothed.het.resam$pval
    }
    if(is.element("score", methods)){
      score.resam <- pb.score(n00 = n00.resam, n01 = n01.resam, n10 = n10.resam, n11 = n11.resam)
      stat.score.resam[i] <- score.resam$stat
      if(theo.pval) pval.score.theo.resam[i] <- score.resam$pval
    }
    if(is.element("count", methods)){
      count.resam <- pb.count(n00 = n00.ori.resam, n01 = n01.ori.resam, n10 = n10.ori.resam, n11 = n11.ori.resam)
      stat.count.resam[i] <- count.resam$stat
      if(theo.pval) pval.count.theo.resam[i] <- count.resam$pval
    }
  }

  if(is.element("rank", methods)) pval.rank <- (sum(abs(stat.rank.resam) >= abs(stat.rank)) + 1)/(iter.resam + 1)
  if(is.element("reg", methods)) pval.reg <- (sum(abs(stat.reg.resam) >= abs(stat.reg)) + 1)/(iter.resam + 1)
  if(is.element("reg.het", methods)) pval.reg.het <- (sum(abs(stat.reg.het.resam) >= abs(stat.reg.het)) + 1)/(iter.resam + 1)
  if(is.element("skew", methods)) pval.skew <- (sum(abs(stat.skew.resam) >= abs(stat.skew)) + 1)/(iter.resam + 1)
  if(is.element("skew.het", methods)) pval.skew.het <- (sum(abs(stat.skew.het.resam) >= abs(stat.skew.het)) + 1)/(iter.resam + 1)
  if(is.element("inv.sqrt.n", methods)) pval.inv.sqrt.n <- (sum(abs(stat.inv.sqrt.n.resam) >= abs(stat.inv.sqrt.n)) + 1)/(iter.resam + 1)
  if(is.element("trimfill", methods)) pval.trimfill <- (sum(abs(stat.trimfill.resam) >= abs(stat.trimfill)) + 1)/(iter.resam + 1)
  if(is.element("n", methods)) pval.n <- (sum(abs(stat.n.resam) >= abs(stat.n)) + 1)/(iter.resam + 1)
  if(is.element("inv.n", methods)) pval.inv.n <- (sum(abs(stat.inv.n.resam) >= abs(stat.inv.n)) + 1)/(iter.resam + 1)
  if(is.element("as.rank", methods)) pval.as.rank <- (sum(abs(stat.as.rank.resam) >= abs(stat.as.rank)) + 1)/(iter.resam + 1)
  if(is.element("as.reg", methods)) pval.as.reg <- (sum(abs(stat.as.reg.resam) >= abs(stat.as.reg)) + 1)/(iter.resam + 1)
  if(is.element("as.reg.het", methods)) pval.as.reg.het <- (sum(abs(stat.as.reg.het.resam) >= abs(stat.as.reg.het)) + 1)/(iter.resam + 1)
  if(is.element("smoothed", methods)) pval.smoothed <- (sum(abs(stat.smoothed.resam) >= abs(stat.smoothed)) + 1)/(iter.resam + 1)
  if(is.element("smoothed.het", methods)) pval.smoothed.het <- (sum(abs(stat.smoothed.het.resam) >= abs(stat.smoothed.het)) + 1)/(iter.resam + 1)
  if(is.element("score", methods)) pval.score <- (sum(abs(stat.score.resam) >= abs(stat.score)) + 1)/(iter.resam + 1)
  if(is.element("count", methods)) pval.count <- (sum(abs(stat.count.resam) >= abs(stat.count)) + 1)/(iter.resam + 1)

  if(length(methods) == 1){
    if(!theo.pval) out <- get(paste0("pval.", methods))
    if(theo.pval){
      out <- list(get(paste0("pval.", methods)), get(paste0("pval.", methods, ".theo")))
      names(out) <- c(paste0("pval.", methods), paste0("pval.", methods, ".theo"))
    }
  }

  if(length(methods) > 1){
    stat.hybrid <- min(c(pval.rank, pval.reg, pval.reg.het, pval.skew, pval.skew.het, pval.inv.sqrt.n, pval.trimfill,
      pval.n, pval.inv.n, pval.as.rank, pval.as.reg, pval.as.reg.het, pval.smoothed, pval.smoothed.het, pval.score, pval.count), na.rm = TRUE)
    if(theo.pval) stat.hybrid.theo <- min(c(pval.rank.theo, pval.reg.theo, pval.reg.het.theo, pval.skew.theo, pval.skew.het.theo, pval.inv.sqrt.n.theo, pval.trimfill.theo,
      pval.n.theo, pval.inv.n.theo, pval.as.rank.theo, pval.as.reg.theo, pval.as.reg.het.theo, pval.smoothed.theo, pval.smoothed.het.theo, pval.score.theo, pval.count.theo), na.rm = TRUE)
    pval.rank.resam <- pval.reg.resam <- pval.reg.het.resam <- pval.skew.resam <- pval.skew.het.resam <-
      pval.inv.sqrt.n.resam <- pval.trimfill.resam <-
      pval.n.resam <- pval.inv.n.resam <- pval.as.rank.resam <- pval.as.reg.resam <- pval.as.reg.het.resam <-
      pval.smoothed.resam <- pval.smoothed.het.resam <- pval.score.resam <- pval.count.resam <- rep(NA, iter.resam)
    for(i in 1:iter.resam){
      if(theo.pval) stat.hybrid.theo.resam[i] <- min(c(pval.rank.theo.resam[i], pval.reg.theo.resam[i], pval.reg.het.theo.resam[i], pval.skew.theo.resam[i], pval.skew.het.theo.resam[i], pval.inv.sqrt.n.theo.resam[i], pval.trimfill.theo.resam[i],
        pval.n.theo.resam[i], pval.inv.n.theo.resam[i], pval.as.rank.theo.resam[i], pval.as.reg.theo.resam[i], pval.as.reg.het.theo.resam[i], pval.smoothed.theo.resam[i], pval.smoothed.het.theo.resam[i], pval.score.theo.resam[i], pval.count.theo.resam[i]), na.rm = TRUE)
      if(is.element("rank", methods)) pval.rank.resam[i] <- (sum(abs(stat.rank.resam[-i]) >= abs(stat.rank.resam[i])) + 1)/iter.resam
      if(is.element("reg", methods)) pval.reg.resam[i] <- (sum(abs(stat.reg.resam[-i]) >= abs(stat.reg.resam[i])) + 1)/iter.resam
      if(is.element("reg.het", methods)) pval.reg.het.resam[i] <- (sum(abs(stat.reg.het.resam[-i]) >= abs(stat.reg.het.resam[i])) + 1)/iter.resam
      if(is.element("skew", methods)) pval.skew.resam[i] <- (sum(abs(stat.skew.resam[-i]) >= abs(stat.skew.resam[i])) + 1)/iter.resam
      if(is.element("skew.het", methods)) pval.skew.het.resam[i] <- (sum(abs(stat.skew.het.resam[-i]) >= abs(stat.skew.het.resam[i])) + 1)/iter.resam
      if(is.element("inv.sqrt.n", methods)) pval.inv.sqrt.n.resam[i] <- (sum(abs(stat.inv.sqrt.n.resam[-i]) >= abs(stat.inv.sqrt.n.resam[i])) + 1)/iter.resam
      if(is.element("trimfill", methods)) pval.trimfill.resam[i] <- (sum(abs(stat.trimfill.resam[-i]) >= abs(stat.trimfill.resam[i])) + 1)/iter.resam
      if(is.element("n", methods)) pval.n.resam[i] <- (sum(abs(stat.n.resam[-i]) >= abs(stat.n.resam[i])) + 1)/iter.resam
      if(is.element("inv.n", methods)) pval.inv.n.resam[i] <- (sum(abs(stat.inv.n.resam[-i]) >= abs(stat.inv.n.resam[i])) + 1)/iter.resam
      if(is.element("as.rank", methods)) pval.as.rank.resam[i] <- (sum(abs(stat.as.rank.resam[-i]) >= abs(stat.as.rank.resam[i])) + 1)/iter.resam
      if(is.element("as.reg", methods)) pval.as.reg.resam[i] <- (sum(abs(stat.as.reg.resam[-i]) >= abs(stat.as.reg.resam[i])) + 1)/iter.resam
      if(is.element("as.reg.het", methods)) pval.as.reg.het.resam[i] <- (sum(abs(stat.as.reg.het.resam[-i]) >= abs(stat.as.reg.het.resam[i])) + 1)/iter.resam
      if(is.element("smoothed", methods)) pval.smoothed.resam[i] <- (sum(abs(stat.smoothed.resam[-i]) >= abs(stat.smoothed.resam[i])) + 1)/iter.resam
      if(is.element("smoothed.het", methods)) pval.smoothed.het.resam[i] <- (sum(abs(stat.smoothed.het.resam[-i]) >= abs(stat.smoothed.het.resam[i])) + 1)/iter.resam
      if(is.element("score", methods)) pval.score.resam[i] <- (sum(abs(stat.score.resam[-i]) >= abs(stat.score.resam[i])) + 1)/iter.resam
      if(is.element("count", methods)) pval.count.resam[i] <- (sum(abs(stat.count.resam[-i]) >= abs(stat.count.resam[i])) + 1)/iter.resam
      stat.hybrid.resam[i] <- min(c(pval.rank.resam[i], pval.reg.resam[i], pval.reg.het.resam[i], pval.skew.resam[i], pval.skew.het.resam[i], pval.inv.sqrt.n.resam[i], pval.trimfill.resam[i],
        pval.n.resam[i], pval.inv.n.resam[i], pval.as.rank.resam[i], pval.as.reg.resam[i], pval.as.reg.het.resam[i], pval.smoothed.resam[i], pval.smoothed.het.resam[i], pval.score.resam[i], pval.count.resam[i]), na.rm = TRUE)
    }
    pval.hybrid <- (sum(stat.hybrid.resam <= stat.hybrid) + 1)/(iter.resam + 1)
    if(!theo.pval){
      out <- list(pval.rank = pval.rank, pval.reg = pval.reg, pval.reg.het = pval.reg.het, pval.skew = pval.skew, pval.skew.het = pval.skew.het,
        pval.inv.sqrt.n = pval.inv.sqrt.n, pval.trimfill = pval.trimfill,
        pval.n = pval.n, pval.inv.n = pval.inv.n, pval.as.rank = pval.as.rank, pval.as.reg = pval.as.reg, pval.as.reg.het = pval.as.reg.het,
        pval.smoothed = pval.smoothed, pval.smoothed.het = pval.smoothed.het, pval.score = pval.score, pval.count = pval.count,
        pval.hybrid = pval.hybrid)
    }
    if(theo.pval){
      pval.hybrid.theo <- (sum(stat.hybrid.theo.resam <= stat.hybrid.theo) + 1)/(iter.resam + 1)
      out <- list(pval.rank = pval.rank, pval.rank.theo = pval.rank.theo, pval.reg = pval.reg, pval.reg.theo = pval.reg.theo, pval.reg.het = pval.reg.het, pval.reg.het.theo = pval.reg.het.theo, pval.skew = pval.skew, pval.skew.theo = pval.skew.theo, pval.skew.het = pval.skew.het, pval.skew.het.theo = pval.skew.het.theo,
        pval.inv.sqrt.n = pval.inv.sqrt.n, pval.inv.sqrt.n.theo = pval.inv.sqrt.n.theo, pval.trimfill = pval.trimfill, pval.trimfill.theo = pval.trimfill.theo,
        pval.n = pval.n, pval.n.theo = pval.n.theo, pval.inv.n = pval.inv.n, pval.inv.n.theo = pval.inv.n.theo, pval.as.rank = pval.as.rank, pval.as.rank.theo = pval.as.rank.theo, pval.as.reg = pval.as.reg, pval.as.reg.theo = pval.as.reg.theo, pval.as.reg.het = pval.as.reg.het, pval.as.reg.het.theo = pval.as.reg.het.theo,
        pval.smoothed = pval.smoothed, pval.smoothed.theo = pval.smoothed.theo, pval.smoothed.het = pval.smoothed.het, pval.smoothed.het.theo = pval.smoothed.het.theo, pval.score = pval.score, pval.score.theo = pval.score.theo, pval.count = pval.count, pval.count.theo = pval.count.theo,
        pval.hybrid = pval.hybrid, pval.hybrid.theo = pval.hybrid.theo)
    }
    na.test <- which(is.na(out))
    if(length(na.test) > 0) out <- out[-na.test]
  }

  return(out)
}

## JAGS model for the Bayesian model for combining
## standardized mean differences and odds ratios
meta.or.smd.model <- function(o){
  out <- "
    model{
      for(i in 1:M){
        r0[i] ~ dbinom(p0[i], m0[i])
        r1[i] ~ dbinom(p1[i], m1[i])
        probit(p0[i]) <- delta[i]
        probit(p1[i]) <- delta[i] - theta[idx.bin[i]]
        delta[i] ~ dnorm(0, 0.0001)
        theta[idx.bin[i]] ~ dnorm(d, inv.tausq)
      }

      for(i in 1:N){
        y0[i] ~ dnorm(mu0[i], inv.var0[i])
        y1[i] ~ dnorm(mu1[i], inv.var1[i])
        sd.sq0[i] ~ dgamma(alpha0[i], beta0[i])
        sd.sq1[i] ~ dgamma(alpha1[i], beta1[i])

        inv.var0[i] <- n0[i]/sigsq[i]
        alpha0[i] <- (n0[i]-1)/2
        beta0[i] <- (n0[i]-1)/(2*sigsq[i])
        inv.var1[i] <- n1[i]/sigsq[i]
        alpha1[i] <- (n1[i]-1)/2
        beta1[i] <- (n1[i]-1)/(2*sigsq[i])

        mu1[i] <- mu0[i] + theta[idx.cont[i]] * sqrt(sigsq[i])

        theta[idx.cont[i]] ~ dnorm(d, inv.tausq)
        mu0[i] ~ dnorm(0, 0.0001)
        inv.sigsq[i] ~ dgamma(0.001, 0.001)
        sigsq[i] <- 1/inv.sigsq[i]
      }

      d ~ dnorm(0, 0.0001)
      inv.tausq <- pow(tau, -2)
      tau ~ dunif(0, 2)
    }"
  return(out)
}

maprop.twostep <- function(e, n, data, link = "logit", method = "ML",
  alpha = 0.05, pop.avg = TRUE, int.approx = 10000, b.iter = 1000,
  seed = 1234){
  if(missing(e)) stop("need to specify event counts.")
  if(missing(n)) stop("need to specify sample sizes.")
  if(!missing(data)){
    e <- eval(substitute(e), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if(length(e) != length(n)) stop("lengths of e and n differ.")
  if(!is.element(link, c("log", "logit", "arcsine", "double.arcsine"))){
    stop("cannot recognize the data transformation (link).")
  }
  if(alpha < 0 | alpha > 1) stop("alpha should be between 0 and 1.")
  N <- length(e)
  if(any(e == 0) | any(e == n)){
    e <- e + 0.5
    n <- n + 1
  }
  p <- e/n
  if(link == "log"){
    y <- log(p)
    v <- 1/e - 1/n
    back.trans <- function(x) exp(x)
    back.trans <- Vectorize(back.trans)
  }
  if(link == "logit"){
    y <- log(p/(1 - p))
    v <- 1/e + 1/(n - e)
    back.trans <- function(x) exp(x)/(1 + exp(x))
    back.trans <- Vectorize(back.trans)
  }
  if(link == "arcsine"){
    y <- asin(sqrt(p))
    v <- 1/(4 * n)
    back.trans <- function(x) (sin(x))^2
    back.trans <- Vectorize(back.trans)
  }
  if(link == "double.arcsine"){
    y <- asin(sqrt(e/(n + 1))) + asin(sqrt((e + 1)/(n + 1)))
    v <- 1/(n + 0.5)
    back.trans.da <- function(x, pooled.n){
      if(x > asin(sqrt(1/(pooled.n + 1))) &
        x < (asin(sqrt(pooled.n/(pooled.n + 1))) + asin(1))){
        output <- 0.5 * (1 - sign(cos(x)) * sqrt(
          1 - (sin(x) + (sin(x) - 1/sin(x))/pooled.n)^2))
      }else{
        output <- (sin(x/2))^2
      }
      return(output)
    }
    back.trans.da <- Vectorize(back.trans.da)
    pooled.n.harmonic <- 1/mean(1/n)
    pooled.n.geometric <- exp(mean(log(n)))
    pooled.n.arithmetic <- mean(n)
  }
  rslt <- rma(yi = y, vi = v, method = method)
  mu.est <- as.numeric(rslt$beta)
  mu.se <- rslt$se
  mu.ci <- c(mu.est - qnorm(1 - alpha/2) * mu.se,
    mu.est + qnorm(1 - alpha/2) * mu.se)
  tau.est <- sqrt(rslt$tau2)
  if(link != "double.arcsine"){
    prop.c.est <- back.trans(mu.est)
    prop.c.ci <- back.trans(mu.ci)
  }
  if(link == "double.arcsine"){
    pooled.n.invvar <- 1/mu.se^2
    prop.c.est <- c(
      "harmonic" = back.trans.da(mu.est, pooled.n.harmonic),
      "geometric" = back.trans.da(mu.est, pooled.n.geometric),
      "arithmetic" = back.trans.da(mu.est, pooled.n.arithmetic),
      "invvar" = back.trans.da(mu.est, pooled.n.invvar))
    prop.c.ci <- rbind(
      "harmonic" = back.trans.da(mu.ci, pooled.n.harmonic),
      "geometric" = back.trans.da(mu.ci, pooled.n.geometric),
      "arithmetic" = back.trans.da(mu.ci, pooled.n.arithmetic),
      "invvar" = back.trans.da(mu.ci, pooled.n.invvar))
  }
  out <- list(prop.c.est = prop.c.est, prop.c.ci = prop.c.ci)
  if(pop.avg){
    set.seed(seed)
    stdnorms <- rnorm(int.approx)
    if(link != "double.arcsine"){
      prop.m.est <- mean(back.trans(mu.est + tau.est * stdnorms))
      mu.est.b <- prop.m.est.b <- rep(NA, b.iter)
    }
    if(link == "double.arcsine"){
      prop.m.est <- c(
        "harmonic" = mean(back.trans.da(mu.est + tau.est * stdnorms,
          pooled.n.harmonic)),
        "geometric" = mean(back.trans.da(mu.est + tau.est * stdnorms,
          pooled.n.geometric)),
        "arithmetic" = mean(back.trans.da(mu.est + tau.est * stdnorms,
          pooled.n.arithmetic)),
        "invvar" = mean(back.trans.da(mu.est + tau.est * stdnorms,
          pooled.n.invvar)))
      mu.est.b <- rep(NA, b.iter)
      prop.m.est.b <- matrix(NA, b.iter, 4)
    }
    set.seed(seed)
    b.idx <- 0
    b.w <- b.e <- 0
    while(b.idx < b.iter){
      idx.temp <- sample(N, replace = TRUE)
      y.temp <- y[idx.temp]
      v.temp <- v[idx.temp]
      suppressMessages(out.W.E <- tryCatch.W.E(
        rslt.temp <- rma(yi = y.temp, vi = v.temp, method = method)))
      if(is.null(out.W.E$warning) & !inherits(out.W.E$value, "error")){
        b.idx <- b.idx + 1
        mu.est.b[b.idx] <- as.numeric(rslt.temp$beta)
        tau.est.temp <- sqrt(rslt.temp$tau2)
        if(link != "double.arcsine"){
          prop.m.est.b[b.idx] <- mean(back.trans(mu.est.b[b.idx] +
            tau.est.temp * stdnorms))
        }
        if(link == "double.arcsine"){
          prop.m.est.b[b.idx,1] <- mean(back.trans.da(mu.est.b[b.idx] +
            tau.est.temp * stdnorms, pooled.n.harmonic))
          prop.m.est.b[b.idx,2] <- mean(back.trans.da(mu.est.b[b.idx] +
            tau.est.temp * stdnorms, pooled.n.geometric))
          prop.m.est.b[b.idx,3] <- mean(back.trans.da(mu.est.b[b.idx] +
            tau.est.temp * stdnorms, pooled.n.arithmetic))
          prop.m.est.b[b.idx,4] <- mean(back.trans.da(mu.est.b[b.idx] +
            tau.est.temp * stdnorms, pooled.n.invvar))
        }
      }
      if(!is.null(out.W.E$warning)){
        b.w <- b.w + 1
      }
      if(inherits(out.W.E$value, "error")){
        b.e <- b.e + 1
      }
    }
    mu.ci.b <- quantile(mu.est.b, probs = c(alpha/2, 1 - alpha/2))
    names(mu.ci.b) <- NULL
    if(link != "double.arcsine"){
      prop.c.ci.b <- back.trans(mu.ci.b)
      prop.m.ci.b <- quantile(prop.m.est.b, probs = c(alpha/2, 1 - alpha/2))
      names(prop.m.ci.b) <- NULL
    }
    if(link == "double.arcsine"){
      prop.c.ci.b <- rbind(
        "harmonic" = back.trans.da(mu.ci.b, pooled.n.harmonic),
        "geometric" = back.trans.da(mu.ci.b, pooled.n.geometric),
        "arithmetic" = back.trans.da(mu.ci.b, pooled.n.arithmetic),
        "invvar" = back.trans.da(mu.ci.b, pooled.n.invvar))
      prop.m.ci.b <- rbind(
        "harmonic" = quantile(prop.m.est.b[,1],
          probs = c(alpha/2, 1 - alpha/2)),
        "geometric" = quantile(prop.m.est.b[,2],
          probs = c(alpha/2, 1 - alpha/2)),
        "arithmetic" = quantile(prop.m.est.b[,3],
          probs = c(alpha/2, 1 - alpha/2)),
        "invvar" = quantile(prop.m.est.b[,4],
          probs = c(alpha/2, 1 - alpha/2)))
      colnames(prop.m.ci.b) <- NULL
    }
    out <- c(out, list(prop.c.ci.b = prop.c.ci.b,
      prop.m.est = prop.m.est, prop.m.ci.b = prop.m.ci.b))
    if(link == "double.arcsine"){
      out <- c(out, list(pooled.n = 
        c("harmonic" = pooled.n.harmonic,
        "geometric" = pooled.n.geometric,
        "arithmetic" = pooled.n.arithmetic,
        "invvar" = pooled.n.invvar)))
    }
    out <- c(out, list(b.w.e =
      c("bootstrap warnings" = b.w, "bootstrap errors" = b.e)))
  }
  return(out)
}

metapen.lambda <- function(y, s2, penalty = "tau2", lambda, tau2.re, tol = 10^(-10)) {
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(missing(lambda)) stop("please specify candidates of lambda to optimize.")
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  if(missing(tau2.re)) {tau2.re <- meta.ml(y, s2, tol)}

  n <- length(y)

  if (penalty == "tau2") {
    pen <- function(tau2) {
      tau2
    }
    pen1 <- function(tau2) {
      1
    }
  } else {
    pen <- function(tau2) {
      sqrt(tau2)
    }
    pen1 <- function(tau2) {
      1/(2 * sqrt(tau2))
    }
  }
  pen <- Vectorize(pen)
  pen1 <- Vectorize(pen1)

  mu.hat <- function(tau2) {
    sum(y/(s2 + tau2))/sum(1/(s2 + tau2))
  }
  mu.hat <- Vectorize(mu.hat)

  tau2.hat <- function(lambda) {
    target <- function(tau2) {
      mean(log(s2 + tau2) + (y - mu.hat(tau2))^2/(s2 + tau2)) + lambda * pen(tau2)/n
    }
    target <- Vectorize(target)

    target1 <- function(tau2) {
      p1 <- mean(1/(s2 + tau2))
      p2 <- mean((y - mu.hat(tau2))^2/(s2 + tau2)^2)
      out <- p1 - p2 + lambda * pen1(tau2)/n
      return(out)
    }

    out <- optim(par = tau2.re, fn = target, gr = target1, method = "Brent", lower = 0, upper = tau2.re *
                   1.1, control = list(reltol = tol))$par
    return(out)
  }
  tau2.hat <- Vectorize(tau2.hat)

  out.tau2 <- tau2.hat(lambda)
  out.mu <- mu.hat(out.tau2)
  out <- cbind(lambda = lambda, mu = out.mu, tau2 = out.tau2)
  rownames(out) <- rep("", length(lambda))
  return(out)
}

meta.ml <- function(y, s2, tol = 10^(-10)) {
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  if (var(y) == 0) stop(return(0))
  
  mu.hat <- function(tau2) {
    sum(y/(s2 + tau2))/sum(1/(s2 + tau2))
  }
  mu.hat <- Vectorize(mu.hat)
  
  target.ml <- function(tau2) {
    mean(log(s2 + tau2) + (y - mu.hat(tau2))^2/(s2 + tau2))
  }
  target.ml <- Vectorize(target.ml)
  
  target1.ml <- function(tau2) {
    p1 <- mean(1/(s2 + tau2))
    p2 <- mean((y - mu.hat(tau2))^2/(s2 + tau2)^2)
    out <- p1 - p2
    return(out)
  }
  target1.ml <- Vectorize(target1.ml)
  
  tau2.ml <- optim(par = var(y), fn = target.ml, gr = target1.ml, method = "Brent", lower = 0, upper = 100 * 
                     var(y), control = list(reltol = tol))$par
  
  return(tau2.ml)
}

mvma.hybrid.bayesian <- function(ys, vars, data, n.adapt = 1000, n.chains = 3, n.burnin = 10000, n.iter = 10000, n.thin = 1, data.name = NULL, traceplot = FALSE, coda = FALSE){
  if(missing(ys)) stop("the argument ys is missing.")
  if(missing(vars)) stop("the argument vars is missing.")
  if(!missing(data)){
    ys <- eval(substitute(ys), data, parent.frame())
    vars <- eval(substitute(vars), data, parent.frame())
  }
  ys <- as.matrix(ys)
  vars <- as.matrix(vars)
  n <- dim(ys)[1]
  m <- dim(ys)[2]
  if(m < 2) stop("please directly use the univariate model.")

  seed <- sample(1000000, 1)

  idx.na <- numeric(0)
  for(i in 1:n){
    if(all(is.na(ys[i,]))) idx.na <- c(idx.na, i)
  }
  if(length(idx.na) > 0){
    ys <- ys[-idx.na,]
    vars <- vars[-idx.na,]
    n <- dim(ys)[1]
  }

  y <- NULL
  v <- NULL
  n.rf <- NULL
  rf.id <- NULL
  for(i in 1:n){
    y <- c(y, ys[i,][!is.na(ys[i,])])
    v <- c(v, vars[i,][!is.na(vars[i,])])
    n.rf <- c(n.rf, sum(!is.na(ys[i,])))
    rf.id <- c(rf.id, which(!is.na(ys[i,])))
  }
  len <- length(y)
  idx.low <- cumsum(c(1, n.rf[-n]))
  idx.upp <- cumsum(n.rf)

  modelstring<-"
  model{
    for(i in 1:n){
      y[idx.low[i]:idx.upp[i]] ~ dmnorm(mu[rf.id[idx.low[i]:idx.upp[i]]], D.inv[idx.low[i]:idx.upp[i], idx.low[i]:idx.upp[i]]%*%inverse(R[rf.id[idx.low[i]:idx.upp[i]], rf.id[idx.low[i]:idx.upp[i]]])%*%D.inv[idx.low[i]:idx.upp[i], idx.low[i]:idx.upp[i]])
    }
    for(i in 1:len){
      for(j in 1:len){
        D.inv[i, j] <- ifelse(i == j, 1/sqrt((s[i])^2 + (tau[rf.id[i]])^2), 0)
      }
    }
    for(i in 1:m){
      mu[i] ~ dnorm(0, 0.001)
      tau[i] ~ dunif(0.001, 10)
    }

    R[1:m, 1:m] <- L[1:m, 1:m]%*%t(L[1:m, 1:m])
    L[1, 1] <- 1
    for(j in 2:m){
      L[1, j] <- 0
    }
    for(i in 2:(m - 1)){
      L[i, 1] <- cos(theta[i - 1, 1])
      for(j in 2:i){
        L[i, j] <- ifelse(j < i, prod(sin(theta[i - 1, 1:(j - 1)]))*cos(theta[i - 1, j]), prod(sin(theta[i - 1, 1:(i - 1)])))
      }
      for(j in (i + 1):m){
        L[i, j] <- 0
      }
    }
    L[m, 1] <- cos(theta[m - 1, 1])
    for(j in 2:(m - 1)){
      L[m, j] <- prod(sin(theta[m - 1, 1:(j - 1)]))*cos(theta[m - 1, j])
    }
    L[m, m] <- prod(sin(theta[m - 1, 1:(m - 1)]))

    for(i in 1:(m - 1)){
      for(j in 1:(m - 1)){
        theta[i, j] ~ dunif(0.01, 3.1415926 - 0.01)
      }
    }
  }"

  data.jags <- list(y = y, s = sqrt(v), n = n, len = len, m = m, rf.id = rf.id,
    idx.low = idx.low, idx.upp = idx.upp)

  init.jags <- vector("list", n.chains)
  set.seed(seed)
  init.seeds <- sample(1:100000, n.chains)
  for(i in 1:n.chains){
    init.jags[[i]] <- list(mu = rep(0, m), tau = rep(1, m), theta = matrix(3.1415926/2, m - 1, m - 1),
      .RNG.name = "base::Wichmann-Hill", .RNG.seed = init.seeds[i])
  }

  set.seed(seed)
  jags.m <- tryCatch.W.E(jags.model(file = textConnection(modelstring), data = data.jags, inits = init.jags,
    n.chains = n.chains, n.adapt = n.adapt))
  warn.adapt <- jags.m$warning
  jags.m <- jags.m$value
  if(is(warn.adapt, "warning")) cat("Adaptation incomplete; users may increase n.adapt.\n")
  update(jags.m, n.iter = n.burnin)
  jags.out <- coda.samples(model = jags.m, variable.names = c("mu", "tau", "R", "theta"),
    n.iter = n.iter, thin = n.thin)
  smry <- summary(jags.out)
  smry <- cbind(smry$statistics[,c("Mean", "SD")], smry$quantiles[,c("2.5%", "50%", "97.5%")])
  if(is.null(data.name)){
    trace.prefix <- "H_Traceplot_"
    coda.prefix <- "H_Coda_mu_"
  }else{
    trace.prefix <- paste(data.name, "_H_Traceplot_", sep = "")
    coda.prefix <- paste(data.name, "_H_Coda_mu_", sep = "")
  }

  conv.out <- gelman.diag(jags.out, multivariate = FALSE)
  conv.out <- conv.out$psrf

  out <- list(smry = smry, conv = conv.out)

  if(traceplot){
    for(i in 1:m){
      png(paste(trace.prefix, "mu[", i , "]", ".png",sep=""), res = 600, height = 8.5, width = 11, units = "in")
      par(mfrow = c(length(jags.out),1))
      for(j in 1:length(jags.out)){
        temp <- as.vector(jags.out[[j]][,paste("mu[", i, "]", sep = "")])
        plot(temp, type = "l", col = "red", ylab = paste("mu[", i, "]", sep = ""), xlab = "Iterations", main = paste("Chain",j))
      }
      dev.off()
      png(paste(trace.prefix, "tau[", i, "]", ".png",sep=""), res = 600, height = 8.5, width = 11, units = "in")
      par(mfrow = c(length(jags.out),1))
      for(j in 1:length(jags.out)){
        temp <- as.vector(jags.out[[j]][,paste("tau[", i, "]", sep = "")])
        plot(temp, type = "l", col = "red", ylab = paste("tau[", i, "]", sep = ""), xlab = "Iterations", main = paste("Chain",j))
      }
      dev.off()
    }
  }

  if(coda){
    for(i in 1:m){
      temp <- NULL
      for(j in 1:n.chains){
        temp <- c(temp, as.vector(jags.out[[j]][,paste("mu[",i,"]",sep = "")]))
      }
      write.table(matrix(temp, nrow = length(temp), ncol = 1),
        paste(coda.prefix, i, ".txt", sep = ""), row.names = FALSE, col.names = FALSE)
    }
  }

  return(out)
}

#' Alternative tests and measures for between-study inconsistency in meta-analysis
#'
#' This function jointly computes the expected gamma statistics
#' and the bootstrap-based adaptive sum of powered score (ASPU) p-values
#' for assessing between-study heterogeneity in meta-analysis.
#'
#' @param y Numeric vector of observed effect estimates, or the name of a column in \code{data}.
#' @param s2 Numeric vector of within-study variances corresponding to \code{y}, or the name of a column in \code{data}.
#' @param data Optional data frame containing the variables \code{y} and \code{s2}.
#' @param gam Numeric vector of power parameters \eqn{\gamma}, e.g., \code{1:6}.
#' @param testinf Logical; if \code{TRUE}, includes the \eqn{Q_{\infty}} statistic (maximum standardized deviation).
#' @param iter.resam Integer; number of bootstrap resamples used to estimate expected values and p-values.
#'
#' @details
#' The function generalizes the adaptive sum of powered score (ASPU) framework
#' by combining standardized powered deviations across studies.
#' Specifically, for each power parameter gamma, it calculates a statistic
#' that measures the average standardized deviation of individual study effects
#' from the overall mean, with greater gamma values placing more weight on
#' extreme deviations.
#'
#' For each gamma, the expected value of this statistic is obtained through
#' parametric bootstrap under the null hypothesis of homogeneity. The observed
#' statistic is then standardized by subtracting its expected value and dividing
#' by the observed total deviation, resulting in the expected gamma (E-gamma)
#' measure.
#'
#' In addition, a hybrid measure is computed by combining the p-values across
#' all gamma values using the minimum-p combination rule. This hybrid measure
#' provides an overall summary of between-study inconsistency that integrates
#' evidence from multiple power levels.
#'
#' @return A list named \code{out} containing:
#' \itemize{
#'   \item \code{E_gamma}: Vector of E-gamma statistics for each \eqn{\gamma}, including \eqn{E_{\infty}} and the hybrid \eqn{E_{Lhyb}} term.
#'   \item \code{p_gamma}: Bootstrap p-values corresponding to each \eqn{\gamma}.
#'   \item \code{p_hyb}: Hybrid p-value combining all \eqn{\gamma} values.
#'   \item \code{gam}: Vector of power parameters used in the test.
#'   \item \code{table}: Combined summary table for reporting.
#' }
#'
#' @examples
#' data("dat.adams")
#' set.seed(123)
#' metahet.hybrid(y, s2, data = dat.adams, gam = 1:8,
#'                       testinf = TRUE, iter.resam = 200)
#'
#' @keywords heterogeneity
#' @importFrom stats rnorm
#' @export
metahet.hybrid <- function(y, s2, data, gam, testinf = TRUE, iter.resam = 200) {
  # --- input checks ---
  if (missing(y)) stop("Please specify the effect size variable 'y'.")
  if (missing(s2)) stop("Please specify the within-study variance variable 's2'.")
  if (!missing(data)) {
    y  <- eval(substitute(y),  data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
  }
  if (length(y) != length(s2) || any(s2 <= 0))
    stop("Input data error: 'y' and 's2' must have equal length and s2 > 0.")
  
  # --- compute weighted mean under homogeneity ---
  n <- length(y)
  mu_ce <- sum(y / s2) / sum(1 / s2)
  
  # --- compute Q_gamma statistics ---
  N_gam <- length(gam)
  Q <- sapply(gam, function(g) sum((sqrt(1 / s2) * abs(y - mu_ce))^g))
  if (testinf) {
    Q <- c(Q, max(sqrt(1 / s2) * abs(y - mu_ce)))
    N_gam <- N_gam + 1
  }
  
  # --- parametric bootstrap ---
  Q.resam <- matrix(NA, iter.resam, N_gam)
  for (b in 1:iter.resam) {
    s2.temp <- sample(s2, n, replace = TRUE)
    y.temp  <- rnorm(n, mean = mu_ce, sd = sqrt(s2.temp))
    mu.temp <- sum(y.temp / s2.temp) / sum(1 / s2.temp)
    Q.temp  <- sapply(gam, function(g) sum((sqrt(1 / s2.temp) * abs(y.temp - mu.temp))^g))
    if (testinf) Q.temp <- c(Q.temp, max(sqrt(1 / s2.temp) * abs(y.temp - mu.temp)))
    Q.resam[b, ] <- Q.temp
  }
  
  # --- E_gamma ---
  EQ <- colMeans(Q.resam, na.rm = TRUE)
  E_gamma <- pmax(0, Q - EQ) / Q
  
  # --- empirical p-values ---
  p_gamma <- sapply(1:N_gam, function(i)
    (sum(Q.resam[, i] >= Q[i], na.rm = TRUE) + 1) / (iter.resam + 1))
  p_gammab <- matrix(NA, iter.resam, N_gam)
  for (b in 1:iter.resam) {
    for (i in 1:N_gam) {
      p_gammab[b, i] <- (sum(Q.resam[-b, i] >= Q.resam[b, i], na.rm = TRUE) + 1) / iter.resam
    }
  }
  Qhyb <- suppressWarnings(min(p_gamma, na.rm = TRUE))
  Qhyb_b <- apply(p_gammab, 1, min, na.rm = TRUE)
  p_hyb <- (sum(Qhyb_b <= Qhyb, na.rm = TRUE) + 1) / (iter.resam + 1)
  
  # --- E_hyb (hybrid E_gamma) ---
  Lhyb <- -log10(Qhyb)
  Lhyb_b <- -log10(Qhyb_b)
  E_Lhyb <- mean(Lhyb_b, na.rm = TRUE)
  E_hyb <- max(0, Lhyb - E_Lhyb) / Lhyb
  
  # --- label construction ---
  labels <- c(as.character(gam), if (testinf) "Inf", "Hybrid")
  Evals  <- c(E_gamma, E_hyb)
  Pvals  <- c(p_gamma, p_hyb)
  
  # --- summary table ---
  table_out <- data.frame(
    gamma   = labels,
    E_gamma = round(Evals, 4),
    p_gamma = round(Pvals, 4),
    stringsAsFactors = FALSE
  )
  
  # --- output ---
  out <- list(
    E_gamma = E_gamma,
    E_hyb   = E_hyb,
    p_gamma = p_gamma,
    p_hyb   = p_hyb,
    gam     = gam,
    table   = table_out
  )
  class(out) <- "metahet.hybrid"
  return(out)
}

#' @export
print.metahet.hybrid <- function(x, digits = 4, ...) {
  cat("\nAlternative tests results and measures for heterogeneity\n")
  if (!is.null(x$table) && nrow(x$table) > 0) {
    print(x$table, row.names = FALSE, right = TRUE)
  } else {
    cat("No valid results.\n")
  }
  invisible(x)
}

pb.inv.sqrt.n <- function(y, s2 = NA, n, n00 = NA, n01 = NA, n10 = NA, n11 = NA){
  x.reg <- 1/sqrt(n)
  y.reg <- y
  w.reg <- n
  if(all(abs(diff(x.reg[!is.na(x.reg)])) < 1e-5) | all(abs(diff(y.reg[!is.na(y.reg)])) < 1e-5)){
    out <- list(pval = 1, stat = 0, coef = c(0, 0))
  }else{
    out <- lm(y.reg ~ x.reg, weights = w.reg)
    out <- summary(out)$coefficients
    pval <- out["x.reg", "Pr(>|t|)"]
    coef <- as.numeric(out[,1])
    out <- list(pval = pval, stat = as.numeric(out["x.reg", "t value"]), coef = coef)
  }
  return(out)
}

meta.pen <- function(y, s2, data, tuning.para = "tau", upp = 1, n.cand = 100, tol=1e-10){
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
  }
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  if (!is.element(tuning.para, c("tau", "lambda"))) stop("the tuning parameter must be specified as 'tau' or 'lambda'.")
  n <- length(y)
  w <- 1/s2
  y.bar <- sum(w*y)/sum(w)
  Q <- sum(w*(y - y.bar)^2)
  if(Q < n - 1) Q <- n - 1
  I2 <- (Q - (n - 1))/Q 
  
  tau2.re <- meta.ml(y, s2, tol) ## The ML estimate of the between-study variance, other methods can be used to calculate the estimate
  mu.fe <- y.bar
  se.fe <- sqrt(1/sum(w))
  w.re <- 1/(s2 + tau2.re)
  mu.re <- sum(w.re*y)/sum(w.re)
  se.re <- sqrt(1/sum(w.re))
  
  if(tuning.para == "tau"){
    tau.cand <- seq(from = 0, to = sqrt(upp*tau2.re), length.out = n.cand)
    tau2.cand <- tau.cand^2
    errs <- matrix(0, n, n.cand)
    for (i in 1:n) {
      y.train <- y[-i]
      s2.train <- s2[-i]
      tau2.re.train <- meta.ml(y.train, s2.train, tol)
      for (j in 1:n.cand) {
        tau2.temp <- tau2.cand[j]
        w.temp <- 1/(s2.train + tau2.temp)
        mu_i <- sum(w.temp*y.train)/sum(w.temp)
        var_i <- sum(w.temp^2*(s2.train + tau2.re.train))/(sum(w.temp))^2
        errs[i, j] <- (mu_i - y[i])^2/(var_i + s2[i] + tau2.re.train)
      }
    }
    loss <- sqrt(colMeans(errs))
    opt.idx <- which(loss == min(loss))[1] ## Use the first value to achieve minimization
    tau2.opt <- tau2.cand[opt.idx]
    tau.opt <- tau.cand[opt.idx]
    w.opt <- 1/(s2 + tau2.opt)
    mu.opt <- sum(w.opt*y)/sum(w.opt)
    var.opt <- sum(w.opt^2*(s2 + tau2.re))/(sum(w.opt))^2
    
    out <- NULL
    out$n.study <- n
    out$I2 <- I2
    out$tau2.re <- tau2.re
    out$mu.fe <- mu.fe
    out$se.fe <- se.fe
    out$mu.re <- mu.re
    out$se.re <- se.re
    out$loss <- loss
    out$tau.cand <- tau.cand
    out$tau.opt <- tau.opt ## The estimated between-study standard deviation by tuning tau
    out$mu.opt <- mu.opt  ## The estimated overall effect size by tuning tau
    out$se.opt <- sqrt(var.opt)  ## The standard error of the overall effect size estimate by tuning tau
    return(out)
  }
  
  if(tuning.para == "lambda"){
    lambda <- determine.lambda(y, s2, tau2.re, penalty = "tau2", n.lambda = n.cand, lambda.scale = "log", tol = tol, lam.c = upp)
    
    diff <- matrix(NA, n.cand, n)
    for (i in 1:n) {
      y_i <- y[-i]
      s2_i <- s2[-i]
      w_i <- 1/s2_i
      tau2.re_i <- meta.ml(y_i, s2_i, tol)
      out <- metapen.lambda(y_i, s2_i, penalty = "tau2", lambda = lambda, tol = tol)
      mu_i <- out[, "mu"]
      tau2_i <- out[, "tau2"]
      mu.hat.var <- lapply(X = 1:n.cand, FUN = function(lam) {
        sum((s2_i + tau2.re_i)/(s2_i + tau2_i[lam])^2)/(sum(1/(s2_i + tau2_i[lam])))^2
      })
      mu.hat.var <- unlist(mu.hat.var)
      diffi <- (y[i] - mu_i)^2/(s2[i] + tau2.re_i + mu.hat.var)
      diff[, i] <- diffi
    }

    full <- metapen.lambda(y, s2, penalty = "tau2", lambda = lambda, tol = tol)
    mu <- full[, "mu"]
    tau2 <- full[, "tau2"]
    tau <- sqrt(tau2)

    loss <- sqrt(apply(diff, 1, mean))
    opt.idx <- which(loss == min(loss))[1]
    tau2.opt <- tau2[opt.idx]
    tau.opt <- tau[opt.idx]
    w.opt <- 1/(s2 + tau2.opt)
    mu.opt <- sum(w.opt*y)/sum(w.opt)
    var.opt <- sum(w.opt^2*(s2 + tau2.re))/(sum(w.opt))^2

    out <- NULL
    out$n.study <- n
    out$I2 <- I2
    out$tau2.re <- tau2.re
    out$mu.fe <- mu.fe
    out$se.fe <- se.fe
    out$mu.re <- mu.re
    out$se.re <- se.re
    out$loss <- loss
    out$lambda.cand <- lambda
    out$tau.opt <- tau.opt ## The estimated between-study standard deviation by tuning lambda
    out$mu.opt <- mu.opt  ## The estimated overall effect size by tuning lambda
    out$se.opt <- sqrt(var.opt)  ## The standard error of the overall effect size estimate by tuning lambda
    return(out)
  }
}

pb.skew <- function(y, s2, n = NA, n00 = NA, n01 = NA, n10 = NA, n11 = NA){
  std.res <- pb.reg(y, s2)$std.res
  cm2 <- var(std.res)
  cm3 <- mean((std.res - mean(std.res))^3)
  skewness <- cm3/(cm2^(1.5))
  pval <- 2*pnorm(-sqrt(length(y)/6)*abs(skewness))
  out <- list(pval = as.numeric(pval), stat = as.numeric(skewness))
  return(out)
}

maprop.glmm <- function(e, n, data, link = "logit",
  alpha = 0.05, pop.avg = TRUE, int.approx = 10000, b.iter = 1000,
  seed = 1234, ...){
  if(missing(e)) stop("need to specify event counts.")
  if(missing(n)) stop("need to specify sample sizes.")
  if(!missing(data)){
    e <- eval(substitute(e), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if(length(e) != length(n)) stop("lengths of e and n differ.")
  if(!is.element(link, c("log", "logit", "probit", "cauchit", "cloglog"))){
    stop("cannot recognize the link function.")
  }
  if(alpha < 0 | alpha > 1) stop("alpha should be between 0 and 1.")
  N <- length(e)
  data <- data.frame(sid = 1:N, e = e, n = n)
  rslt <- glmer(cbind(e, n - e) ~ 1 + (1 | sid),
    data = data, family = binomial(link = link), ...)
  mu.est <- summary(rslt)$coefficients[1]
  mu.se <- summary(rslt)$coefficients[2]
  mu.ci <- c(mu.est - qnorm(1 - alpha/2) * mu.se,
    mu.est + qnorm(1 - alpha/2) * mu.se)
  tau.est <- sqrt(as.numeric(summary(rslt)$varcor))
  AICtab <- summary(rslt)$AICtab
  if(link == "log"){
    back.trans <- function(x) exp(x)
  }
  if(link == "logit"){
    back.trans <- function(x) plogis(x)
  }
  if(link == "probit"){
    back.trans <- function(x) pnorm(x)
  }
  if(link == "cauchit"){
    back.trans <- function(x) pcauchy(x)
  }
  if(link == "cloglog"){
    back.trans <- function(x) 1 - exp(-exp(x))
  }
  back.trans <- Vectorize(back.trans)
  prop.c.est <- back.trans(mu.est)
  prop.c.ci <- back.trans(mu.ci)
  out <- list(prop.c.est = prop.c.est, prop.c.ci = prop.c.ci, AICtab = AICtab)
  if(pop.avg){
    if(link == "probit"){
      prop.m.est <- pnorm(mu.est/sqrt(1 + tau.est^2))
    }else{
      set.seed(seed)
      stdnorms <- rnorm(int.approx)
      prop.m.est <- mean(back.trans(mu.est + tau.est * stdnorms))
    }
    mu.est.b <- prop.m.est.b <- rep(NA, b.iter)
    set.seed(seed)
    b.idx <- 0
    b.w <- b.e <- 0
    while(b.idx < b.iter){
      idx.temp <- sample(N, replace = TRUE)
      data.temp <- data[idx.temp,]
      data.temp$sid <- 1:N
      suppressMessages(out.W.E <- tryCatch.W.E(
        rslt.temp <- glmer(cbind(e, n - e) ~ 1 + (1 | sid),
          data = data.temp, family = binomial(link = link), ...)))
      if(is.null(out.W.E$warning) & !inherits(out.W.E$value, "error")){
        b.idx <- b.idx + 1
        mu.est.b[b.idx] <- summary(rslt.temp)$coefficients[1]
        tau.est.temp <- sqrt(as.numeric(summary(rslt.temp)$varcor))
        if(link == "probit"){
          prop.m.est.b[b.idx] <-
            pnorm(mu.est.b[b.idx]/sqrt(1 + tau.est.temp^2))
        }else{
          prop.m.est.b[b.idx] <- mean(back.trans(mu.est.b[b.idx] +
            tau.est.temp * stdnorms))
        }
      }
      if(!is.null(out.W.E$warning)){
        b.w <- b.w + 1
      }
      if(inherits(out.W.E$value, "error")){
        b.e <- b.e + 1
      }
    }
    mu.ci.b <- quantile(mu.est.b, probs = c(alpha/2, 1 - alpha/2))
    names(mu.ci.b) <- NULL
    prop.c.ci.b <- back.trans(mu.ci.b)
    prop.m.ci.b <- quantile(prop.m.est.b, probs = c(alpha/2, 1 - alpha/2))
    names(prop.m.ci.b) <- NULL
    out <- c(out, list(prop.c.ci.b = prop.c.ci.b,
      prop.m.est = prop.m.est, prop.m.ci.b = prop.m.ci.b,
      b.w.e = c("bootstrap warnings" = b.w, "bootstrap errors" = b.e)))
  }
  return(out)
}

ssfunnel.lrr <- function(y, s2, n, alpha, log.ss, p0, xlim, ylim, xlab, ylab, cols.contour, cols, cex.pts, lwd.contour, pch, ...){
  plot.default(y, n, xlim = xlim, ylim = ylim,
    type = "n", log = ifelse(log.ss, "y", ""),
    xlab = xlab, ylab = ylab, ...)
  points(y, n, pch = pch, col = cols, cex = cex.pts)
  if(length(alpha) > 0){
    for(i in 1:length(alpha)){
      contour <- function(x){
        if(is.na(p0)){
          qnorm(alpha[i]/2)^2*(exp(abs(x)) - 1)/x^2
        }else{
          qnorm(alpha[i]/2)^2*(sqrt(1/p0 - 1) + sqrt(exp(-x)/p0 - 1))^2/x^2
        }
      }
      contour <- Vectorize(contour)
      if(is.na(p0)){
        curve(contour, from = -max(abs(y)) - 1, to = max(abs(y)) + 1, add = TRUE, col = cols.contour[i], lwd = lwd.contour)
      }else{
        curve(contour, from = -max(abs(y)) - 1, to = min(max(abs(y)) + 1, -log(p0)),
          add = TRUE, col = cols.contour[i], lwd = lwd.contour)
      }
    }
  }
}

meta.dt <- function(tp, fp, fn, tn, data, method = "biv.glmm", alpha = 0.05, ...){
  if(missing(tp)) stop("need to specify counts of true positives.")
  if(missing(fp)) stop("need to specify counts of false positives.")
  if(missing(fn)) stop("need to specify counts of false negatives.")
  if(missing(tn)) stop("need to specify counts of true negatives.")
  if(!missing(data)){
    tp <- eval(substitute(tp), data, parent.frame())
    fp <- eval(substitute(fp), data, parent.frame())
    fn <- eval(substitute(fn), data, parent.frame())
    tn <- eval(substitute(tn), data, parent.frame())
  }
  if((length(tp) != length(fp)) | (length(tp) != length(fn)) | (length(tp) != length(tn))){
    stop("the input data have different dimensions.")
  }
  if(!is.element(method, c("s.roc", "biv.lmm", "biv.glmm"))){
    stop("do not recognize the specified method.")
  }
  
  idx.allzero <- which(tp == 0 & fp == 0 & fn == 0 & tn == 0)
  if(length(idx.allzero) > 0){
    tp <- tp[-idx.allzero]
    fp <- fp[-idx.allzero]
    fn <- fn[-idx.allzero]
    tn <- tn[-idx.allzero]
    message("studies with zero counts in all four data cells are removed.")
  }
  if(length(tp) < 2) stop("less than two studies in the meta-analysis.")
  dat.ori <- data.frame(tp = tp, fp = fp, fn = fn, tn = tn)
  n.stu <- length(tp)
  sid <- rep(1:n.stu, each = 2)
  sens.id <- rep(c(1, 0), n.stu)
  
  AUC_int <- function(roc){
    tryCatch(
      {
        AUC <- integrate(roc, lower = 0, upper = 1)$value
        return(AUC)
      },
      error=function(error_message) {
        tryCatch(
          {
            AUC <- integrate(roc, lower = 0.0001, upper = 0.9999)$value
            return(AUC)
          },
          error=function(error_message) {
            tryCatch(
              {
                AUC <- integrate(roc, lower = 0.001, upper = 0.999)$value
                return(AUC)
              },
              error=function(error_message) {
                tryCatch(
                  {
                    AUC <- integrate(roc, lower = 0.01, upper = 0.99)$value
                    return(AUC)
                  },
                  error=function(error_message) {
                    message("The estimated ROC is not integrable.")
                    return(0)
                  }
                )
              }
            )
          }
        )
      }
    )
  }
  
  if(method == "s.roc"){
    for(i in 1:n.stu){
      if(tp[i] == 0 | fp[i] == 0 | fn[i] == 0 | tn[i] == 0){
        tp[i] <- tp[i] + 0.5
        fp[i] <- fp[i] + 0.5
        fn[i] <- fn[i] + 0.5
        tn[i] <- tn[i] + 0.5
      }
    }
    sens <- tp/(tp + fn)
    spec <- tn/(fp + tn)
    y1 <- log(sens/(1 - sens))
    y2 <- log(spec/(1 - spec))
    v1 <- 1/tp + 1/fn
    v2 <- 1/fp + 1/tn
    D <- y1 + y2
    S <- y1 - y2
    V <- v1 + v2
    
    lm.unwtd <- lm(D ~ S, ...)
    smry.unwtd <- summary(lm.unwtd)
    inter.unwtd <- smry.unwtd$coefficients[1,1]
    slope.unwtd <- smry.unwtd$coefficients[2,1]
    vcov.unwtd <- vcov(lm.unwtd)
    logDOR.meanS.unwtd <- inter.unwtd + slope.unwtd*mean(S)
    temp.coef <- c(1, mean(S))
    logDOR.meanS.unwtd.var <- as.numeric(t(temp.coef) %*% vcov.unwtd %*% temp.coef)
    logDOR.meanS.unwtd.ci <- c(logDOR.meanS.unwtd - qnorm(1 - alpha/2)*sqrt(logDOR.meanS.unwtd.var),
                               logDOR.meanS.unwtd + qnorm(1 - alpha/2)*sqrt(logDOR.meanS.unwtd.var))
    Q.unwtd <- 1/(1 + exp(-inter.unwtd/2))
    inter.unwtd.ci <- c(inter.unwtd - qnorm(1 - alpha/2)*sqrt(vcov.unwtd[1,1]),
                        inter.unwtd + qnorm(1 - alpha/2)*sqrt(vcov.unwtd[1,1]))
    Q.unwtd.ci <- 1/(1 + exp(-inter.unwtd.ci/2))
    roc.curve.unwtd <- function(xval){
      spec <- 1 - xval
      sens <- 1/(1 + exp(-inter.unwtd/(1 - slope.unwtd))*(spec/(1 - spec))^((1 + slope.unwtd)/(1 - slope.unwtd)))
      return(sens)
    }
    AUC.unwtd <- AUC_int(roc.curve.unwtd)
    
    lm.wtd <- lm(D ~ S, weights = 1/V, ...)
    smry.wtd <- summary(lm.wtd)
    inter.wtd <- smry.wtd$coefficients[1,1]
    slope.wtd <- smry.wtd$coefficients[2,1]
    vcov.wtd <- vcov(lm.wtd)
    logDOR.meanS.wtd <- inter.wtd + slope.wtd*mean(S)
    logDOR.meanS.wtd.var <- as.numeric(t(temp.coef) %*% vcov.wtd %*% temp.coef)
    logDOR.meanS.wtd.ci <- c(logDOR.meanS.wtd - qnorm(1 - alpha/2)*sqrt(logDOR.meanS.wtd.var),
                             logDOR.meanS.wtd + qnorm(1 - alpha/2)*sqrt(logDOR.meanS.wtd.var))
    Q.wtd <- 1/(1 + exp(-inter.wtd/2))
    inter.wtd.ci <- c(inter.wtd - qnorm(1 - alpha/2)*sqrt(vcov.wtd[1,1]),
                      inter.wtd + qnorm(1 - alpha/2)*sqrt(vcov.wtd[1,1]))
    Q.wtd.ci <- 1/(1 + exp(-inter.wtd.ci/2))
    roc.curve.wtd <- function(xval){
      spec <- 1 - xval
      sens <- 1/(1 + exp(-inter.wtd/(1 - slope.wtd))*(spec/(1 - spec))^((1 + slope.wtd)/(1 - slope.wtd)))
      return(sens)
    }
    AUC.wtd <- AUC_int(roc.curve.wtd)
    
    out <- list(method = method, alpha = alpha,
                inter.unwtd = inter.unwtd, slope.unwtd = slope.unwtd, vcov.unwtd = vcov.unwtd,
                DOR.meanS.unwtd = exp(logDOR.meanS.unwtd), DOR.meanS.unwtd.ci = exp(logDOR.meanS.unwtd.ci),
                Q.unwtd = Q.unwtd, Q.unwtd.ci = Q.unwtd.ci, AUC.unwtd = AUC.unwtd,
                inter.wtd = inter.wtd, slope.wtd = slope.wtd, vcov.wtd = vcov.wtd,
                DOR.meanS.wtd = exp(logDOR.meanS.wtd), DOR.meanS.wtd.ci = exp(logDOR.meanS.wtd.ci),
                Q.wtd = Q.wtd, Q.wtd.ci = Q.wtd.ci, AUC.wtd = AUC.wtd, data = dat.ori)
  }
  
  if(is.element(method, c("biv.lmm", "biv.glmm"))){
    if(method == "biv.lmm"){
      for(i in 1:n.stu){
        if(tp[i] == 0 | fp[i] == 0 | fn[i] == 0 | tn[i] == 0){
          tp[i] <- tp[i] + 0.5
          fp[i] <- fp[i] + 0.5
          fn[i] <- fn[i] + 0.5
          tn[i] <- tn[i] + 0.5
        }
      }
      sens <- tp/(tp + fn)
      spec <- tn/(fp + tn)
      y1 <- log(sens/(1 - sens))
      y2 <- log(spec/(1 - spec))
      y <- c(rbind(y1, y2))
      v1 <- 1/tp + 1/fn
      v2 <- 1/fp + 1/tn
      v <- c(rbind(v1, v2))
      dat <- data.frame(sid = sid, sens.id = factor(sens.id), y = y, v = v)
      rslt <- rma.mv(yi = y, V = v, mods = ~ sens.id, random = ~ sens.id | sid, struct = "HCS", data = dat, ...)
      mu.sens <- rslt$beta[1,1] + rslt$beta[2,1]
      mu.spec <- rslt$beta[1,1]
      names(mu.sens) <- names(mu.spec) <- NULL
      temp.vcov <- rslt$vb
      rownames(temp.vcov) <- colnames(temp.vcov) <- NULL
      C.mat <- matrix(c(1, 1, 1, 0), 2, 2)
      mu.vcov <- C.mat %*% temp.vcov %*% t(C.mat)
      sig.sens <- sqrt(rslt$tau2[2])
      sig.spec <- sqrt(rslt$tau2[1])
      rho <- rslt$rho
    }
    
    if(method == "biv.glmm"){
      e <- c(rbind(tp, tn))
      tot.p <- tp + fn
      tot.n <- fp + tn
      tot <- c(rbind(tot.p, tot.n))
      dat <- data.frame(sid = sid, sens.id = sens.id, spec.id = 1 - sens.id, e = e, tot = tot)
      rslt <- glmer(cbind(e, tot - e) ~ sens.id + spec.id - 1 + (sens.id + spec.id - 1 | sid),
                    data = dat, family = binomial(link = "logit"), ...)
      smry <- summary(rslt)
      mu.sens <- smry$coefficients[1,1]
      mu.spec <- smry$coefficients[2,1]
      mu.vcov <- matrix(as.numeric(smry$vcov), 2, 2)
      sig.sens <- attr(smry$varcor[[1]], "stddev")[1]
      if (sig.sens == 0){
        sig.sens <- 0.0001
      }
      sig.spec <- attr(smry$varcor[[1]], "stddev")[2]
      rho <- attr(smry$varcor[[1]], "correlation")[1,2]
      if (is.nan(rho)){
        rho <- 0
      }
      names(sig.sens) <- names(sig.spec) <- NULL
    }
    
    sens.overall <- 1/(1 + exp(-mu.sens))
    mu.sens.ci <- c(mu.sens - qnorm(1 - alpha/2)*sqrt(mu.vcov[1,1]),
                    mu.sens + qnorm(1 - alpha/2)*sqrt(mu.vcov[1,1]))
    sens.overall.ci <- 1/(1 + exp(-mu.sens.ci))
    spec.overall <- 1/(1 + exp(-mu.spec))
    mu.spec.ci <- c(mu.spec - qnorm(1 - alpha/2)*sqrt(mu.vcov[2,2]),
                    mu.spec + qnorm(1 - alpha/2)*sqrt(mu.vcov[2,2]))
    spec.overall.ci <- 1/(1 + exp(-mu.spec.ci))
    logDOR.overall <- mu.sens + mu.spec
    logDOR.overall.stderr <- sqrt(sum(mu.vcov))
    logDOR.overall.ci <- c(logDOR.overall - qnorm(1 - alpha/2)*logDOR.overall.stderr,
                           logDOR.overall + qnorm(1 - alpha/2)*logDOR.overall.stderr)
    DOR.overall <- exp(logDOR.overall)
    DOR.overall.ci <- exp(logDOR.overall.ci)
    roc.curve <- function(xval){
      spec <- 1 - xval
      logit.spec <- log(spec/(1 - spec))
      logit.sens <- mu.sens + rho*sig.sens/sig.spec*(logit.spec - mu.spec)
      sens <- 1/(1 + exp(-logit.sens))
      return(sens)
    }
    AUC <- AUC_int(roc.curve)
    
    out <- list(method = method, alpha = alpha,
                sens.overall = sens.overall, sens.overall.ci = sens.overall.ci,
                spec.overall = spec.overall, spec.overall.ci = spec.overall.ci,
                DOR.overall = DOR.overall, DOR.overall.ci = DOR.overall.ci, AUC = AUC,
                mu.sens = mu.sens, mu.spec = mu.spec, mu.vcov = mu.vcov,
                sig.sens = sig.sens, sig.spec = sig.spec, rho = rho, data = dat.ori)
  }
  
  class(out) <- "meta.dt"
  return(out)
}

mvma.fe <- function(ys, covs, tol = 1E-10){
  if(missing(ys)) stop("the argument ys is missing.")
  if(missing(covs)) stop("the argument covs is missing.")
  ys <- as.matrix(ys)
  n <- dim(ys)[1]
  p <- dim(ys)[2]
  if(p < 2) stop("please directly use the univariate model.")

  idx.na <- numeric(0)
  for(i in 1:n){
    if(all(is.na(ys[i,]))) idx.na <- c(idx.na, i)
  }
  if(length(idx.na) > 0){
    ys <- ys[-idx.na,]
    covs <- covs[-idx.na]
    n <- dim(ys)[1]
  }

  idx.obs <- vector("list", n)
  X <- vector("list", n)
  ys.tilde <- vector("list", n)
  covs.tilde <- vector("list", n)
  for(i in 1:n){
    idx.obs[[i]] <- which(!is.na(ys[i,]))
    X[[i]] <- diag(p)[idx.obs[[i]],]
    if(length(idx.obs[[i]]) == 1) X[[i]] <- matrix(X[[i]], nrow = 1, ncol = p)
    ys.tilde[[i]] <- ys[i, idx.obs[[i]]]
    covs.tilde[[i]] <- as.matrix(covs[[i]][idx.obs[[i]], idx.obs[[i]]])
  }

  target <- function(x){
    mu <- x[1:p]
    Tau <- matrix(0, nrow = p, ncol = p)

    out1 <- 0
    for(i in 1:n){
      cov.temp <- covs.tilde[[i]] + Tau[idx.obs[[i]], idx.obs[[i]]]
      y.temp <- ys.tilde[[i]]
      y_mu <- y.temp - mu[idx.obs[[i]]]
      y_mu <- as.matrix(y_mu)
      logdet <- determinant(as.matrix(cov.temp), logarithm = TRUE)
      if(logdet$sign <= 0) logdet <- -Inf else logdet <- logdet$modulus
      temp <- logdet + as.numeric(t(y_mu)%*%solve(cov.temp)%*%y_mu)
      out1 <- out1 + temp
    }
    out <- as.numeric(out1)
    return(out)
  }

  inits <- rep(0, p)
  low <- rep(-Inf, p)
  upp <- rep(Inf, p)
  optim.out <- optim(par = inits, fn = target, lower = low, upper = upp, method = "L-BFGS-B", control = list(factr = tol))
  pars <- optim.out$par

  mu.est <- pars[1:p]
  Tau.est <- matrix(0, nrow = p, ncol = p)

  temp <- matrix(0, p, p)
  for(i in 1:n){
    cov.temp <- covs.tilde[[i]] + Tau.est[idx.obs[[i]], idx.obs[[i]]]
    if(length(idx.obs[[i]]) == 1){
      cov.temp <- as.numeric(cov.temp)
      temp <- temp + t(X[[i]])%*%X[[i]]/cov.temp
    }else{
      temp <- temp + t(X[[i]])%*%solve(cov.temp)%*%X[[i]]
    }
  }
  mu.cov <- solve(temp)

  out <- list(mu.est = mu.est, mu.cov = mu.cov, method = "Fixed-effects model")
  return(out)
}

## JAGS model for FE NMA with evidence consistency
nma.fe.c <- function(o){
out <- "
model{
  for(i in 1:NS){
    mu[i] ~ dnorm(0, 0.0001)  # vague priors for trial baselines
    for(k in 1:na[i]){
      r[i,k] ~ dbin(p[i,t[i,k]], n[i,k])  # binomial likelihood
      logit(p[i,t[i,k]]) <- mu[i] + md[i,t[i,k]]  # model
      rhat[i,k] <- p[i,t[i,k]]*n[i,k] # expected counts
      dev[i,k] <- 2*(r[i,k]*(log(r[i,k]) - log(rhat[i,k])) +
        (n[i,k] - r[i,k])*(log(n[i,k] - r[i,k]) - log(n[i,k] - rhat[i,k])))
                                      # deviance contribution
    }
    resdev[i] <- sum(dev[i,1:na[i]]) # residual deviance for this trial
    md[i,t[i,1]] <- 0
    for(k in 2:na[i]){
      md[i,t[i,k]] <- d[t[i,k]] - d[t[i,1]]  # LOR
    }
  }
  totresdev <- sum(resdev[])  # total residual deviance

  d[1] <- 0
  for(k in 2:NT){
    d[k] ~ dnorm(0, 0.0001)
  }

  # pairwise ORs
  for(c in 1:(NT - 1)){
    for(k in (c + 1):NT){
      lor[c,k] <- d[k] - d[c]
    }
  }
}"
return(out)
}

## JAGS model for RE NMA with evidence inconsistency
nma.re.ic <- function(o){
out <- "
model{
  for(i in 1:NS){
    delta[i,1] <- 0
    mu[i] ~ dnorm(0, 0.0001)  # vague priors for trial baselines
    for(k in 1:na[i]){
      r[i,k] ~ dbin(p[i,k], n[i,k])  # binomial likelihood
      logit(p[i,k]) <- mu[i] + delta[i,k]  # model
      rhat[i,k] <- p[i,k]*n[i,k] # expected counts
      dev[i,k] <- 2*(r[i,k]*(log(r[i,k]) - log(rhat[i,k])) +
        (n[i,k] - r[i,k])*(log(n[i,k] - r[i,k]) - log(n[i,k] - rhat[i,k])))
                                      # deviance contribution
    }
    resdev[i] <- sum(dev[i,1:na[i]]) # residual deviance for this trial
    for(k in 2:na[i]){
      delta[i,k] ~ dnorm(lor[t[i,1],t[i,k]], prec)  # LOR
    }
  }
  totresdev <- sum(resdev[])  # total residual deviance

  for(c in 1:(NT - 1)){
    for(k in (c + 1):NT){
      lor[c,k] ~ dnorm(0, 0.0001)
    }
  }

  prec <- 1/tau^2
  tau ~ dunif(0, 5)  #  prior for heterogeneity standard deviation
}"
return(out)
}

## adapted from the package "metafor"
pb.trimfill <- function (x, side, estimator = "L0", maxiter = 100, verbose = FALSE, ...){
  if (!is.element("rma.uni", class(x))) 
    stop("Argument 'x' must be an object of class \"rma.uni\".")
  if (!x$int.only) 
    stop("Trim-and-fill method only applicable for models without moderators.")
  if (missing(side)) 
    side <- NULL
  estimator <- match.arg(estimator, c("L0", "R0", "Q0"))
  if (x$k == 1) 
    stop("Stopped because k = 1.")
  yi <- x$yi
  vi <- x$vi
  weights <- x$weights
  ni <- x$ni
  if (is.null(side)) {
    args <- list(yi, vi, weights = weights, mods = sqrt(vi), intercept = TRUE, method = x$method, weighted = x$weighted, ...)
    args <- args[!sapply(args, is.null)]
    res <- do.call(rma, args)
#    res <- rma(yi, vi, weights = weights, mods = sqrt(vi), 
#      intercept = TRUE, method = x$method, weighted = x$weighted, ...)
    if (is.na(res$b[2])) res$b[2] <- 0
    if (res$b[2] < 0) {
      side <- "right"
    }
    else {
      side <- "left"
    }
  }
  else {
    side <- match.arg(side, c("left", "right"))
  }
  if (side == "right") {
    yi <- -1 * yi
  }
  idix <- sort(yi, index.return = TRUE)$ix
  yi <- yi[idix]
  vi <- vi[idix]
  weights <- weights[idix]
  ni <- ni[idix]
  k <- length(yi)
  k0.sav <- -1
  k0 <- 0
  iter <- 0
  while (abs(k0 - k0.sav) > 0 & iter <= maxiter) {
    k0.sav <- k0
    iter <- iter + 1
#   if (iter > maxiter) 
#     warning("Trim and fill algorithm did not converge.")
    yi.t <- yi[1:(k - k0)]
    vi.t <- vi[1:(k - k0)]
    weights.t <- weights[1:(k - k0)]
    args <- list(yi = yi.t, vi = vi.t, weights = weights.t, intercept = TRUE, method = x$method, weighted = x$weighted, ...)
    args <- args[!sapply(args, is.null)]
    res <- do.call(rma, args)
#    res <- rma(yi.t, vi.t, weights = weights.t, intercept = TRUE, 
#      method = x$method, weighted = x$weighted, ...)
    b <- c(res$b)
    yi.c <- yi - b
    yi.c.r <- rank(abs(yi.c), ties.method = "first")
    yi.c.r.s <- sign(yi.c) * yi.c.r
    if (estimator == "R0") {
      k0 <- (k - max(-1 * yi.c.r.s[yi.c.r.s < 0])) - 1
      se.k0 <- sqrt(2 * max(0, k0) + 2)
    }
    if (estimator == "L0") {
      Sr <- sum(yi.c.r.s[yi.c.r.s > 0])
      k0 <- (4 * Sr - k * (k + 1))/(2 * k - 1)
      varSr <- 1/24 * (k * (k + 1) * (2 * k + 1) + 10 * 
        k0^3 + 27 * k0^2 + 17 * k0 - 18 * k * k0^2 - 
        18 * k * k0 + 6 * k^2 * k0)
      se.k0 <- 4 * sqrt(varSr)/(2 * k - 1)
    }
    if (estimator == "Q0") {
      Sr <- sum(yi.c.r.s[yi.c.r.s > 0])
      k0 <- k - 1/2 - sqrt(2 * k^2 - 4 * Sr + 1/4)
      varSr <- 1/24 * (k * (k + 1) * (2 * k + 1) + 10 * 
        k0^3 + 27 * k0^2 + 17 * k0 - 18 * k * k0^2 - 
        18 * k * k0 + 6 * k^2 * k0)
      se.k0 <- 2 * sqrt(varSr)/sqrt((k - 1/2)^2 - k0 * 
        (2 * k - k0 - 1))
    }
    k0 <- max(0, k0)
    k0 <- round(k0)
    se.k0 <- max(0, se.k0)
    if (verbose) 
      cat("Iteration:", iter, "\tmissing =", k0, "\t  b =", 
        ifelse(side == "right", -1 * b, b), "\n")
  }
  if (k0 > 0) {
    if (side == "right") {
      yi.c <- -1 * (yi.c - b)
    }
    else {
      yi.c <- yi.c - b
    }
    yi.fill <- c(x$yi.f, -1 * yi.c[(k - k0 + 1):k])
    vi.fill <- c(x$vi.f, vi[(k - k0 + 1):k])
    weights.fill <- c(x$weights.f, weights[(k - k0 + 1):k])
    ni.fill <- c(x$ni.f, ni[(k - k0 + 1):k])
    attr(yi.fill, "measure") <- x$measure
    args <- list(yi = yi.fill, vi = vi.fill, weights = weights.fill, ni = ni.fill, intercept = TRUE, method = x$method, weighted = x$weighted, ...)
    args <- args[!sapply(args, is.null)]
    res <- do.call(rma, args)
#    res <- rma(yi.fill, vi.fill, weights = weights.fill, 
#      ni = ni.fill, intercept = TRUE, method = x$method, 
#      weighted = x$weighted, ...)
    res$fill <- c(rep(FALSE, k), rep(TRUE, k0))
    res$ids <- c(x$ids, (x$k.f + 1):(x$k.f + k0))
    if (x$slab.null) {
      res$slab <- c(paste("Study", x$ids), paste("Filled", 
        seq_len(k0)))
    }
    else {
      res$slab <- c(x$slab, paste("Filled", seq_len(k0)))
    }
    res$slab.null <- FALSE
  }
  else {
    res <- x
    res$fill <- rep(FALSE, k)
  }
  res$k0 <- k0
  res$se.k0 <- se.k0
  res$side <- side
  res$k0.est <- estimator
  if (estimator == "R0") {
    m <- -1:(k0 - 1)
    res$pval <- 1 - sum(choose(0 + m + 1, m + 1) * 0.5^(0 + m + 2))
  }
  else {
    res$pval <- NA
  }
  class(res) <- c("rma.uni.trimfill", class(res))
  return(res)
}

mvma.hybrid <- function(ys, vars, data, method = "reml", tol = 1E-10){
  if(missing(ys)) stop("the argument ys is missing.")
  if(missing(vars)) stop("the argument vars is missing.")
  if(!missing(data)){
    ys <- eval(substitute(ys), data, parent.frame())
    vars <- eval(substitute(vars), data, parent.frame())
  }
  if(method != "ml" & method != "reml") stop("method must be either reml or ml.")
  ys <- as.matrix(ys)
  vars <- as.matrix(vars)
  n <- dim(ys)[1]
  p <- dim(ys)[2]
  if(p < 2) stop("please directly use the univariate model.")

  idx.na <- numeric(0)
  for(i in 1:n){
    if(all(is.na(ys[i,]))) idx.na <- c(idx.na, i)
  }
  if(length(idx.na) > 0){
    ys <- ys[-idx.na,]
    vars <- vars[-idx.na,]
    n <- dim(ys)[1]
  }

  idx.obs <- vector("list", n)
  X <- vector("list", n)
  ys.tilde <- vector("list", n)
  vars.tilde <- vector("list", n)
  for(i in 1:n){
    idx.obs[[i]] <- which(!is.na(ys[i,]))
    X[[i]] <- diag(p)[idx.obs[[i]],]
    if(length(idx.obs[[i]]) == 1) X[[i]] <- matrix(X[[i]], nrow = 1, ncol = p)
    ys.tilde[[i]] <- ys[i, idx.obs[[i]]]
    vars.tilde[[i]] <- vars[i, idx.obs[[i]]]
  }

  target <- function(x){
    mu <- x[1:p]
    tau2 <- x[(p + 1):(2*p)]
    thetas <- matrix(NA, nrow = p - 1, ncol = p - 1)
    for(i in 1:(p - 1)){
      for(j in 1:i){
        if(i == j) thetas[i, j] <- x[2*p + (i - 1)*i/2 + j]
        if(i != j) thetas[i, j] <- thetas[j, i] <- x[2*p + (i - 1)*i/2 + j]
      }
    }
    # Cholesky decomposition: all theta's are between 0 and pi
    L <- matrix(0, nrow = p, ncol = p)
    L[1, 1] <- 1
    for(i in 2:p){
      for(j in 1:i){
        if(j == 1) L[i, j] <- cos(thetas[i - 1, 1])
        if(j >= 2 & j <= i - 1) L[i, j] <- prod(sin(thetas[i - 1, 1:(j - 1)]))*cos(thetas[i - 1, j])
        if(j == i) L[i, j] <- prod(sin(thetas[i - 1, 1:(i - 1)]))
      }
    }

    R <- L%*%t(L)

    out1 <- 0
    out2 <- matrix(0, p, p)
    for(i in 1:n){
      var.temp <- vars.tilde[[i]] + tau2[idx.obs[[i]]]
      y.temp <- ys.tilde[[i]]
      y_mu_D_half <- (y.temp - mu[idx.obs[[i]]])*sqrt(1/var.temp)
      y_mu_D_half <- as.matrix(y_mu_D_half)
      logdet <- determinant(as.matrix(R[idx.obs[[i]], idx.obs[[i]]]), logarithm = TRUE)
      if(logdet$sign <= 0) logdet <- -Inf else logdet <- logdet$modulus
      temp <- logdet+ sum(log(var.temp)) + as.numeric(t(y_mu_D_half)%*%solve(R[idx.obs[[i]], idx.obs[[i]]])%*%y_mu_D_half)
      out1 <- out1 + temp
      if(method == "reml"){
        if(length(idx.obs[[i]]) == 1){
          var.temp <- as.numeric(var.temp)
          out2 <- out2 + t(X[[i]])%*%X[[i]]/var.temp
        }else{
          half.inv <- as.matrix(diag(1/sqrt(var.temp)))
          out2 <- out2 + t(X[[i]])%*%half.inv%*%solve(R[idx.obs[[i]], idx.obs[[i]]])%*%half.inv%*%X[[i]]
        }
      }
    }
    if(method == "ml") out <- as.numeric(out1)
    if(method == "reml"){
      logdet2 <- determinant(as.matrix(out2), logarithm = TRUE)
      if(logdet2$sign <= 0) logdet2 <- -Inf else logdet2 <- logdet2$modulus
      out <- out1 + logdet2
    }
    return(out)
  }

  pi <- 3.1415926
  optim.out <- optim(par = c(rep(0, p), rep(1, p), rep(pi/2, (p - 1)*p/2)), fn = target,
    lower = c(rep(-Inf, p), rep(10^(-6), p), rep(0 + 0.01, (p - 1)*p/2)),
    upper = c(rep(Inf, p), rep(Inf, p), rep(pi - 0.01, (p - 1)*p/2)),
    method = "L-BFGS-B", control = list(factr = tol))
  pars <- optim.out$par

  mu.est <- pars[1:p]
  tau2.est <- pars[(p + 1):(2*p)]

  thetas.est <- matrix(NA, nrow = p - 1, ncol = p - 1)
  for(i in 1:(p - 1)){
    for(j in 1:i){
      if(i == j) thetas.est[i, j] <- pars[2*p + (i - 1)*i/2 + j]
      if(i != j) thetas.est[i, j] <- thetas.est[j, i] <- pars[2*p + (i - 1)*i/2 + j]
    }
  }

  L.est <- matrix(0, nrow = p, ncol = p)
  L.est[1, 1] <- 1
  for(i in 2:p){
    for(j in 1:i){
      if(j == 1) L.est[i, j] <- cos(thetas.est[i - 1, 1])
      if(j >= 2 & j <= i - 1) L.est[i, j] <- prod(sin(thetas.est[i - 1, 1:(j - 1)]))*cos(thetas.est[i - 1, j])
      if(j == i) L.est[i, j] <- prod(sin(thetas.est[i - 1, 1:(i - 1)]))
    }
  }

  mar.R <- L.est%*%t(L.est)

  temp <- matrix(0, p, p)
  for(i in 1:n){
    var.temp <- vars.tilde[[i]] + tau2.est[idx.obs[[i]]]
    if(length(idx.obs[[i]]) == 1){
      var.temp <- as.numeric(var.temp)
      temp <- temp + t(X[[i]])%*%X[[i]]/var.temp
    }else{
      half.inv <- as.matrix(diag(1/sqrt(var.temp)))
      temp <- temp + t(X[[i]])%*%half.inv%*%solve(mar.R[idx.obs[[i]], idx.obs[[i]]])%*%half.inv%*%X[[i]]
    }
  }
  mu.cov <- solve(temp)

  out <- list(mu.est = mu.est, tau2.est = tau2.est, mar.R = mar.R, mu.cov = mu.cov, method = paste("Random-effects model using ", method, sep =""))
  return(out)
}

pb.as.reg.het <- function(y = NA, s2 = NA, n = NA, n00, n01, n10, n11){
  delta <- asin(sqrt(n11/(n10 + n11))) - asin(sqrt(n01/(n00 + n01)))
  gamma <- 1/(4*(n10 + n11)) + 1/(4*(n00 + n01))
  out <- pb.reg.het(y = delta, s2 = gamma)
  return(out)
}

print.meta.dt <- function(x, digits = 3, ...){
  if(!inherits(x, "meta.dt")){
    stop("The input must be an object of \"meta.dt\".")
  }

  cat(paste0("The meta-analysis contains ", dim(x$data)[1], " studies\n"))
  alpha <- x$alpha
  cat(paste0("Significance level: ", alpha, "\n"))
  if(x$method == "s.roc"){
    cat("The summary ROC approach is used\n")

    cat("Unweighted regression:\n")
    inter <- x$inter.unwtd
    inter.se <- sqrt(x$vcov.unwtd[1,1])
    inter.ci <- c(inter - qnorm(1 - alpha/2)*inter.se, inter + qnorm(1 - alpha/2)*inter.se)
    inter <- format(round(inter, digits), nsmall = digits)
    inter.se <- format(round(inter.se, digits), nsmall = digits)
    inter.ci <- format(round(inter.ci, digits), nsmall = digits)
    inter.ci <- gsub(" ", "", inter.ci)
    slope <- x$slope.unwtd
    slope.se <- sqrt(x$vcov.unwtd[2,2])
    slope.ci <- c(slope - qnorm(1 - alpha/2)*slope.se, slope + qnorm(1 - alpha/2)*slope.se)
    slope <- format(round(slope, digits), nsmall = digits)
    slope.se <- format(round(slope.se, digits), nsmall = digits)
    slope.ci <- format(round(slope.ci, digits), nsmall = digits)
    slope.ci <- gsub(" ", "", slope.ci)
    DOR.meanS <- x$DOR.meanS.unwtd
    DOR.meanS.ci <- x$DOR.meanS.unwtd.ci
    DOR.meanS <- format(round(DOR.meanS, digits), nsmall = digits)
    DOR.meanS.ci <- format(round(DOR.meanS.ci, digits), nsmall = digits)
    DOR.meanS.ci <- gsub(" ", "", DOR.meanS.ci)
    Q <- x$Q.unwtd
    Q.ci <- x$Q.unwtd.ci
    Q <- format(round(Q, digits), nsmall = digits)
    Q.ci <- format(round(Q.ci, digits), nsmall = digits)
    Q.ci <- gsub(" ", "", Q.ci)
    AUC <- x$AUC.unwtd
    AUC <- format(round(AUC, digits), nsmall = digits)
    cat(paste0("  intercept = ", inter, " (SE, ", inter.se, "; CI, ", inter.ci[1], " to ", inter.ci[2], ")\n"))
    cat(paste0("  slope = ", slope, " (SE, ", slope.se, "; CI, ", slope.ci[1], " to ", slope.ci[2], ")\n"))
    cat(paste0("  DOR = ", DOR.meanS, " (CI, ", DOR.meanS.ci[1], " to ", DOR.meanS.ci[2], ")\n"))
    cat(paste0("  Q point = ", Q, " (CI, ", Q.ci[1], " to ", Q.ci[2], ")\n"))
    cat(paste0("  AUC = ", AUC, "\n"))

    cat("Weighted regression:\n")
    inter <- x$inter.wtd
    inter.se <- sqrt(x$vcov.wtd[1,1])
    inter.ci <- c(inter - qnorm(1 - alpha/2)*inter.se, inter + qnorm(1 - alpha/2)*inter.se)
    inter <- format(round(inter, digits), nsmall = digits)
    inter.se <- format(round(inter.se, digits), nsmall = digits)
    inter.ci <- format(round(inter.ci, digits), nsmall = digits)
    inter.ci <- gsub(" ", "", inter.ci)
    slope <- x$slope.wtd
    slope.se <- sqrt(x$vcov.wtd[2,2])
    slope.ci <- c(slope - qnorm(1 - alpha/2)*slope.se, slope + qnorm(1 - alpha/2)*slope.se)
    slope <- format(round(slope, digits), nsmall = digits)
    slope.se <- format(round(slope.se, digits), nsmall = digits)
    slope.ci <- format(round(slope.ci, digits), nsmall = digits)
    slope.ci <- gsub(" ", "", slope.ci)
    DOR.meanS <- x$DOR.meanS.wtd
    DOR.meanS.ci <- x$DOR.meanS.wtd.ci
    DOR.meanS <- format(round(DOR.meanS, digits), nsmall = digits)
    DOR.meanS.ci <- format(round(DOR.meanS.ci, digits), nsmall = digits)
    DOR.meanS.ci <- gsub(" ", "", DOR.meanS.ci)
    Q <- x$Q.wtd
    Q.ci <- x$Q.wtd.ci
    Q <- format(round(Q, digits), nsmall = digits)
    Q.ci <- format(round(Q.ci, digits), nsmall = digits)
    Q.ci <- gsub(" ", "", Q.ci)
    AUC <- x$AUC.wtd
    AUC <- format(round(AUC, digits), nsmall = digits)
    cat(paste0("  intercept = ", inter, " (SE, ", inter.se, "; CI, ", inter.ci[1], " to ", inter.ci[2], ")\n"))
    cat(paste0("  slope = ", slope, " (SE, ", slope.se, "; CI, ", slope.ci[1], " to ", slope.ci[2], ")\n"))
    cat(paste0("  DOR = ", DOR.meanS, " (CI, ", DOR.meanS.ci[1], " to ", DOR.meanS.ci[2], ")\n"))
    cat(paste0("  Q point = ", Q, " (CI, ", Q.ci[1], " to ", Q.ci[2], ")\n"))
    cat(paste0("  AUC = ", AUC, "\n"))
  }

  if(is.element(x$method, c("biv.lmm", "biv.glmm"))){
    if(x$method == "biv.lmm"){
      cat("The bivariate linear mixed model is used\n")
    }
    if(x$method == "biv.glmm"){
      cat("The bivariate generalized linear mixed model is used\n")
    }

    sens.overall <- x$sens.overall
    sens.overall.ci <- x$sens.overall.ci
    sens.overall <- format(round(sens.overall, digits), nsmall = digits)
    sens.overall.ci <- format(round(sens.overall.ci, digits), nsmall = digits)
    sens.overall.ci <- gsub(" ", "", sens.overall.ci)
    spec.overall <- x$spec.overall
    spec.overall.ci <- x$spec.overall.ci
    spec.overall <- format(round(spec.overall, digits), nsmall = digits)
    spec.overall.ci <- format(round(spec.overall.ci, digits), nsmall = digits)
    spec.overall.ci <- gsub(" ", "", spec.overall.ci)
    DOR.overall <- x$DOR.overall
    DOR.overall.ci <- x$DOR.overall.ci
    DOR.overall <- format(round(DOR.overall, digits), nsmall = digits)
    DOR.overall.ci <- format(round(DOR.overall.ci, digits), nsmall = digits)
    DOR.overall.ci <- gsub(" ", "", DOR.overall.ci)
    AUC <- x$AUC
    AUC <- format(round(AUC, digits), nsmall = digits)
    mu.sens <- x$mu.sens
    mu.spec <- x$mu.spec
    sig.sens <- x$sig.sens
    sig.spec <- x$sig.spec
    rho <- x$rho
    mu.sens <- format(round(mu.sens, digits), nsmall = digits)
    mu.spec <- format(round(mu.spec, digits), nsmall = digits)
    sig.sens <- format(round(sig.sens, digits), nsmall = digits)
    sig.spec <- format(round(sig.spec, digits), nsmall = digits)
    rho <- format(round(rho, digits), nsmall = digits)
    cat(paste0("  Overall sensitivity = ", sens.overall, " (CI, ", sens.overall.ci[1], " to ", sens.overall.ci[2], ")\n"))
    cat(paste0("  Overall specificity = ", spec.overall, " (CI, ", spec.overall.ci[1], " to ", spec.overall.ci[2], ")\n"))
    cat(paste0("  Overall DOR = ", DOR.overall, " (CI, ", DOR.overall.ci[1], " to ", DOR.overall.ci[2], ")\n"))
    cat(paste0("  AUC = ", AUC, "\n"))
    cat("  Estimates of model parameters:\n")
    cat(paste0("    mu = ", mu.sens, ", sigma = ", sig.sens, " (sensitivity)\n"))
    cat(paste0("    mu = ", mu.spec, ", sigma = ", sig.spec, " (specificity)\n"))
    cat(paste0("    rho = ", rho, " (correlation coefficient)\n"))
  }
}

determine.lambda <- function(y, s2, tau2.re, penalty = "tau2", n.lambda = 100, lambda.scale = "log",
                             tol = 10^(-10), lam.c = 1.2) {
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  if(missing(tau2.re)) {tau2.re <- meta.ml(y, s2, tol)}

  mu.hat <- function(tau2) {
    sum(y/(s2 + tau2))/sum(1/(s2 + tau2))
  }
  mu.hat <- Vectorize(mu.hat)

  if (penalty == "tau2") {
    fcn <- function(tau2) {
      sum((y - mu.hat(tau2))^2/(s2 + tau2)^2) - sum(1/(s2 + tau2))
    }
  }
  if (penalty == "tau") {
    fcn <- function(tau2) {
      (sum((y - mu.hat(tau2))^2/(s2 + tau2)^2) - sum(1/(s2 + tau2))) * (2 * sqrt(tau2))
    }
  }
  fcn <- Vectorize(fcn)
  lam.max <- optimize(f = fcn, lower = 0, upper = tau2.re*1.1, maximum = TRUE)$objective

  if (lambda.scale == "linear") lam.max <- max(c(0, lam.max))*lam.c
  if (lambda.scale == "log") lam.max <- exp(log(max(c(0, lam.max)) + 1) + log(lam.c)) - lam.c

  if (lambda.scale == "linear") lambda <- seq(from = 0, to = lam.max, length.out = n.lambda)
  if (lambda.scale == "log") lambda <- exp(seq(from = 0, to = log(lam.max + 1), length.out = n.lambda)) - 1

  return(lambda)
}

metaoutliers <- function(y, s2, data, model){
  if(missing(y)) stop("please specify effect size.")
  if(missing(s2)) stop("please specify within-study variance.")
  if(!missing(data)){
    y <- eval(substitute(y), data, parent.frame())
    s2 <- eval(substitute(s2), data, parent.frame())
  }
  if(length(y) != length(s2) | any(s2 < 0)) stop("error in the input data.")
  w <- 1/s2
  y.p <- sum(y*w)/sum(w)
  n <- length(y)

  if(missing(model)){
    hetmeasure <- metahet.base(y, s2)
    Ir2 <- hetmeasure$Ir2
    if(Ir2 < 0.3){
      model <- "FE"
      cat("This function uses fixed-effect meta-analysis because Ir2 < 30%.\n")
    }else{
      model <- "RE"
      cat("This function uses random-effects meta-analysis because Ir2 >= 30%.\n")
    }
  }

  if(!is.element(model, c("FE", "RE"))) stop("wrong input for the argument model.")

  y.p.i <- res <- std.res <- numeric(n)
  if(model == "FE"){
    for(i in 1:n){
      w.temp <- w[-i]
      y.temp <- y[-i]
      y.p.i[i] <- sum(y.temp*w.temp)/sum(w.temp)
      res[i] <- y[i] - y.p.i[i]
      var.res.i <- 1/sum(w.temp) + s2[i]
      std.res[i] <- res[i]/sqrt(var.res.i)
    }
  }else{
    for(i in 1:n){
      s2.temp <- s2[-i]
      y.temp <- y[-i]
      tau2.temp <- metahet.base(y.temp, s2.temp)$tau2.DL
      w.temp <- 1/(s2.temp + tau2.temp)
      y.p.i[i] <- sum(y.temp*w.temp)/sum(w.temp)
      res[i] <- y[i] - y.p.i[i]
      var.res.i <- 1/sum(w.temp) + s2[i] + tau2.temp
      std.res[i] <- res[i]/sqrt(var.res.i)
    }
  }

  outliers <- which(abs(std.res) >= 3)
  if(length(outliers) == 0) outliers <- "All the standardized residuals are smaller than 3"

  out <- NULL
  out$model <- model
  out$std.res <- std.res
  out$outliers <- outliers

  class(out) <- "metaoutliers"
  return(out)
}

meta.biv <- function(sid, tid, e, n, data, link = "logit",
  alpha = 0.05, b.iter = 1000, seed = 1234, ...){
  if(missing(sid)) stop("need to specify study IDs.")
  if(missing(tid)) stop("need to specify treatment IDs (0/1).")
  if(missing(e)) stop("need to specify event counts.")
  if(missing(n)) stop("need to specify sample sizes.")
  if(!missing(data)){
    sid <- eval(substitute(sid), data, parent.frame())
    tid <- eval(substitute(tid), data, parent.frame())
    e <- eval(substitute(e), data, parent.frame())
    n <- eval(substitute(n), data, parent.frame())
  }
  if((length(sid) != length(n)) | (length(tid) != length(n)) | (length(e) != length(n))){
    stop("the input data have different dimensions.")
  }
  if(!is.element(link, c("logit", "probit"))){
    stop("specify either the logit or probit link function.")
  }
  if(alpha < 0 | alpha > 1) stop("alpha should be between 0 and 1.")

  ord <- order(sid)
  sid <- sid[ord]
  tid <- tid[ord]
  e <- e[ord]
  n <- n[ord]
  data <- data.frame(sid = sid, tid = tid, cid = 1 - tid, e = e, n = n)
  rslt <- glmer(cbind(e, n - e) ~ factor(tid) + (cid + tid - 1 | sid),
    data = data, family = binomial(link = link), ...)
  smry <- summary(rslt)
  mu0 <- smry$coefficients[1,1]
  mu1 <- smry$coefficients[2,1] + mu0
  sd0 <- attr(smry$varcor[[1]], "stddev")[1]
  sd1 <- attr(smry$varcor[[1]], "stddev")[2]
  rho <- attr(smry$varcor[[1]], "correlation")[1,2]
  names(sd0) <- names(sd1) <- NULL

  if(link == "logit"){
    logOR.c <- summary(rslt)$coefficients[2,1]
    logOR.c.se <- summary(rslt)$coefficients[2,2]
    OR.c <- exp(logOR.c)
    OR.c.ci <- exp(c(logOR.c - qnorm(1 - alpha/2)*logOR.c.se,
      logOR.c + qnorm(1 - alpha/2)*logOR.c.se))
    C <- 16*sqrt(3)/(15*3.1415926)
    p0.m <- 1/(1 + exp(-mu0/sqrt(1 + C^2*sd0^2)))
    p1.m <- 1/(1 + exp(-mu1/sqrt(1 + C^2*sd1^2)))

    out <- list(OR.c = OR.c, OR.c.ci = OR.c.ci)
  }

  if(link == "probit"){
    p0.m <- pnorm(mu0/sqrt(1 + sd0^2))
    p1.m <- pnorm(mu1/sqrt(1 + sd1^2))
    out <- NULL
  }

  OR.m <- p1.m/(1 - p1.m)/p0.m*(1 - p0.m)
  RR.m <- p1.m/p0.m
  RD.m <- p1.m - p0.m

  p0.m.b <- p1.m.b <- OR.m.b <- RR.m.b <- RD.m.b <- rho.b <- rep(NA, b.iter)
  set.seed(seed)
  b.idx <- 0
  b.w <- b.e <- 0
  while(b.idx < b.iter){
    idx.temp <- sample(length(unique(sid)), replace = TRUE)
    idx.temp <- c(rbind(idx.temp*2 - 1, idx.temp*2))
    data.temp <- data[idx.temp,]
    data.temp$sid <- rep(1:length(unique(sid)), each = 2)
    suppressMessages(out.W.E <- tryCatch.W.E(
      rslt.temp <- glmer(cbind(e, n - e) ~ factor(tid) + (cid + tid - 1 | sid),
        data = data.temp, family = binomial(link = link), ...)))
    suppressMessages(out.W.E2 <- tryCatch.W.E(smry.temp <- summary(rslt.temp)))
    if(is.null(out.W.E$warning) & !inherits(out.W.E$value, "error") &
      is.null(out.W.E2$warning) & !inherits(out.W.E2$value, "error")){
      b.idx <- b.idx + 1
      mu0.temp <- smry.temp$coefficients[1,1]
      mu1.temp <- smry.temp$coefficients[2,1] + mu0.temp
      sd0.temp <- attr(smry.temp$varcor[[1]], "stddev")[1]
      sd1.temp <- attr(smry.temp$varcor[[1]], "stddev")[2]
      rho.b[b.idx] <- attr(smry.temp$varcor[[1]], "correlation")[1,2]
      if(link == "logit"){
        C <- 16*sqrt(3)/(15*3.1415926)
        p0.m.temp <- 1/(1 + exp(-mu0.temp/sqrt(1 + C^2*sd0.temp^2)))
        p1.m.temp <- 1/(1 + exp(-mu1.temp/sqrt(1 + C^2*sd1.temp^2)))
      }
      if(link == "probit"){
        p0.m.temp <- pnorm(mu0.temp/sqrt(1 + sd0.temp^2))
        p1.m.temp <- pnorm(mu1.temp/sqrt(1 + sd1.temp^2))
      }
      p0.m.b[b.idx] <- p0.m.temp
      p1.m.b[b.idx] <- p1.m.temp
      OR.m.b[b.idx] <- p1.m.temp/(1 - p1.m.temp)/p0.m.temp*(1 - p0.m.temp)
      RR.m.b[b.idx] <- p1.m.temp/p0.m.temp
      RD.m.b[b.idx] <- p1.m.temp - p0.m.temp
    }
    if(!is.null(out.W.E$warning) | !is.null(out.W.E2$warning)){
      b.w <- b.w + 1
    }
    if(inherits(out.W.E$value, "error") | inherits(out.W.E2$value, "error")){
      b.e <- b.e + 1
    }
  }
  p0.m.ci <- quantile(p0.m.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  p1.m.ci <- quantile(p1.m.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  OR.m.ci <- quantile(OR.m.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  RR.m.ci <- quantile(RR.m.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  RD.m.ci <- quantile(RD.m.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  rho.ci <- quantile(rho.b, probs = c(alpha/2, 1 - alpha/2), na.rm = TRUE)
  names(p0.m.ci) <- names(p1.m.ci) <- names(OR.m.ci) <- names(RR.m.ci) <-
    names(RD.m.ci) <- names(rho.ci) <- NULL

  out <- c(out, list(p0.m = p0.m, p0.m.ci = p0.m.ci, p1.m = p1.m, p1.m.ci = p1.m.ci,
    OR.m = OR.m, OR.m.ci = OR.m.ci, RR.m = RR.m, RR.m.ci = RR.m.ci,
    RD.m = RD.m, RD.m.ci = RD.m.ci, rho = rho, rho.ci = rho.ci,
    b.w.e = c("bootstrap warnings" = b.w, "bootstrap errors" = b.e)))
  return(out)
}

plot.metaoutliers <- function(x, xtick.cex = 1, ytick.cex = 0.5, ...){
	std.res <- x$std.res
	n <- length(std.res)
	abs.max <- max(abs(std.res))
	abs.limit <- abs.max*1.1
	plot.default(x = std.res, y = n:1, ylab = "Study", xlab = "Standardized Residual",
		ylim = c(1, n), xlim = c(-abs.limit, abs.limit), yaxt = "n", xaxt = "n", pch = 20, cex = 1.5, cex.lab = 1)
	axis(side = 2, at = 1:n, labels = n:1, cex.axis = ytick.cex)
	axis(side = 1, at = -4:4, cex.axis = xtick.cex)
	if(abs.limit > 3) abline(v = c(-3, 3), lwd = 2, lty = 2)
}

plot.meta.dt <- function(x, add = FALSE, xlab, ylab, alpha,
  studies = TRUE, cex.studies, col.studies, pch.studies,
  roc, col.roc, lty.roc, lwd.roc, weight = FALSE,
  eqline, col.eqline, lty.eqline, lwd.eqline,
  overall = TRUE, cex.overall, col.overall, pch.overall,
  confid = TRUE, col.confid, lty.confid, lwd.confid,
  predict = FALSE, col.predict, lty.predict, lwd.predict, ...){
  if(!inherits(x, "meta.dt")){
    stop("The input must be an object of class \"meta.dt\".")
  }
  if(x$method == "s.roc"){
    overall <- confid <- predict <- FALSE
    if(missing(eqline)) eqline <- TRUE
    if(missing(roc)) roc <- TRUE
  }else{
    if(missing(eqline)) eqline <- FALSE
    if(missing(roc)) roc <- FALSE
  }

  if(!add){
    if(missing(xlab)) xlab <- "1 - Specificity"
    if(missing(ylab)) ylab <- "Sensitivity"
    plot.default(0, type = "n", xlim = c(0, 1), ylim = c(0, 1),
      xlab = xlab, ylab = ylab, ...)
  }

  if(studies){
    dat <- x$data
    tp <- dat$tp
    fp <- dat$fp
    fn <- dat$fn
    tn <- dat$tn
    xval <- 1 - tn/(fp + tn)
    yval <- tp/(tp + fn)
    if(missing(cex.studies)) cex.studies <- 1
    if(missing(col.studies)) col.studies <- "black"
    if(missing(pch.studies)) pch.studies <- 1
    points(xval, yval, cex = cex.studies, col = col.studies, pch = pch.studies)
  }

  if(roc){
    if(missing(col.roc)) col.roc <- "black"
    if(missing(lty.roc)) lty.roc <- 1
    if(missing(lwd.roc)) lwd.roc <- 1
    if(x$method == "s.roc"){
      if(!weight){
        inter <- x$inter.unwtd
        slope <- x$slope.unwtd
      }else{
        inter <- x$inter.wtd
        slope <- x$slope.wtd
      }
      roc.curve <- function(xval){
        spec <- 1 - xval
        sens <- 1/(1 + exp(-inter/(1 - slope))*(spec/(1 - spec))^((1 + slope)/(1 - slope)))
        return(sens)
      }
    }
    if(is.element(x$method, c("biv.lmm", "biv.glmm"))){
      mu.sens <- x$mu.sens
      mu.spec <- x$mu.spec
      sig.sens <- x$sig.sens
      sig.spec <- x$sig.spec
      rho <- x$rho
      roc.curve <- function(xval){
        spec <- 1 - xval
        logit.spec <- log(spec/(1 - spec))
        logit.sens <- mu.sens + rho*sig.sens/sig.spec*(logit.spec - mu.spec)
        sens <- 1/(1 + exp(-logit.sens))
        return(sens)
      }
    }
    curve(roc.curve, from = 0, to = 1, add = TRUE,
      col = col.roc, lty = lty.roc, lwd = lwd.roc)
  }

  if(eqline){
    if(missing(col.eqline)) col.eqline <- "black"
    if(missing(lty.eqline)) lty.eqline <- 4
    if(missing(lwd.eqline)) lwd.eqline <- 1
    eqline.fcn <- function(x) 1 - x
    curve(eqline.fcn, from = 0, to = 1, add = TRUE,
      col = col.eqline, lty = lty.eqline, lwd = lwd.eqline)
  }

  if(overall){
    if(missing(cex.overall)) cex.overall <- 1
    if(missing(col.overall)) col.overall <- "black"
    if(missing(pch.overall)) pch.overall <- 15
    points(1 - x$spec.overall, x$sens.overall,
      cex = cex.overall, col = col.overall, pch = pch.overall)
  }

  if(confid){
    if(missing(alpha)) alpha <- x$alpha
    mu.sens <- x$mu.sens
    mu.spec <- x$mu.spec
    mu.vcov <- x$mu.vcov
    mu.vcov.inv <- solve(mu.vcov)
    confid.contour <- function(xval, yval){
      sens <- yval
      spec <- 1 - xval
      logit.sens <- log(sens/(1 - sens))
      logit.spec <- log(spec/(1 - spec))
      out <- mu.vcov.inv[1,1]*(logit.sens - mu.sens)^2 + mu.vcov.inv[2,2]*(logit.spec - mu.spec)^2 +
        2*mu.vcov.inv[1,2]*(logit.sens - mu.sens)*(logit.spec - mu.spec) - qchisq(1 - alpha, df = 2)
      return(out)
    }
    xval <- yval <- seq(0, 1, length = 1000)
    zval <- outer(xval, yval, confid.contour)
    if(missing(col.confid)) col.confid <- "black"
    if(missing(lty.confid)) lty.confid <- 2
    if(missing(lwd.confid)) lwd.confid <- 1
    contour(x = xval, y = yval, z = zval, levels = 0, drawlabels = FALSE, axes = FALSE,
     col = col.confid, lty = lty.confid, lwd = lwd.confid, add = TRUE)
  }

  if(predict){
    if(missing(alpha)) alpha <- x$alpha
    mu.sens <- x$mu.sens
    mu.spec <- x$mu.spec
    mu.vcov <- x$mu.vcov
    sig.sens <- x$sig.sens
    sig.spec <- x$sig.spec
    rho <- x$rho
    re.mat <- matrix(c(sig.sens^2, rho*sig.sens*sig.spec, rho*sig.sens*sig.spec, sig.spec^2), 2, 2)
    tot.vcov <- mu.vcov + re.mat
    tot.vcov.inv <- solve(tot.vcov)
    predict.contour <- function(xval, yval){
      sens <- yval
      spec <- 1 - xval
      logit.sens <- log(sens/(1 - sens))
      logit.spec <- log(spec/(1 - spec))
      out <- tot.vcov.inv[1,1]*(logit.sens - mu.sens)^2 + tot.vcov.inv[2,2]*(logit.spec - mu.spec)^2 +
        2*tot.vcov.inv[1,2]*(logit.sens - mu.sens)*(logit.spec - mu.spec) - qchisq(1 - alpha, df = 2)
      return(out)
    }
    xval <- yval <- seq(0, 1, length = 1000)
    zval <- outer(xval, yval, predict.contour)
    if(missing(col.predict)) col.predict <- "black"
    if(missing(lty.predict)) lty.predict <- 3
    if(missing(lwd.predict)) lwd.predict <- 1
    contour(x = xval, y = yval, z = zval, levels = 0, drawlabels = FALSE, axes = FALSE,
      col = col.predict, lty = lty.predict, lwd = lwd.predict, add = TRUE)
  }
}

tryCatch.W.E <- function(expr){
	W <- NULL
	w.handler <- function(w){ # warning handler
		W <<- w
		invokeRestart("muffleWarning")
	}
	list(value = withCallingHandlers(tryCatch(expr, error = function(e) e),
		warning = w.handler), warning = W)
}

\name{dat.dep}
\alias{dat.dep}
\docType{data}
\title{
A Meta-Analysis of Binary and Continuous Outcomes on Depression
}
\description{
This dataset serves as an example of meta-analysis of combining standardized mean differences and odds ratios.
}
\usage{data("dat.dep")}
\format{
  A data frame with 6 studies with the following 15 variables within each study.
  \describe{
    \item{\code{author}}{The first author of each study.}
    \item{\code{year}}{The publication year of each study.}
    \item{\code{treatment}}{The treatment group.}
    \item{\code{control}}{The control group.}
    \item{\code{y1}}{The sample mean in the treatment group for the continuous outcome.}
    \item{\code{sd1}}{The sample standard deviation in the treatment group for the continuous outcome.}
    \item{\code{n1}}{The sample size in the treatment group for the continuous outcome.}
    \item{\code{y0}}{The sample mean in the control group for the continuous outcome.}
    \item{\code{sd0}}{The sample standard deviation in the control group for the continuous outcome.}
    \item{\code{n0}}{The sample size in the control group for the continuous outcome.}
    \item{\code{r1}}{The event count in the treatment group for the binary outcome.}
    \item{\code{m1}}{The sample size in the treatment group for the binary outcome.}
    \item{\code{r0}}{The event count in the control group for the binary outcome.}
    \item{\code{m0}}{The sample size in the control group for the binary outcome.}
    \item{\code{id.bin}}{An indicator of whether the outcome is binary (1) or continuous (0).}
  }
}
\details{
This dataset is from Cipriani et al.\sspace(2016), comparing the efficacy and tolerability of antidepressants for major depressive disorders in children and adolescents. Our case study focuses on efficacy. The authors originally performed a network meta-analysis; however, here we restrict the comparison to fluoxetine and placebo. The continuous outcomes are measured by the mean overall changes in depressive symptoms from baseline to endpoint. For the binary outcomes, events are defined as whether patients' depression rating scores were reduced by at least a specified cutoff value.
}
\source{
Cipriani A, Zhou X, Del Giovane C, Hetrick SE, Qin B, Whittington C, Coghill D, Zhang Y, Hazell P, Leucht S, Cuijpers P, Pu J, Cohen D, Ravindran AV, Liu Y, Michael KD, Yang L, Liu L, Xie P (2016). "Comparative efficacy and tolerability of antidepressants for major depressive disorder in children and adolescents: a network meta-analysis." \emph{The Lancet}, \bold{388}(10047), 881--890. <\doi{10.1016/S0140-6736(16)30385-3}>
}
\keyword{dataset}

\name{pb.bayesian.binary}
\alias{pb.bayesian.binary}

\title{
Bayesian Method for Assessing Publication Bias/Small-Study Effects in Meta-Analysis of a Binary Outcome
}
\description{
Performs multiple methods introduced in Shi et al.\sspace(2020) to assess publication bias/small-study effects under the Bayesian framework in a meta-analysis of (log) odds ratios.
}
\usage{
pb.bayesian.binary(n00, n01, n10, n11, p01 = NULL, p11 = NULL, data,
                   sig.level = 0.1,  method = "bay", het = "mul",
                   sd.prior = "unif", n.adapt = 1000, n.chains = 3,
                   n.burnin = 5000, n.iter = 10000, thin = 2,
                   upp.het = 2, phi = 0.5, coda = FALSE,
                   traceplot = FALSE, seed = 1234)
}

\arguments{
  \item{n00}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of non-events in treatment group 0 in the collected studies.
  }
  \item{n01}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of events in treatment group 0 in the collected studies.
  }
  \item{n10}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of non-events in treatment group 1 in the collected studies.
  }
  \item{n11}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of events in treatment group 1 in the collected studies. 
  }
  \item{p01}{
  an optional numeric vector specifying true event rates (e.g., from simulations) in the treatment group 0 across studies.
  }
  \item{p11}{
  an optional numeric vector specifying true event rates (e.g., from simulations) in the treatment group 1 across studies.
  }
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{n00}, \code{n01}, \code{n10}, \code{n11}, \code{p01} (if any), and \code{p11} (if any) should be specified as their corresponding column names in \code{data}.
  }
  \item{sig.level}{
  a numeric value specifying the statistical significance level \eqn{\alpha} for testing for publication bias. The default is 0.1. It corresponds to \eqn{(1 - \alpha) \times 100\%} confidence/credible intervals.
  }
  \item{method}{
  a character string specifying the method for assessing publication bias via Bayesian hierarchical models. It can be one of \code{"bay"} (the Bayesian approach proposed in Shi et al., 2020), \code{"reg.bay"} (Egger's regression test, see Egger et al., 1997, under the Bayesian framework) and \code{"smoothed.bay"} (the regression test based on the smoothed sample variance, see Jin et al., 2014, under the Bayesian framework), where all regression tests are under the random-effects setting. The default is \code{"bay"}. 
}
  \item{het}{
  a character string specifying the type of heterogeneity assumption for the publication bias tests. It can be either \code{"mul"} (multiplicative heterogeneity assumption; see Thompson and Sharpe, 1999) or \code{"add"} (additive heterogeneity assumption). The default is \code{"mul"}.
  }
  \item{sd.prior}{
  a character string specifying prior distributions for standard deviation parameters. It can be either \code{"unif"} (uniform distribution) or \code{"hn"} (half-normal distribution). The default is \code{"unif"}.
  }
  \item{n.adapt}{
  the number of iterations for adaptation in the Markov chain Monte Carlo (MCMC) algorithm. The default is 1,000. This argument and the following \code{n.chains}, \code{n.burnin}, \code{n.iter}, and \code{thin} are passed to the functions in the package \pkg{rjags}.
  }
  \item{n.chains}{
  the number of MCMC chains. The default is 1.
  }
  \item{n.burnin}{
  the number of iterations for burn-in period. The default is 5,000.
  }
  \item{n.iter}{
  the total number of iterations in each MCMC chain after the burn-in period. The default is 10,000.
  }
  \item{thin}{
  a positive integer specifying thinning rate. The default is 2.
  }
  \item{upp.het}{
  a positive number for specifying the upper bound of uniform priors for standard deviation parameters (if \code{sd.prior} = \code{"unif"}). The default is 2.
  }
  \item{phi}{
  a positive number for specifying the hyper-parameter of half-normal priors for standard deviation parameters (if \code{sd.prior} = \code{"hn"}). The default is 0.5. 
  }
  \item{coda}{
  a logical value indicating whether to output MCMC posterior samples. The default is \code{FALSE}.
  }
  \item{traceplot}{
  a logical value indicating whether to draw trace plots for the regression slopes. The default is \code{FALSE}.
  }
  \item{seed}{
  an integer for specifying the seed value for reproducibility.
  }
}
\details{
The Bayesian models are specified in Shi et al.\sspace(2020). The vague prior N(0, \eqn{10^4}) is used for the regression intercept and slope, and the uniform prior U(0, \code{upp.het}) and half-normal prior HN(\code{phi}) are used for standard deviation parameters. The half-normal priors may be preferred in meta-analyses with rare events or small sample sizes.
}
\value{
This function returns a list containing estimates of regression slopes and their credible intervals with the specified significance level (\code{sig.level}) as well as MCMC posterior samples (if \code{coda} = \code{TRUE}). Each element name in this list is related to a certain publication bias method (e.g., \code{est.bay} and \code{ci.bay} represent the slope estimate and its credible interval based on the proposed Bayesian method). In addition, trace plots for the regression slope are drawn if \code{traceplot} = \code{TRUE}. 
}
\references{
Egger M, Davey Smith G, Schneider M, Minder C (1997). "Bias in meta-analysis detected by a simple, graphical test." \emph{BMJ}, \bold{315}(7109), 629--634. <\doi{10.1136/bmj.315.7109.629}>

Jin Z-C, Wu C, Zhou X-H, He J (2014). "A modified regression method to test publication bias in meta-analyses with binary outcomes." \emph{BMC Medical Research Methodology}, \bold{14}, 132. <\doi{10.1186/1471-2288-14-132}>

Shi L, Chu H, Lin L (2020). "A Bayesian approach to assessing small-study effects in meta-analysis of a binary outcome with controlled false positive rate". \emph{Research Synthesis Methods}, \bold{11}(4), 535--552. <\doi{10.1002/jrsm.1415}>

Thompson SG, Sharp SJ (1999). "Explaining heterogeneity in meta-analysis: a comparison of methods." \emph{Statistics in Medicine}, \bold{18}(20), 2693--2708. <\doi{10.1002/(SICI)1097-0258(19991030)18:20<2693::AID-SIM235>3.0.CO;2-V}>
}
\author{
Linyu Shi \email{ls16d@my.fsu.edu}
}
\note{
The current version does not support other effect measures such as relative risks or risk differences.
}

\seealso{
\code{\link{pb.hybrid.binary}}, \code{\link{pb.hybrid.generic}}
}
\examples{
\donttest{
data("dat.poole")
set.seed(654321)
## increase n.burnin and n.iter for better convergence of MCMC
rslt.poole <- pb.bayesian.binary(n00, n01, n10, n11, data = dat.poole,
  method = "bay", het = "mul", sd.prior = "unif", n.adapt = 1000,
  n.chains = 3, n.burnin = 500, n.iter = 2000, thin = 2, upp.het = 2)
rslt.poole

data("dat.ducharme")
set.seed(654321)
## increase n.burnin and n.iter for better convergence of MCMC
rslt.ducharme <- pb.bayesian.binary(n00, n01, n10, n11, data = dat.ducharme,
  method = "bay", het = "mul", sd.prior = "unif", n.adapt = 1000,
  n.chains = 3, n.burnin = 500, n.iter = 2000, thin = 2, upp.het = 2)
rslt.ducharme

data("dat.henry")
set.seed(654321)
## increase n.burnin and n.iter for better convergence of MCMC
rslt.henry <- pb.bayesian.binary(n00, n01, n10, n11, data = dat.henry,
  method = "bay", het = "mul", sd.prior = "unif", n.adapt = 1000,
  n.chains = 3, n.burnin = 500, n.iter = 2000, thin = 2, upp.het = 2)
rslt.henry
}
}
\keyword{publication bias/small-study effects}

\name{pb.hybrid.binary}
\alias{pb.hybrid.binary}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Hybrid Test for Publication Bias/Small-Study Effects in Meta-Analysis With Binary Outcomes
}
\description{
Performs the hybrid test for publication bias/small-study effects introduced in Lin (2020), which synthesizes results from multiple popular publication bias tests, in a meta-analysis with binary outcomes.
}
\usage{
pb.hybrid.binary(n00, n01, n10, n11, data, methods,
                 iter.resam = 1000, theo.pval = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{n00}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of non-events in treatment group 0 in the collected studies.
}
  \item{n01}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of events in treatment group 0 in the collected studies.
}
  \item{n10}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of non-events in treatment group 1 in the collected studies.
}
  \item{n11}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the counts of events in treatment group 1 in the collected studies.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{n00}, \code{n01}, \code{n10}, and \code{n11}, should be specified as their corresponding column names in \code{data}.
}
  \item{methods}{
  a vector of character strings specifying the publication bias tests to be included in the hybrid test. They can be a subset of \code{"rank"} (Begg's rank test; see Begg and Mazumdar, 1994), \code{"reg"} (Egger's regression test under the fixed-effect setting; see Egger et al., 1997), \code{"reg.het"} (Egger's regression test accounting for additive heterogeneity), \code{"skew"} (the skewness-based test under the fixed-effect setting; see Lin and Chu, 2018), \code{"skew.het"} (the skewness-based test accounting for additive heterogeneity), \code{"inv.sqrt.n"} (the regression test based on sample sizes; see Tang and Liu, 2000), \code{"trimfill"} (the trim-and-fill method; see Duval and Tweedie, 2000), \code{"n"} (the regressoin test with sample sizes as the predictor; see Macaskill et al., 2001), \code{"inv.n"} (the regressoin test with the inverse of sample sizes as the predictor; see Peters et al., 2006), \code{"as.rank"} (the rank test based on the arcsine-transformed effect sizes; see Rucker et al., 2008), \code{"as.reg"} (the regression test based on the arcsine-transformed effect sizes under the fixed-effect setting), \code{"as.reg.het"} (the regression test based on the arcsine-transformed effect sizes accounting for additive heterogeneity), \code{"smoothed"} (the regression test based on the smoothed sample variances under the fixed-effect setting; see Jin et al., 2014), \code{"smoothed.het"} (the regression test based on the smoothed sample variances accounting for additive heterogeneity), \code{"score"} (the regression test based on the score function; see Harbord et al., 2006), and \code{"count"} (the test based on the hypergeometric distributions of event counts, designed for sparse data; see Schwarzer et al., 2007). The default is to include all aforementioned tests.
}
  \item{iter.resam}{
  a positive integer specifying the number of resampling iterations for calculating the p-value of the hybrid test.
}
  \item{theo.pval}{
  a logical value indicating whether additionally calculating the p-values of the tests specified in \code{methods} based on the test statistics' theoretical null distributions. Regardless of this argument, the resampling-based p-values are always produced by this function for the tests specified in \code{methods}.
}
}
\details{
The hybrid test statistic is defined as the minimum p-value among the publication bias tests considered in the set specified by the argument \code{methods}. Note that the minimum p-value is no longer a genuine p-value because it cannot control the type I error rate. Its p-value needs to be calculated via the resampling approach. See more details in Lin (2020).
}
\value{
This function returns a list containing p-values of the publication bias tests specified in \code{methods} as well as the hybrid test. Each element's name in this list has the format of \code{pval.x}, where \code{x} stands for the character string corresponding to a certain publication bias test, such as \code{rank}, \code{reg}, \code{skew}, etc. The hybrid test's p-value has the name \code{pval.hybrid}. If \code{theo.pval} = \code{TRUE}, additional elements of p-values of the tests in \code{methods} based on theorectical null distributions are included in the produced list; their names have the format of \code{pval.x.theo}. Another p-value of the hybrid test is also produced based on them; its corresponding element has the name \code{pval.hybrid.theo}.
}
\references{
Begg CB, Mazumdar M (1994). "Operating characteristics of a rank correlation test for publication bias." \emph{Biometrics}, \bold{50}(4), 1088--1101. <\doi{10.2307/2533446}>

Duval S, Tweedie R (2000). "A nonparametric `trim and fill' method of accounting for publication bias in meta-analysis." \emph{Journal of the American Statistical Association}, \bold{95}(449), 89--98. <\doi{10.1080/01621459.2000.10473905}>

Egger M, Davey Smith G, Schneider M, Minder C (1997). "Bias in meta-analysis detected by a simple, graphical test." \emph{BMJ}, \bold{315}(7109), 629--634. <\doi{10.1136/bmj.315.7109.629}>

Harbord RM, Egger M, Sterne JAC (2006). "A modified test for small-study effects in meta-analyses of controlled trials with binary endpoints." \emph{Statistics in Medicine}, \bold{25}(20), 3443--3457. <\doi{10.1002/sim.2380}>

Jin Z-C, Wu C, Zhou X-H, He J (2014). "A modified regression method to test publication bias in meta-analyses with binary outcomes." \emph{BMC Medical Research Methodology}, \bold{14}, 132. <\doi{10.1186/1471-2288-14-132}>

Lin L (2020). "Hybrid test for publication bias in meta-analysis." \emph{Statistical Methods in Medical Research}, \bold{29}(10), 2881--2899. <\doi{10.1177/0962280220910172}>

Lin L, Chu H (2018). "Quantifying publication bias in meta-analysis." \emph{Biometrics}, \bold{74}(3), 785--794. <\doi{10.1111/biom.12817}>

Macaskill P, Walter SD, Irwig L (2001). "A comparison of methods to detect publication bias in meta-analysis." \emph{Statistics in Medicine}, \bold{20}(4), 641--654. <\doi{10.1002/sim.698}>

Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2006). "Comparison of two methods to detect publication bias in meta-analysis." \emph{JAMA}, \bold{295}(6), 676--680. <\doi{10.1001/jama.295.6.676}>

Rucker G, Schwarzer G, Carpenter J (2008). "Arcsine test for publication bias in meta-analyses with binary outcomes." \emph{Statistics in Medicine}, \bold{27}(5), 746--763. <\doi{10.1002/sim.2971}>

Schwarzer G, Antes G, Schumacher M (2007). "A test for publication bias in meta-analysis with sparse binary data." \emph{Statistics in Medicine}, \bold{26}(4), 721--733. <\doi{10.1002/sim.2588}>

Tang J-L, Liu JLY (2000). "Misleading funnel plot for detection of bias in meta-analysis." \emph{Journal of Clinical Epidemiology}, \bold{53}(5), 477--484. <\doi{10.1016/S0895-4356(99)00204-8}>

Thompson SG, Sharp SJ (1999). "Explaining heterogeneity in meta-analysis: a comparison of methods." \emph{Statistics in Medicine}, \bold{18}(20), 2693--2708. <\doi{10.1002/(SICI)1097-0258(19991030)18:20<2693::AID-SIM235>3.0.CO;2-V}>
}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{pb.bayesian.binary}}, \code{\link{pb.hybrid.generic}}
}
\examples{
## meta-analysis of (log) odds ratios
data("dat.whiting")
# based on only 10 resampling iterations
set.seed(1234)
out.whiting <- pb.hybrid.binary(n00 = n00, n01 = n01,
  n10 = n10, n11 = n11, data = dat.whiting, iter.resam = 10)
out.whiting
# increases the number of resampling iterations to 10000,
#  taking longer time
}
\keyword{publication bias/small-study effects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metahet.hybrid.R
\name{metahet.hybrid}
\alias{metahet.hybrid}

\title{Alternative Tests and Measures for Between-Study Inconsistency in Meta-Analysis}
\description{
Performs several alternative tests (including a hybrid test) and calculates the associated measures for assessing between-study inconsistency in meta-analysis.
}

\usage{
metahet.hybrid(y, s2, data, gam, testinf = TRUE, iter.resam = 200)
}

\arguments{
\item{y}{Numeric vector of observed effect estimates, or the name of a column in \code{data}.}

\item{s2}{Numeric vector of within-study variances corresponding to \code{y}, or the name of a column in \code{data}.}

\item{data}{Optional data frame containing the variables \code{y} and \code{s2}.}

\item{gam}{Numeric vector of power parameters \eqn{\gamma}, e.g., \code{1:8}.}

\item{testinf}{Logical; if \code{TRUE}, includes the \eqn{Q_{\infty}} statistic (maximum standardized deviation).}

\item{iter.resam}{Integer; number of bootstrap resamples used to estimate expected values and p-values.}
}

\value{
A list named \code{out} containing:
\itemize{
\item \code{E_gamma}: Vector of E-gamma statistics for each \eqn{\gamma}, including \eqn{E_{\infty}} and the hybrid \eqn{E_{Lhyb}} term.
\item \code{p_gamma}: Bootstrap p-values corresponding to each \eqn{\gamma}.
\item \code{p_hyb}: Hybrid p-value combining all \eqn{\gamma} values.
\item \code{gam}: Vector of power parameters used in the tests.
\item \code{table}: Combined summary table for reporting.
}
}

\details{
The function generalizes the adaptive sum of powered score (aSPU) framework by combining standardized powered deviations across studies. Specifically, for each power parameter gamma, it calculates a statistic that measures the average standardized deviation of individual study effects from the overall mean, with greater gamma values placing more weight on extreme deviations.

For each gamma, the expected value of this statistic is obtained through a parametric bootstrap under the null hypothesis of homogeneity. The observed statistic is then standardized by subtracting its expected value and dividing by the observed total deviation, resulting in the expected gamma (E-gamma) measure.

In addition, a hybrid test is computed by combining the p-values across all gamma values using the minimum-p combination rule. It provides an overall summary of between-study inconsistency that integrates evidence from multiple power levels.
}

\references{
Pan W, Kim J, Zhang Y, Shen X, Wei P (2014). "A powerful and adaptive association test for rare variants." \emph{Genetics}, \bold{197}(4), 1081--1095. <\doi{10.1534/genetics.114.165035}>

Xu G, Lin L, Wei P, Pan W (2016). "An adaptive two-sample test for high-dimensional means." \emph{Biometrika}, \bold{103}(3), 609--624. <\doi{10.1093/biomet/asw029}>

Yu Z, Xiao M, Xing X, Lin L (2025). "Alternative tests and measures for between-study inconsistency in meta-analysis." \emph{BMC Medical Research Methodology}, \bold{25}, 261. <\doi{10.1186/s12874-025-02719-7}>
}

\author{
Zhiyuan Yu, Xing Xing, Lifeng Lin 
}

\seealso{
\code{\link{metahet}}
}
\examples{
data("dat.hughes")
set.seed(123)
## increase iter.resam to reduce Monte Carlo error due to resampling
metahet.hybrid(y, s2, data = dat.hughes, gam = 1:8,
  testinf = TRUE, iter.resam = 200)
}
\keyword{heterogeneity}

\name{print.meta.dt}
\alias{print.meta.dt}
\title{
Print Method for "meta.dt" Objects
}
\description{
Prints information about a meta-analysis of diagnostic tests.
}
\usage{
\method{print}{meta.dt}(x, digits = 3, ...)
}
\arguments{
  \item{x}{
  an object of class \code{"meta.dt"} produced by the function \code{\link{meta.dt}}.
}
  \item{digits}{
  an integer specifying the number of decimal places to which the printed results should be rounded.
}
  \item{\dots}{
  other arguments.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
None.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{meta.dt}}, \code{\link{plot.meta.dt}}
}
%%\examples{
%%}
\keyword{print}

\name{dat.williams}
\alias{dat.williams}
\docType{data}
\title{
A Meta-Analysis on the Effect of Pharmacotherapy for Social Anxiety Disorder
}
\description{
This dataset serves as an example of meta-analysis of (log) relative risks.
}
\usage{data("dat.williams")}
\format{
  A data frame with 20 studies with the following 7 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of log relative risks.}
    \item{\code{s2}}{sample variances of log relative risks.}
    \item{\code{n1}}{sample sizes in treatment group 1 (medication).}
    \item{\code{n2}}{sample sizes in treatment group 2 (placebo).}
    \item{\code{r1}}{event counts in treatment group 1.}
    \item{\code{r2}}{event counts in treatment group 2.}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Williams T, Hattingh CJ, Kariuki CM, Tromp SA, van Balkom AJ, Ipser JC, Stein DJ (2017). "Pharmacotherapy for social anxiety disorder (SAnD)." \emph{Cochrane Database of Systematic Reviews}, \bold{10}, Art.{\sspace}No.: CD001206. <\doi{10.1002/14651858.CD001206.pub3}>
}
\keyword{dataset}

\name{maprop.twostep}
\alias{maprop.twostep}
\title{
Meta-Analysis of Proportions Using Two-Step Methods
}
\description{
Performs a meta-analysis of proportions using conventional two-step methods with various data transformations.
}
\usage{
maprop.twostep(e, n, data, link = "logit", method = "ML", alpha = 0.05,
               pop.avg = TRUE, int.approx = 10000, b.iter = 1000,
               seed = 1234)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{e}{
  a numeric vector specifying the event counts in the collected studies.
}
  \item{n}{
  a numeric vector specifying the sample sizes in the collected studies.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{e} and \code{n}, should be specified as their corresponding column names in \code{data}.
}
  \item{link}{
  a character string specifying the data transformation for each study's proportion used in the two-step method, which can be one of \code{"log"} (log transformation), \code{"logit"} (logit transformation, the default), \code{"arcsine"} (arcsine transformation), and \code{"double.arcsine"} (Freeman--Tukey double-arcsine transformation).
}
  \item{method}{
  a character string specifying the method to perform the meta-analysis, which is passed to the argument \code{method} in the function \code{\link[metafor]{rma.uni}} in the package \pkg{metafor}. It can be one of \code{"ML"} (maximum likelihood, the default), \code{"REML"} (restricted maximum likelihood), and many other options; see more details in the manual of \pkg{metafor}. The default is set to \code{"ML"} for consistency with the function \code{\link{maprop.glmm}}, where generalized linear mixed models are often estimated via the maximum likelihood approach. For the two-step method, users might also use \code{"REML"} because the restricted maximum likelihood estimation may have superior performance in many cases.
}
  \item{alpha}{
  a numeric value specifying the statistical significance level.
}
  \item{pop.avg}{
  a logical value indicating whether the population-averaged proportion and its confidence interval are to be produced. This quantity is the marginal mean of study-specific proportions, while the commonly-reported overall proportion usually represents the median (or interpreted as a conditional measure); see more details about this quantity in Section 13.2.3 in Agresti (2013), Chu et al.{\sspace}(2012), Lin and Chu (2020), and Zeger et al.{\sspace}(1988). If \code{pop.avg} = \code{TRUE} (the default), the bootstrap resampling is used to produce the confidence interval of the population-averaged proportion; the confidence interval of the commonly-reported median proportion will be also produced, in addition to its conventional confidence interval (by back-transforming the Wald-type confidence interval derived on the scale specified by \code{link}).
}
  \item{int.approx}{
  an integer specifying the number of independent standard normal samples for numerically approximating the integration involved in the calculation of the population-averaged proportion; see details in Lin and Chu (2020). It is only used when \code{pop.avg} = \code{TRUE}. For the commonly-used data transformations available for \code{link}, the population-averaged proportion does not have a closed form.
}
  \item{b.iter}{
  an integer specifying the number of bootstrap iterations; it is only used when \code{pop.avg} = \code{TRUE}.
}
  \item{seed}{
  an integer for specifying the seed of the random number generation for reproducibility during the bootstrap resampling (and numerical approximation for the population-averaged proportion); it is only used when \code{pop.avg} = \code{TRUE}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
This function returns a list containing the point and interval estimates of the overall proportion. Specifically, \code{prop.c.est} is the commonly-reported median (or conditional) proportion, and \code{prop.c.ci} is its confidence interval. If \code{pop.avg} = \code{TRUE}, the following additional elements will be also in the produced list: \code{prop.c.ci.b} is the bootstrap confidence interval of the commonly-reported median (conditional) proportion, \code{prop.m.est} is the point estimate of the population-averaged (marginal) proportion, \code{prop.m.ci.b} is the bootstrap confidence interval of the population-averaged (marginal) proportion, and \code{b.w.e} is a vector of two numeric values, indicating the counts of warnings and errors occurred during the bootstrap iterations. Moreover, if the Freeman--Tukey double-arcsine transformation (\code{link} = \code{"double.arcsine"}) is used, the back-transformation will be implemented at four values as the overall sample size: the harmonic, geometric, and arithmetic means of the study-specific sample sizes, and the inverse of the synthesized result's variance. See details in Barendregt et al.{\sspace}(2013) and Schwarzer et al.{\sspace}(2019).
}
\references{
Agresti A (2013). \emph{Categorical Data Analysis}. Third edition. John Wiley & Sons, Hoboken, NJ.

Barendregt JJ, Doi SA, Lee YY, Norman RE, Vos T (2013). "Meta-analysis of prevalence." \emph{Journal of Epidemiology and Community Health}, \bold{67}(11), 974--978. <\doi{10.1136/jech-2013-203104}>

Freeman MF, Tukey JW (1950). "Transformations related to the angular and the square root." \emph{The Annals of Mathematical Statistics}, \bold{21}(4), 607--611. <\doi{10.1214/aoms/1177729756}>

Lin L, Chu H (2020). "Meta-analysis of proportions using generalized linear mixed models." \emph{Epidemiology}, \bold{31}(5), 713--717. <\doi{10.1097/ede.0000000000001232}>

Miller JJ (1978). "The inverse of the Freeman--Tukey double arcsine transformation." \emph{The American Statistician}, \bold{32}(4), 138. <\doi{10.1080/00031305.1978.10479283}>

Schwarzer G, Chemaitelly H, Abu-Raddad LJ, Rucker G (2019). "Seriously misleading results using inverse of Freeman-Tukey double arcsine transformation in meta-analysis of single proportions." \emph{Research Synthesis Methods}, \bold{10}(3), 476--483. <\doi{10.1002/jrsm.1348}>

Viechtbauer W (2010). "Conducting meta-analyses in R with the metafor package." \emph{Journal of Statistical Software}, \bold{36}, 3. <\doi{10.18637/jss.v036.i03}>

Zeger SL, Liang K-Y, Albert PS (1988). "Models for longitudinal data: a generalized estimating equation approach." \emph{Biometrics}, \bold{44}(4), 1049--1060. <\doi{10.2307/2531734}>
}
%%\author{
%%  ~~who you are~~
%%}
\note{
This function implements the two-step method for the meta-analysis of proportions via the \code{\link[metafor]{rma.uni}} function in the package \pkg{metafor}. It is possible that the algorithm of the maximum likelihood or restricted maximum likelihood estimation may not converge for some bootstrapped meta-analyses when \code{pop.avg} = \code{TRUE}, and the \code{\link[metafor]{rma.uni}} function may report warnings or errors about the convergence issue. The bootstrap iterations are continued until \code{b.iter} replicates without any warnings or errors are obtained; those replicates with any warnings or errors are discarded.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{maprop.glmm}}
}
\examples{
# chorioamnionitis data
data("dat.chor")
# two-step method with the logit transformation
out.chor.twostep.logit <- maprop.twostep(e, n, data = dat.chor,
  link = "logit", b.iter = 10, seed = 1234)
out.chor.twostep.logit
# not calculating the population-averaged (marginal) proportion,
#  without bootstrap resampling
out.chor.twostep.logit <- maprop.twostep(e, n, data = dat.chor,
  link = "logit", pop.avg = FALSE)
out.chor.twostep.logit
\donttest{
# increases the number of bootstrap iterations to 1000,
#  taking longer time
out.chor.twostep.logit <- maprop.twostep(e, n, data = dat.chor,
  link = "logit", b.iter = 1000, seed = 1234)
out.chor.twostep.logit

# two-step method with the log transformation
out.chor.twostep.log <- maprop.twostep(e, n, data = dat.chor,
  link = "log", b.iter = 10, seed = 1234)
out.chor.twostep.log
# two-step method with the arcsine transformation
out.chor.twostep.arcsine <- maprop.twostep(e, n, data = dat.chor,
  link = "arcsine", b.iter = 10, seed = 1234)
out.chor.twostep.arcsine
# two-step method with the Freeman--Tukey double-arcsine transformation
out.chor.twostep.double.arcsine <- maprop.twostep(e, n, data = dat.chor,
  link = "double.arcsine", b.iter = 10, seed = 1234)
out.chor.twostep.double.arcsine


# depression data
data("dat.beck17")
out.beck17.twostep.log <- maprop.twostep(e, n, data = dat.beck17,
  link = "log", b.iter = 10, seed = 1234)
out.beck17.twostep.log
out.beck17.twostep.logit <- maprop.twostep(e, n, data = dat.beck17,
  link = "logit", b.iter = 10, seed = 1234)
out.beck17.twostep.logit
out.beck17.twostep.arcsine <- maprop.twostep(e, n, data = dat.beck17,
  link = "arcsine", b.iter = 10, seed = 1234)
out.beck17.twostep.arcsine
out.beck17.twostep.double.arcsine <- maprop.twostep(e, n, data = dat.beck17,
  link = "double.arcsine", b.iter = 10, seed = 1234)
out.beck17.twostep.double.arcsine
}
}
\keyword{proportion}

\name{dat.adams}
\alias{dat.adams}
\docType{data}
\title{
A Meta-Analysis on the Effects of Fluvastatin 20 mg/day on High-Density Lipoprotein
}
\description{
This meta-analysis serves as an example to illustrate the usage of the function \code{\link{meta.pen}}.
}
\usage{data("dat.ha")}
\format{
  A data frame containing 32 studies.
  \describe{
    \item{\code{y}}{the observed effect size (mean difference) for each study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Adams SP, Sekhon SS, Tsang M, Wright JM (2018). "Fluvastatin for lowering lipids." \emph{Cochrane Database of Systematic Reviews}, \bold{7}. Art.{\sspace}No.: CD012282. <\doi{10.1002/14651858.CD012282.pub2}>
}
\keyword{dataset}

\name{meta.dt}
\alias{meta.dt}
\title{
Meta-Analysis of Diagnostic Tests
}
\description{
Performs a meta-analysis of diagnostic tests using approaches described in Reitsma et al.\sspace(2005) and Chu and Cole (2006).
}
\usage{
meta.dt(tp, fp, fn, tn, data, method = "biv.glmm", alpha = 0.05, ...)
}
\arguments{
  \item{tp}{
  counts of true positives.
}
  \item{fp}{
  counts of false positives.
}
  \item{fn}{
  counts of false negatives.
}
  \item{tn}{
  counts of true negatives.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{tp}, \code{fp}, \code{fn}, and \code{tn}, should be specified as their corresponding column names in \code{data}.
}
  \item{method}{
  a character string specifying the method used to implement the meta-analysis of diagnostic tests. It should be one of \code{"s.roc"} (summary ROC approach), \code{"biv.lmm"} (bivariate linear mixed model), and \code{"biv.glmm"} (bivariate generalized linear mixed model, the default). See details.
}
  \item{alpha}{
  a numeric value specifying the statistical significance level.
}
  \item{\dots}{
  other arguments that can be passed to the function \code{\link[stats]{lm}} (when \code{method} = \code{"s.roc"}), the function \code{\link[metafor]{rma.mv}} in the package \pkg{metafor} (when \code{method} = \code{"biv.lmm"}), or the function \code{\link[lme4]{glmer}} in the package \pkg{lme4} (when \code{method} = \code{"biv.glmm"}).
}
}
\details{
Suppose a meta-analysis of diagnostic tests contains \eqn{N} studies. Each study reports the counts of true positives, false positives, false negatives, and true negatives, denoted by \eqn{TP_i}, \eqn{FP_i}, \eqn{FN_i}, and \eqn{TN_i}, respectively. The study-specific estimates of sensitivity and specificity are calculated as \eqn{Se_i = TP_i/(TP_i + FN_i)} and \eqn{Sp_i = TN_i/(FP_i + TN_i)} for \eqn{i = 1, \cdots, N}. They are analyzed on the logarithmic scale in the meta-analysis. When using the summary ROC (receiver operating characteristic) approach or the bivariate linear mixed model, 0.5 needs to be added to all four counts in a study when at least one count is zero.

The summary ROC approach first calculates
\deqn{D_i = \log \left( \frac{Se_i}{1 - Se_i} \right) + \log \left( \frac{Sp_i}{1 - Sp_i} \right);}
\deqn{S_i = \log \left( \frac{Se_i}{1 - Se_i} \right) - \log \left( \frac{Sp_i}{1 - Sp_i} \right),}
where \eqn{D_i} represents the log diagnostic odds ratio (DOR) in study \eqn{i}. A linear regression is then fitted:
\deqn{D_i = \alpha + \beta \cdot S_i.}
The regression could be either unweighted or weighted; this function performs both versions. If weighted, the study-specific weights are the inverse of the variances of \eqn{D_i}, i.e., \eqn{1/TP_i + 1/FP_i + 1/FN_i + 1/TN_i}. Based on the estimated regression intercept \eqn{\hat{\alpha}} and slope \eqn{\hat{\beta}}, one may obtain the DOR at mean of \eqn{S_i}, Q point, summary ROC curve, and area under the curve (AUC). The Q point is the point on the summary ROC curve where sensitivity and specificity are equal. The ROC curve is given by
\deqn{Se = \left\{ 1 + e^{-\hat{\alpha}/(1 - \hat{\beta})} \cdot [Sp/(1 - Sp)]^{(1 + \hat{\beta})/(1 - \hat{\beta})} \right\}^{-1}.}
See more details of the summary ROC approach in Moses et al.\sspace(1993) and Irwig et al.\sspace(1995).

The bivariate linear mixed model described in Reitsma et al.\sspace(2005) assumes that the logit sensitivity and logit specificity independently follow normal distributions within each study: \eqn{g(Se_i) \sim N(\theta_{i, Se}, s_{i, Se}^2)} and \eqn{g(Sp_i) \sim N(\theta_{i, Sp}, s_{i, Sp}^2)}, where \eqn{g(\cdot)} denotes the logit function. The within-study variances are calculated as \eqn{s_{i, Se}^2 = 1/TP_i + 1/FN_i} and \eqn{s_{i, Sp}^2 = 1/FP_i + 1/TN_i}. The parameters \eqn{\theta_{i, Se}} and \eqn{\theta_{i, Sp}} are the underlying true sensitivity and specificity (on the logit scale) in study \eqn{i}. They are assumed to be random effects, jointly following a bivariate normal distribution:
\deqn{\left(\theta_{i, Se}, \theta_{i, Sp}\right)^\prime \sim N\left((\mu_{Se}, \mu_{Sp})^\prime, \mathbf{\Sigma} \right),}
where \eqn{\mathbf{\Sigma}} is the between-study variance-covariance matrix. The diagonal elements of \eqn{\mathbf{\Sigma}} are \eqn{\sigma_{Se}^2} and \eqn{\sigma_{Sp}^2}, representing the heterogeneity variances of sensitivities and specificities (on the logit scale), respectively. The correlation coefficient is \eqn{\rho}.

The bivariate generalized linear mixed model described in Chu and Cole (2006) refines the bivariate linear mixed model by directly modeling the counts of true positives and true negatives. This approach does not require the assumption that the logit sensitivity and logit specificity approximately follow normal distributions within studies, which could be seriously violated in the presence of small data counts. It also avoids corrections for zero counts. Specificially, the counts of true positives and true negatives are modeled using binomial likelihoods:
\deqn{TP_i \sim Bin(TP_i + FN_i, Se_i);}
\deqn{TN_i \sim Bin(FP_i + TN_i, Sp_i);}
\deqn{\left( g(Se_i), g(Sp_i) \right)^\prime \sim N\left((\mu_{Se}, \mu_{Sp})^\prime, \mathbf{\Sigma} \right).}
See more details in Chu and Cole (2006) and Ma et al.\sspace(2016).

For both the bivariate linear mixed model and bivariate generalized linear mixed model, \eqn{\mu_{Se}} and \eqn{\mu_{Sp}} represent the overall sensitivity and specificity (on the logit scale) across studies, respectively, and \eqn{\mu_{Se} + \mu_{Sp}} represents the log DOR. The summary ROC curve may be constructed as
\deqn{Se = \left\{ 1 + e^{-\hat{\mu}_{Se} + \hat{\mu}_{Sp} \cdot \hat{\rho} \hat{\sigma}_{Se}/\hat{\sigma}_{Sp}} \cdot [Sp/(1 - Sp)]^{-\rho \hat{\sigma}_{Se}/\hat{\sigma}_{Sp}} \right\}^{-1}.}
}
\value{
This function returns a list of the meta-analysis results. When \code{method} = \code{"s.roc"}, the list consists of the regression intercept (\code{inter.unwtd}), slope (\code{slope.unwtd}), their variance-covariance matrix (\code{vcov.unwtd}), DOR at mean of \eqn{S_i} (\code{DOR.meanS.unwtd}) with its confidence interval (\code{DOR.meanS.unwtd.ci}), Q point (\code{Q.unwtd}) with its confidence interval (\code{Q.unwtd.ci}), and AUC (\code{AUC.unwtd}) for the unweighted regression; it also consists of the counterparts for the weighted regression. When \code{method} = \code{"biv.lmm"} or \code{"biv.glmm"}, the list consists of the overall sensitivity (\code{sens.overall}) with its confidence interval (\code{sens.overall.ci}), overall specificity (\code{spec.overall}) with its confidence interval (\code{spec.overall.ci}), overall DOR (\code{DOR.overall}) with its confidence interval (\code{DOR.overall.ci}), AUC (\code{AUC}), estimated \eqn{\mu_{Se}} (\code{mu.sens}), \eqn{\mu_{Sp}} (\code{mu.spec}), their variance-covariance matrix (\code{mu.vcov}), estimated \eqn{\sigma_{Se}} (\code{sig.sens}), \eqn{\sigma_{Sp}} (\code{sig.spec}), and \eqn{\rho} (\code{rho}). In addition, the list includes the method used to perform the meta-analysis of diagnostic tests (\code{method}), significance level (\code{alpha}), and original data (\code{data}).
}
\references{
Chu H, Cole SR (2006). "Bivariate meta-analysis of sensitivity and specificity with sparse data: a generalized linear mixed model approach." \emph{Journal of Clinical Epidemiology}, \bold{59}(12), 1331--1332. <\doi{10.1016/j.jclinepi.2006.06.011}>

Irwig L, Macaskill P, Glasziou P, Fahey M (1995). "Meta-analytic methods for diagnostic test accuracy." \emph{Journal of Clinical Epidemiology}, \bold{48}(1), 119--130. <\doi{10.1016/0895-4356(94)00099-C}>

Ma X, Nie L, Cole SR, Chu H (2016). "Statistical methods for multivariate meta-analysis of diagnostic tests: an overview and tutorial." \emph{Statistical Methods in Medical Research}, \bold{25}(4), 1596--1619. <\doi{10.1177/0962280213492588}>

Moses LE, Shapiro D, Littenberg B (1993). "Combining independent studies of a diagnostic test into a summary ROC curve: data-analytic approaches and some additional considerations." \emph{Statistics in Medicine}, \bold{12}(14), 1293--1316. <\doi{10.1002/sim.4780121403}>

Reitsma JB, Glas AS, Rutjes AWS, Scholten RJPM, Bossuyt PM, Zwinderman AH (2005). "Bivariate analysis of sensitivity and specificity produces informative summary measures in diagnostic reviews." \emph{Journal of Clinical Epidemiology}, \bold{58}(10), 982--990. <\doi{10.1016/j.jclinepi.2005.02.022}>

Zhang H, Lu N, Feng C, Thurston SW, Xia Y, Zhu L, Tu XM (2011). "On fitting generalized linear mixed-effects models for binary responses using different statistical packages." \emph{Statistics in Medicine}, \bold{30}(20), 2562--2572. <\doi{10.1002/sim.4265}>
}
\author{
Lifeng Lin, Kristine J.{\sspace}Rosenberger
}
\note{
The original articles by Reitsma et al.\sspace(2005) and Chu and Cole (2006) used SAS to implement (generalized) linear mixed models (specifically, \code{PROC MIXED} and \code{PROC NLMIXED}); this function imports \code{\link[metafor]{rma.mv}} from the package \pkg{metafor} and \code{\link[lme4]{glmer}} from the package \pkg{lme4} for implementing these models. The estimation approaches adopted in SAS and the R packages \pkg{metafor} and \pkg{lme4} may differ, which may impact the results. See, for example, Zhang et al.\sspace(2011).
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{maprop.twostep}}, \code{\link{meta.biv}}, \code{\link{plot.meta.dt}}, \code{\link{print.meta.dt}}
}
\examples{
data("dat.scheidler")
out1 <- meta.dt(tp, fp, fn, tn, data = dat.scheidler[dat.scheidler$dt == "MRI",],
  method = "s.roc")
out1
plot(out1)
out2 <- meta.dt(tp, fp, fn, tn, data = dat.scheidler[dat.scheidler$dt == "MRI",],
  method = "biv.lmm")
out2
plot(out2, predict = TRUE)
out3 <- meta.dt(tp, fp, fn, tn, data = dat.scheidler[dat.scheidler$dt == "MRI",],
  method = "biv.glmm")
out3
plot(out3, add = TRUE, studies = FALSE,
  col.roc = "blue", col.overall = "blue", col.confid = "blue",
  predict = TRUE,col.predict = "blue")

data("dat.smith")
out4 <- meta.dt(tp, fp, fn, tn, data = dat.smith, method = "biv.glmm")
out4
plot(out4, predict = TRUE)
}
\keyword{diagnostic test}
\keyword{generalized linear mixed model}

\name{dat.kaner}
\alias{dat.kaner}
\docType{data}
\title{
A Meta-Analysis on the Effect of Brief Alcohol Interventions vs. Control in Primary Care Populations
}
\description{
This dataset serves as an example of meta-analysis of risk differences.
}
\usage{data("dat.kaner")}
\format{
  A data frame with 13 studies with the following 7 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of risk differences.}
    \item{\code{s2}}{sample variances of risk differences.}
    \item{\code{n1}}{sample sizes in treatment group 1 (brief alcohol interventions).}
    \item{\code{n2}}{sample sizes in treatment group 2 (control).}
    \item{\code{r1}}{event counts in treatment group 1.}
    \item{\code{r2}}{event counts in treatment group 2.}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Kaner EF, Dickinson HO, Beyer FR, Campbell F, Schlesinger C, Heather N, Saunders JB, Burnand B, Pienaar ED (2007). "Effectiveness of brief alcohol interventions in primary care populations." \emph{Cochrane Database of Systematic Reviews}, \bold{2}, Art.{\sspace}No.: CD004148. <\doi{10.1002/14651858.CD004148.pub3}>
}
\keyword{dataset}

\name{dat.slf}
\alias{dat.slf}
\docType{data}
\title{
A Meta-Analysis on the Effect of Nicotine Gum for Smoking Cessation
}
\description{
This meta-analysis serves as an example to illustrate function usage in the package \pkg{altmeta}.
}
\usage{data("dat.slf")}
\format{
  A data frame containing 56 studies with the observed effect sizes and their within-study variances.
  \describe{
    \item{\code{y}}{the observed effect size for each collected study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Stead LF, Perera R, Bullen C, Mant D, Hartmann-Boyce J, Cahill K, Lancaster T (2012). "Nicotine replacement therapy for smoking cessation." \emph{Cochrane Database of Systematic Reviews}, \bold{11}. Art.{\sspace}No.: CD000146. <\doi{10.1002/14651858.CD000146.pub4}>
}
\keyword{dataset}

\name{dat.hughes}
\alias{dat.hughes}
\docType{data}
\title{
A Meta-Analysis on the Effects of Ovulation Suppression for Endometriosis in Subfertile Women
}
\description{
This meta-analysis serves as an example to illustrate the usage of the function \code{\link{metahet.hybrid}}.
}
\usage{data("dat.hughes")}
\format{
  A data frame containing 11 studies.
  \describe{
    \item{\code{y}}{the observed effect size (log odds ratio) for each study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Hughes E, Brown J, Collins JJ, et al. (2007). "Ovulation suppression for endometriosis for women with subfertility." \emph{Cochrane Database of Systematic Reviews}, \bold{7}. Art.{\sspace}No.: CD000155. <\doi{10.1002/14651858.CD000155.pub2}>
}
\keyword{dataset}

\name{dat.aex}
\alias{dat.aex}
\docType{data}
\title{
A Meta-Analysis for Evaluating the Effect of Aerobic Exercise on Visceral Adipose Tissue Content/Volume
}
\description{
This meta-analysis serves as an example to illustrate function usage in the package \pkg{altmeta}.
}
\usage{data("dat.aex")}
\format{
  A data frame containing 29 studies with the observed effect sizes and their within-study variances.
  \describe{
    \item{\code{y}}{the observed effect size for each collected study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Ismail I, Keating SE, Baker MK, Johnson NA (2012). "A systematic review and meta-analysis of the effect of aerobic vs.{\sspace}resistance exercise training on visceral fat." \emph{Obesity Reviews}, \bold{13}(1), 68--91. <\doi{10.1111/j.1467-789X.2011.00931.x}>
}
\keyword{dataset}

\name{dat.sc}
\alias{dat.sc}
\docType{data}
\title{
A Network Meta-Analysis on Smoking Cessation
}
\description{
The dataset is extracted from Lu and Ades (2006); it was initially reported by Hasselblad (1998) (without performing a formal network meta-analysis).
}
\usage{data("dat.sc")}
\format{
  A dataset of network meta-analysis with binary outcomes, containing 24 studies and 4 treatments.
  \describe{
    \item{\code{sid}}{study IDs.}
    \item{\code{tid}}{treatment IDs.}
    \item{\code{r}}{event counts.}
    \item{\code{n}}{sample sizes.}
  }
}
\details{
Treatment IDs represent: 1)no contact; 2) selfhelp; 3) individual counseling; and 4) group counseling.
}
\source{
Hasselblad V (1998). "Meta-analysis of multitreatment studies." \emph{Medical Decision Making}, \bold{18}(1), 37--43. <\doi{10.1177/0272989X9801800110}>

Lu G, Ades AE (2006). "Assessing evidence inconsistency in mixed treatment comparisons." \emph{Journal of the American Statistical Association}, \bold{101}(474), 447--459. <\doi{10.1198/016214505000001302}>
}
\keyword{dataset}

\name{dat.fib}
\alias{dat.fib}
\docType{data}
\title{
A Multivariate Meta-Analysis by the Fibrinogen Studies Collaboration
}
\description{
This multivariate meta-analysis serves as an example to illustrate function usage in the package \pkg{altmeta}. It consists of 31 studies with 4 outcomes.
}
\usage{data("dat.fib")}
\format{
  A list containing three elements, \code{y}, \code{S}, and \code{sd}.
  \describe{
    \item{\code{y}}{a 31 x 4 numeric matrix containing the observed effect sizes; the rows represent studies and the columns represent outcomes.}
    \item{\code{S}}{a list containing 31 elements; each element is within-study covariance matrix of the corresponding study.}
    \item{\code{sd}}{a 31 x 4 numeric matrix containing the within-study standard deviations; the rows represent studies and the columns represent outcomes.}
  }
}
\source{
Fibrinogen Studies Collaboration (2004). "Collaborative meta-analysis of prospective studies of plasma fibrinogen and cardiovascular disease." \emph{European Journal of Cardiovascular Prevention and Rehabilitation}, \bold{11}(1), 9--17. <\doi{10.1097/01.hjr.0000114968.39211.01}>

Fibrinogen Studies Collaboration (2005). "Plasma fibrinogen level and the risk of major cardiovascular diseases and nonvascular mortality: an individual participant meta-analysis." \emph{JAMA}, \bold{294}(14), 1799--1809. <\doi{10.1001/jama.294.14.1799}>
}
\keyword{dataset}

\name{plot.meta.dt}
\alias{plot.meta.dt}
\title{
Plot for Meta-Analysis of Diagnostic Tests
}
\description{
Visualizes meta-analysis of diagnostic tests by presenting summary results, such as ROC (receiver operating characteristic) curve, overall sensitivity and overall specificity (1 \eqn{-} specificity), and their confidence and prediction regions.
}
\usage{
\method{plot}{meta.dt}(x, add = FALSE, xlab, ylab, alpha,
     studies = TRUE, cex.studies, col.studies, pch.studies,
     roc, col.roc, lty.roc, lwd.roc, weight = FALSE,
     eqline, col.eqline, lty.eqline, lwd.eqline,
     overall = TRUE, cex.overall, col.overall, pch.overall,
     confid = TRUE, col.confid, lty.confid, lwd.confid,
     predict = FALSE, col.predict, lty.predict, lwd.predict, ...)
}
\arguments{
  \item{x}{
  an object of class \code{"meta.dt"} created by the function \code{\link{meta.dt}}.
}
  \item{add}{
  a logical value indicating if the plot is added to an already existing plot.
}
  \item{xlab}{
  a label for the x axis; the default is "1 - Specificity".
}
  \item{ylab}{
  a label for the y axis; the default is "Sensitivity".
}
  \item{alpha}{
  a numeric value specifying the statistical significance level for the confidence and prediction regions. If not specified, the plot uses the significance level stored in \code{x} (i.e., \code{x$alpha}).
}
  \item{studies}{
  a logical value indicating if the individual studies are presented in the plot.
}
  \item{cex.studies}{
  the size of points representing individual studies (the default is 1).
}
  \item{col.studies}{
  the color of points representing individual studies (the default is \code{"black"}).
}
  \item{pch.studies}{
  the symbol of points representing individual studies (the default is 1, i.e., circle).
}
  \item{roc}{
  a logical value indicating if the ROC curve is presented in the plot. The default is \code{TRUE} for the summary ROC approach (\code{x$method} = \code{"s.roc"}) and is \code{FALSE} for the bivariate (generalized) linear mixed model (\code{x$method} = \code{"biv.lmm"} or \code{"biv.glmm"}).
}
  \item{col.roc}{
  the color of the ROC curve (the default is \code{"black"}).
}
  \item{lty.roc}{
  the line type of the ROC curve (the default is 1, i.e., solid line).
}
  \item{lwd.roc}{
  the line width of the ROC curve (the default is 1).
}
  \item{weight}{
  a logical value indicating if the weighted (\code{TRUE}) or unweighted (\code{FALSE}, the default) regression is used for the summary ROC approach (when \code{x$method} is \code{"s.roc"}).
}
  \item{eqline}{
  a logical value indicating if the line of sensitivity equaling to specificity is presented in the plot.
}
  \item{col.eqline}{
  the color of the equality line (the default is \code{"black"}).
}
  \item{lty.eqline}{
  the type of the equality line (the default is 4, i.e., dot-dash line).
}
  \item{lwd.eqline}{
  the width of the equality line (the default is 1).
}
  \item{overall}{
  a logical value indicating if the overall sensitivity and overall specificity are presented in the plot. This and the following arguments are used for the bivariate (generalized) linear mixed model (\code{x$method} = \code{"biv.lmm"} or \code{"biv.glmm"}).
}
  \item{cex.overall}{
    the size of the point representing the overall sensitivity and overall specificity  (the default is 1).
}
  \item{col.overall}{
    the color of the point representing the overall sensitivity and overall specificity  (the default is \code{"black"}).
}
  \item{pch.overall}{
    the symbol of the point representing the overall sensitivity and overall specificity  (the default is 15, i.e., filled square).
}
  \item{confid}{
  a logical value indicating if the confidence region of the overall sensitivity and overall specificity is presented in the plot.
}
  \item{col.confid}{
  the line color of the confidence region (the default is \code{"black"}).
}
  \item{lty.confid}{
  the line type of the confidence region (the default is 2, i.e., dashed line).
}
  \item{lwd.confid}{
  the line width of the confidence region (the default is 1).
}
  \item{predict}{
  a logical value indicating if the prediction region of the overall sensitivity and overall specificity is presented in the plot.
}
  \item{col.predict}{
  the line color of the prediction region (the default is \code{"black"}).
}
  \item{lty.predict}{
  the line type of the prediction region (the default is 3, i.e., dotted line).
}
  \item{lwd.predict}{
  the line width of the prediction region (the default is 1).
}
  \item{\dots}{
  other arguments that can be passed to the function \code{\link[graphics]{plot.default}}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
None.
}
%%\references{
%% ~put references to the literature/web site here ~
%%}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{meta.dt}}, \code{\link{print.meta.dt}}
}
%%\examples{
%%}
\keyword{diagnostic test}
\keyword{plot}

\name{ssfunnel}
\alias{ssfunnel}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Contour-Enhanced Sample-Size-Based Funnel Plot
}
\description{
Generates contour-enhanced sample-size-based funnel plot for a meta-analysis of mean differences, standardized mean differences, (log) odds ratios, (log) relative risks, or risk differences.
}
\usage{
ssfunnel(y, s2, n, data, type, alpha = c(0.1, 0.05, 0.01, 0.001),
         log.ss = FALSE, sigma, p0, xlim, ylim, xlab, ylab,
         cols.contour, col.mostsig, cex.pts, lwd.contour, pch,
         x.legend, y.legend, cex.legend, bg.legend, ...)
}
\arguments{
  \item{y}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the observed effect sizes in the collected studies.
}
  \item{s2}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the within-study variances.
}
  \item{n}{
  a numeric vector or the corresponding column name in the argument \code{data}, specifying the study-specific total sample sizes.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y}, \code{s2}, and \code{n}, should be specified as their corresponding column names in \code{data}.
}
  \item{type}{
  a character string specifying the type of effect size, which should be one of \code{"md"} (mean difference), \code{"smd"} (standardized mean difference), \code{"lor"} (log odds ratio), \code{"lrr"} (log relative risk), and \code{"rd"} (risk difference).
}
  \item{alpha}{
  a numeric vector specifying the significance levels to be presented in the sample-size-based funnel plot.
}
  \item{log.ss}{
  a logical value indicating whether sample sizes are plotted on a logarithmic scale (\code{TRUE}) or not (\code{FALSE}, the default).
}
  \item{sigma}{
  a positive numeric value that is required for the mean difference (\code{type} = \code{"md"}), specifying a rough estimate of the common standard deviation of the samples' continuous outcomes in the two groups across studies. It is not used for other effect size types.
}
  \item{p0}{
  an optional numeric value specifying a rough estimate of the common event rate in the control group across studies. It is only used for the (log) odds ratio, (log) relative risk, and risk difference.
}
  \item{xlim}{
  the x limits \code{c(x1, x2)} of the plot.
}
  \item{ylim}{
  the y limits \code{c(y1, y2)} of the plot.
}
  \item{xlab}{
  a label for the x axis.
}
  \item{ylab}{
  a label for the y axis.
}
  \item{cols.contour}{
  a vector of character strings; they indicate colors of the contours to be presented in the sample-size-based funnel plot, and correspond to the significance levels specified in the argument \code{alpha}.
}
  \item{col.mostsig}{
  a character string specifying the color for the most significant result among the studies in the meta-analysis.
}
  \item{cex.pts}{
  the size of the points.
}
  \item{lwd.contour}{
  the width of the contours.
}
  \item{pch}{
  the symbol of the points.
}
  \item{x.legend}{
  the x co-ordinate or a keyword, such as \code{"topleft"} (the default), to be used to position the legend. It is passed to \code{\link[graphics]{legend}}.
}
  \item{y.legend}{
  the y co-ordinate to be used to position the legend (the default is \code{NULL}).
}
  \item{cex.legend}{
  the size of legend text.
}
  \item{bg.legend}{
  the background color for the legend box.
}
  \item{\dots}{
  other arguments that can be passed to \code{\link[graphics]{plot.default}}.
}
}
\details{
  A contour-enhanced sample-size-based funnel plot is generated; it presents study-specific total sample sizes against the corresponding effect size estimates. It is helpful to avoid the confounding effect caused by the intrinsic association between effect size estimates and standard errors in the conventional standard-error-based funnel plot. See details of the derivations of the contours in Lin (2019).
}
\value{
None.
}
\references{
Lin L (2019). "Graphical augmentations to sample-size-based funnel plot in meta-analysis." \emph{Research Synthesis Methods}, \bold{10}(3), 376--388. <\doi{10.1002/jrsm.1340}>

Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2006). "Comparison of two methods to detect publication bias in meta-analysis." \emph{JAMA}, \bold{295}(6), 676--680. <\doi{10.1001/jama.295.6.676}>
}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
## mean difference
data("dat.annane")
# descriptive statistics for sigma (continuous outcomes' standard deviation)
quantile(sqrt(dat.annane$s2/(1/dat.annane$n1 + 1/dat.annane$n2)),
  probs = c(0, 0.25, 0.5, 0.75, 1))
# based on sigma = 8
ssfunnel(y, s2, n, data = dat.annane, type = "md",
  alpha = c(0.1, 0.05, 0.01, 0.001), sigma = 8)
# sample sizes presented on a logarithmic scale with plot title
ssfunnel(y, s2, n, data = dat.annane, type = "md",
  alpha = c(0.1, 0.05, 0.01, 0.001), sigma = 8, log.ss = TRUE,
  main = "Contour-enhanced sample-size-based funnel plot")
# based on sigma = 17, with specified x and y limits
ssfunnel(y, s2, n, data = dat.annane, type = "md",
  xlim = c(-15, 15), ylim = c(30, 500),
  alpha = c(0.1, 0.05, 0.01, 0.001), sigma = 17, log.ss = TRUE)
# based on sigma = 20
ssfunnel(y, s2, n, data = dat.annane, type = "md",
  xlim = c(-15, 15), ylim = c(30, 500),
  alpha = c(0.1, 0.05, 0.01, 0.001), sigma = 20, log.ss = TRUE)

## standardized mean difference
data("dat.barlow")
ssfunnel(y, s2, n, data = dat.barlow, type = "smd",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 1))

## log odds ratio
data("dat.butters")
ssfunnel(y, s2, n, data = dat.butters, type = "lor",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 1.5))
# use different colors for contours
ssfunnel(y, s2, n, data = dat.butters, type = "lor",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 1.5),
  cols.contour = c("blue", "green", "yellow", "red"), col.mostsig = "black")
# based on p0 = 0.3 (common event rate in the control group across studies)
ssfunnel(y, s2, n, data = dat.butters, type = "lor",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 1.5), p0 = 0.3)
# based on p0 = 0.5
ssfunnel(y, s2, n, data = dat.butters, type = "lor",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 1.5), p0 = 0.5)

## log relative risk
data("dat.williams")
ssfunnel(y, s2, n, data = dat.williams, type = "lrr",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-1.5, 2.5))
# based on p0 = 0.2
ssfunnel(y, s2, n, data = dat.williams, type = "lrr",
  alpha = c(0.1, 0.05, 0.01, 0.001), p0 = 0.2, xlim = c(-1.5, 2.5))
# based on p0 = 0.3
ssfunnel(y, s2, n, data = dat.williams, type = "lrr",
  alpha = c(0.1, 0.05, 0.01, 0.001), p0 = 0.3, xlim = c(-1.5, 2.5))

## risk difference
data("dat.kaner")
ssfunnel(y, s2, n, data = dat.kaner, type = "rd",
  alpha = c(0.1, 0.05, 0.01, 0.001), xlim = c(-0.5, 0.5))
# based on p0 = 0.1
ssfunnel(y, s2, n, data = dat.kaner, type = "rd",
  alpha = c(0.1, 0.05, 0.01, 0.001), p0 = 0.1, xlim = c(-0.5, 0.5))
# based on p0 = 0.4
ssfunnel(y, s2, n, data = dat.kaner, type = "rd",
  alpha = c(0.1, 0.05, 0.01, 0.001), p0 = 0.4, xlim = c(-0.5, 0.5))
}
\keyword{publication bias/small-study effects}
\keyword{plot}

\name{dat.xu}
\alias{dat.xu}
\docType{data}
\title{
A Network Meta-Analysis on Immune Checkpoint Inhibitor Drugs
}
\description{
This network meta-analysis investigates the effects of seven immune checkpoint inhibitor (ICI) drugs on all-grade treatment-related adverse events (TrAEs). It aimed to provide a safety ranking of the ICI drugs for the treatment of cancer.
}
\usage{data("dat.xu")}
\format{
  A dataset of network meta-analysis with binary outcomes, containing 23 studies and 7 treatments.
  \describe{
    \item{\code{sid}}{study IDs.}
    \item{\code{tid}}{treatment IDs.}
    \item{\code{r}}{event counts.}
    \item{\code{n}}{sample sizes.}
  }
}
\details{
Treatment IDs represent: 1) conventional therapy; 2) nivolumab; 3) pembrolizumab; 4) two ICIs; 5) ICI and conventional therapy; 6) atezolizumab; and 7) ipilimumab.
}
\source{
Xu C, Chen YP, Du XJ, Liu JQ, Huang CL, Chen L, Zhou GQ, Li WF, Mao YP, Hsu C, Liu Q, Lin AH, Tang LL, Sun Y, Ma J (2018). "Comparative safety of immune checkpoint inhibitors in cancer: systematic review and network meta-analysis." \emph{BMJ}, \bold{363}, k4226. <\doi{10.1136/bmj.k4226}>
}
\keyword{dataset}

\name{dat.butters}
\alias{dat.butters}
\docType{data}
\title{
A Meta-Analysis on the Overall Response of the Addition of Drugs to a Chemotherapy Regimen for Metastatic Breast Cancer
}
\description{
This dataset serves as an example of meta-analysis of (log) odds ratios.
}
\usage{data("dat.butters")}
\format{
  A data frame with 16 studies with the following 7 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of log odds ratios.}
    \item{\code{s2}}{sample variances of log odds ratios.}
    \item{\code{n1}}{sample sizes in treatment group 1 (addition of drug).}
    \item{\code{n2}}{sample sizes in treatment group 2 (control).}
    \item{\code{r1}}{event counts in treatment group 1.}
    \item{\code{r2}}{event counts in treatment group 2.}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Butters DJ, Ghersi D, Wilcken N, Kirk SJ, Mallon PT (2010). "Addition of drug/s to a chemotherapy regimen for metastatic breast cancer." \emph{Cochrane Database of Systematic Reviews}, \bold{11}, Art.{\sspace}No.: CD003368. <\doi{10.1002/14651858.CD003368.pub3}>
}
\keyword{dataset}

\name{metapb}
\alias{metapb}

\title{
Detecting and Quantifying Publication Bias/Small-Study Effects
}
\description{
Performs the regression test and calculates skewness for detecting and quantifying publication bias/small-study effects.
}
\usage{
metapb(y, s2, data, model = "RE", n.resam = 1000)
}

\arguments{
  \item{y}{
  a numeric vector specifying the observed effect sizes in the collected studies; they are assumed to be normally distributed.
}
  \item{s2}{
  a numeric vector specifying the within-study variances.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y} and \code{s2}, should be specified as their corresponding column names in \code{data}.
}
  \item{model}{
  a characher string specifying the fixed-effect (\code{"FE"}) or random-effects (\code{"RE"}, the default) model. If not specified, this function uses the \eqn{Q} statistic to test for heterogeneity: if the p-value is smaller than 0.05, \code{model} is set to \code{"RE"}; otherwise, \code{model} = \code{"FE"}.
}
  \item{n.resam}{
  a positive integer specifying the number of resampling iterations.
}
}
\details{
This function derives the measures of publication bias introduced in Lin and Chu (2018).
}
\value{
This function returns a list containing measures of publication bias, their 95\% confidence intervals, and p-values. Specifically, the components include:
  \item{n}{the number of studies in the meta-analysis.}
  \item{p.Q}{the p-value of the \eqn{Q}-test for heterogeneity.}
  \item{I2}{the \eqn{I^2} statistic for quantifying heterogeneity.}
  \item{tau2}{the DerSimonian--Laird estimate of the between-study variance.}
  \item{model}{the model setting (\code{"FE"} or \code{"RE"}).}
  \item{std.dev}{the standardized deviates of the studies.}
  \item{reg.int}{the estimate of the regression intercept for quantifying publication bias.}
  \item{reg.int.ci}{the 95\% CI of the regression intercept.}
  \item{reg.int.ci.resam}{the 95\% CI of the regression intercept based on the resampling method.}
  \item{reg.pval}{the p-value of the regression intercept.}
  \item{reg.pval}{the p-value of the regression intercept based on the resampling method.}
  \item{skewness}{the estimate of the skewness for quantifying publication bias.}
  \item{skewness.ci}{the 95\% CI of the skewness.}
  \item{skewness.ci.resam}{the 95\% CI of the skewness based on the resampling method.}
  \item{skewness.pval}{the p-value of the skewness.}
  \item{skewness.pval.resam}{the p-value of the skewness based on the resampling method.}
  \item{combined.pval}{the p-value of the combined test that incorporates the regression intercept and the skewness.}
  \item{combined.pval.resam}{the p-value of the combined test that incorporates the regression intercept and the skewness based on the resampling method.}
}
\references{
Egger M, Davey Smith G, Schneider M, Minder C (1997). "Bias in meta-analysis detected by a simple, graphical test." \emph{BMJ}, \bold{315}(7109), 629--634. <\doi{10.1136/bmj.315.7109.629}>

Lin L, Chu H (2018). "Quantifying publication bias in meta-analysis." \emph{Biometrics}, \bold{74}(3), 785--794. <\doi{10.1111/biom.12817}>
}

%%\note{
%%  ~~further notes~~
%%}

\examples{
data("dat.slf")
set.seed(1234)
metapb(y, s2, dat.slf)

data("dat.ha")
set.seed(1234)
metapb(y, s2, dat.ha)

data("dat.lcj")
set.seed(1234)
metapb(y, s2, dat.lcj)
}
\keyword{publication bias/small-study effects}

\name{dat.pte}
\alias{dat.pte}
\docType{data}
\title{
Meta-Analysis of Multiple Risk Factors for Pterygium
}
\description{
This dataset serves as an example to illustrate network meta-analysis of multiple factors. It consists of 29 studies on a total of 8 risk factors: area of residence (rural vs.{\sspace}urban); education attainment (low vs.{\sspace}high); latitude of residence (low vs.{\sspace}high); occupation type (outdoor vs.{\sspace}indoor); smoking status (yes vs.{\sspace}no); use of hat (yes vs.{\sspace}no); use of spectacles (yes vs.{\sspace}no); and use of sunglasses (yes vs.{\sspace}no). Each study only investigates a subset of the 8 risk factors, so the dataset contains many missing values.
}
\usage{data("dat.pte")}
\format{
  A list containing two elements, \code{y} and \code{se}.
  \describe{
    \item{\code{y}}{a 29 x 8 numeric matrix containing the observed effect sizes; the rows represent studies and the columns represent outcomes.}
    \item{\code{se}}{a 29 x 8 numeric matrix containing the within-study standard errors; the rows represent studies and the columns represent outcomes.}
  }
}
\source{
Serghiou S, Patel CJ, Tan YY, Koay P, Ioannidis JPA (2016). "Field-wide meta-analyses of observational associations can map selective availability of risk factors and the impact of model specifications." \emph{Journal of Clinical Epidemiology}, \bold{71}, 58--67. <\doi{10.1016/j.jclinepi.2015.09.004}>
}
\keyword{dataset}

\name{nma.predrank}
\alias{nma.predrank}
\title{
Predictive P-Score for Treatment Ranking in Bayesian Network Meta-Analysis
}
\description{
Calculates the P-score and predictive P-score for a network meta-analysis in the Bayesian framework described in Rosenberger et al.{\sspace}(2021).
}
\usage{
nma.predrank(sid, tid, r, n, data, n.adapt = 1000, n.chains = 3, n.burnin = 2000,
             n.iter = 5000, n.thin = 2, lowerbetter = TRUE, pred = TRUE,
             pred.samples = FALSE, trace = FALSE)
}
\arguments{
  \item{sid}{
  a vector specifying the study IDs, from 1 to the number of studies.
}
  \item{tid}{
  a vector specifying the treatment IDs, from 1 to the number of treatments.
}
  \item{r}{
  a numeric vector specifying the event counts.
}
  \item{n}{
  a numeric vector specifying the sample sizes.
}
  \item{data}{
  an optional data frame containing the network meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{sid}, \code{tid}, \code{r}, and \code{n}, should be specified as their corresponding column names in \code{data}.
  }
  \item{n.adapt}{
  the number of iterations for adaptation in the Markov chain Monte Carlo (MCMC) algorithm. The default is 1,000. This argument and the following \code{n.chains}, \code{n.burnin}, \code{n.iter}, and \code{n.thin} are passed to the functions in the package \pkg{rjags}.
}
  \item{n.chains}{
  the number of MCMC chains. The default is 3.
}
  \item{n.burnin}{
  the number of iterations for burn-in period. The default is 2,000.
}
  \item{n.iter}{
  the total number of iterations in each MCMC chain after the burn-in period. The default is 5,000.
}
  \item{n.thin}{
  a positive integer specifying thinning rate. The default is 2.
}
  \item{lowerbetter}{
  A logical value indicating whether lower effect measures indicate better treaetments. If \code{lowerbetter} = \code{TRUE} (the default), then lower effect measures indicate better treatments.
}
  \item{pred}{
  a logical value indicating whether the treatment ranking measures in a new study are to be derived. These measures are only derived when \code{pred} = \code{TRUE}.
}
  \item{pred.samples}{
  a logical value indicating whether the posterior samples of expected scaled ranks in a new study are to be saved.
}
  \item{trace}{
  a logical value indicating whether all posterior samples are to be saved.
}
}
\details{
Under the frequentist setting, the P-score is built on the quantiles
\deqn{P_{kh} = \Phi((\hat{d}_{1k} - \hat{d}_{1h})/s_{kh}),}
where \eqn{\hat{d}_{1k}} and \eqn{\hat{d}_{1h}} are the point estimates of treatment effects for \eqn{k} vs.{\sspace}1 and \eqn{h} vs.{\sspace}1, respectively, and \eqn{s_{kh}} is the standard error of \eqn{\hat{d}_{1k} - \hat{d}_{1h}} (Rucker and Schwarzer, 2015). Moreover, \eqn{\Phi(\cdot)} is the cumulative distribution function of the standard normal distribution. The quantity \eqn{P_{kh}} can be interpreted as the extent of certainty that treatment \eqn{k} is better than \eqn{h}. The frequentist P-score of treatment \eqn{k} is \eqn{\frac{1}{K-1} \sum_{h \neq k} P_{kh}}.

Analogously to the frequentist P-score, conditional on \eqn{d_{1h}} and \eqn{d_{1k}}, the quantities \eqn{P_{hk}} from the Bayesian perspective can be considered as \eqn{I(d_{1k} > d_{1h})}, which are Bernoulli random variables. To quantify the overall performance of treatment \eqn{k}, we may similarly use
\deqn{\bar{P}_k = \frac{1}{K-1} \sum_{h \neq k} I(d_{1k} > d_{1h}).}
Note that \eqn{\bar{P}_k} is a parameter under the Bayesian framework, while the frequentist P-score is a statistic. Moreover, \eqn{\sum_{h \neq k} I(d_{1k} > d_{1h})} is equivalent to \eqn{K - R_k}, where \eqn{R_k} is the true rank of treatment \eqn{k}. Thus, we may also write \eqn{\bar{P}_k = (K - R_k)/(K - 1)}; this corresponds to the findings by Rucker and Schwarzer (2015). Consequently, we call \eqn{\bar{P}_k} the scaled rank in the network meta-analysis (NMA) for treatment \eqn{k}. It transforms the range of the original rank between 1 and \eqn{K} to a range between 0 and 1. In addition, note that \eqn{E[I(d_{1k} > d_{1h} | Data)] = \Pr(d_{1k} > d_{1h} | Data)}, which is analogous to the quantity of \eqn{P_{kh}} under the frequentist framework. Therefore, we use the posterior mean of the scaled rank \eqn{\bar{P}_k} as the Bayesian P-score; it is a counterpart of the frequentist P-score.

The scaled ranks \eqn{\bar{P}_k} can be feasibly estimated via the MCMC algorithm. Let \eqn{\{d_{1k}^{(j)}; k = 2, \ldots, K\}_{j=1}^J} be the posterior samples of the overall relative effects \eqn{d_{1k}} of all treatments vs.{\sspace}the reference treatment 1 in a total of \eqn{J} MCMC iterations after the burn-in period, where \eqn{j} indexes the iterations. As \eqn{d_{11}} is trivially 0, we set \eqn{d_{11}^{(j)}} to 0 for all \eqn{j}. The \eqn{j}th posterior sample of treatment \eqn{k}'s scaled rank is \eqn{\bar{P}_k^{(j)} = \frac{1}{K-1} \sum_{h \neq k}I(d_{1k}^{(j)} > d_{1h}^{(j)})}. We can make inferences for the scaled ranks from the posterior samples \eqn{\{\bar{P}_k^{(j)}\}_{j=1}^{J}}, and use their posterior means as the Bayesian P-scores. We may also obtain the posterior medians as another set of point estimates, and the 2.5\% and 97.5\% posterior quantiles as the lower and upper bounds of 95\% credible intervals (CrIs), respectively. Because the posterior samples of the scaled ranks take discrete values, the posterior medians and the CrI bounds are also discrete.

Based on the idea of the Bayesian P-score, we can similarly define the predictive P-score for a future study by accounting for the heterogeneity between the existing studies in the NMA and the new study. Specifically, we consider the probabilities in the new study
\deqn{P_{new,kh} = \Pr(\delta_{new,1k} > \delta_{new,1h}),}
conditional on the population parameters \eqn{d_{1h}}, \eqn{d_{1k}}, and \eqn{\tau} from the NMA. Here, \eqn{\delta_{new,1k}} and \eqn{\delta_{new,1h}} represent the treatment effects of \eqn{k} vs.{\sspace}1 and \eqn{h} vs.{\sspace}1 in the new study, respectively. The \eqn{P_{new,kh}} corresponds to the quantity \eqn{P_{kh}} in the NMA; it represents the probability of treatment \eqn{k} being better than \eqn{h} in the new study. Due to heterogeneity, \eqn{\delta_{new,1k} \sim N(d_{1k}, \tau^2)} and \eqn{\delta_{new,1h} \sim N(d_{1h}, \tau^{2})}. The correlation coefficients between treatment comparisons are typically assumed to be 0.5; therefore, such probabilities in the new study can be explicitly calculated as \eqn{P_{new,kh} = \Phi((d_{1k} - d_{1h})/\tau)}, which is a function of \eqn{d_{1h}}, \eqn{d_{1k}}, and \eqn{\tau}. Finally, we use
\deqn{\bar{P}_{new,k} = \frac{1}{K-1} \sum_{h \neq k} P_{new,kh}}
to quantify the performance of treatment \eqn{k} in the new study. The posterior samples of \eqn{\bar{P}_{new,k}} can be derived from the posterior samples of \eqn{d_{1k}}, \eqn{d_{1h}}, and \eqn{\tau} during the MCMC algorithm.

Note that the probabilities \eqn{P_{new,kh}} can be written as \eqn{E[I(\delta_{new,1k} > \delta_{new,1h})]}. Based on similar observations for the scaled ranks in the NMA, the \eqn{\bar{P}_{new,k}} in the new study subsequently becomes
\deqn{\bar{P}_{new,k} = \frac{1}{K-1} E\left[\sum_{h \neq k} I(\delta_{new,1k} > \delta_{new,1h})\right] = E\left[\frac{K - R_{new,k}}{K - 1}\right],}
where \eqn{R_{new,k}} is the true rank of treatment \eqn{k} in the new study. Thus, we call \eqn{\bar{P}_{new,k}} the expected scaled rank in the new study. Like the Bayesian P-score, we define the predictive P-score as the posterior mean of \eqn{\bar{P}_{new,k}}. The posterior medians and 95\% CrIs can also be obtained using the MCMC samples of \eqn{\bar{P}_{new,k}}. See more details in Rosenberger et al.{\sspace}(2021).
}

\value{
This function estimates the P-score for all treatments in a Bayesian NMA, estimates the predictive P-score (if \code{pred} = \code{TRUE}), gives the posterior samples of expected scaled ranks in a new study (if \code{pred.samples} = \code{TRUE}), and outputs all MCMC posterior samples (if \code{trace} = \code{TRUE}).
} 

\references{
Rosenberger KJ, Duan R, Chen Y, Lin L (2021). "Predictive P-score for treatment ranking in Bayesian network meta-analysis." \emph{BMC Medical Research Methodology}, \bold{21}, 213. <\doi{10.1186/s12874-021-01397-5}>

Rucker G, Schwarzer G (2015). "Ranking treatments in frequentist network meta-analysis works without resampling methods." \emph{BMC Medical Research Methodology}, \bold{15}, 58. <\doi{10.1186/s12874-015-0060-8}>
}

\author{
Kristine J.{\sspace}Rosenberger, Lifeng Lin 
}

\examples{
\donttest{
## increase n.burnin (e.g., to 50000) and n.iter (e.g., to 200000)
## for better convergence of MCMC
data("dat.sc")
set.seed(1234)
out1 <- nma.predrank(sid, tid, r, n, data = dat.sc, n.burnin = 500, n.iter = 2000,
  lowerbetter = FALSE, pred.samples = TRUE)
out1$P.score
out1$P.score.pred

cols <- c("red4", "plum4", "paleturquoise4", "palegreen4")
cols.hist <- adjustcolor(cols, alpha.f = 0.4)
trtnames <- c("1) No contact", "2) Self-help", "3) Individual counseling",
  "4) Group counseling")
brks <- seq(0, 1, 0.01)
hist(out1$P.pred[[1]], breaks = brks, freq = FALSE,
  xlim = c(0, 1), ylim = c(0, 5), col = cols.hist[1], border = cols[1],
  xlab = "Expected scaled rank in a new study", ylab = "Density", main = "")
hist(out1$P.pred[[2]], breaks = brks, freq = FALSE,
  col = cols.hist[2], border = cols[2], add = TRUE)
hist(out1$P.pred[[3]], breaks = brks, freq = FALSE,
  col = cols.hist[3], border = cols[3], add = TRUE)
hist(out1$P.pred[[4]], breaks = brks, freq = FALSE,
  col = cols.hist[4], border = cols[4], add = TRUE)
legend("topright", fill = cols.hist, border = cols, legend = trtnames)


data("dat.xu")
set.seed(1234)
out2 <- nma.predrank(sid, tid, r, n, data = dat.xu, n.burnin = 500, n.iter = 2000)
out2
}
}
\keyword{multivariate meta-analysis}

\name{dat.henry}
\alias{dat.henry}
\docType{data}
\title{
A Meta-Analysis for Evaluating the Effect of Tranexamic Acid on Perioperative Allogeneic Blood Transfusion
}
\description{
This meta-analysis serves as an example of meta-analysis with binary outcomes.
}
\usage{data("dat.henry")}
\format{
  A data frame containing 26 studies with the following 4 variables within each study.
  \describe{
    \item{\code{n00}}{counts of non-events in treatment group 0 (placebo).}
    \item{\code{n01}}{counts of events in treatment group 0 (placebo).}
    \item{\code{n10}}{counts of non-events in treatment group 1 (tranexamic acid).}
    \item{\code{n11}}{counts of events in treatment group 1 (tranexamic acid).}
  }
}
\note{The original review collected 27 studies; one double-zero-counts study is excluded from this dataset because its odds ratio is not estimable.}
\source{
Henry DA, Carless PA, Moxey AJ, O'Connell, Stokes BJ, Fergusson DA, Ker K (2011). "Anti-fibrinolytic use for minimising perioperative allogeneic blood transfusion." \emph{Cochrane Database of Systematic Reviews}, \bold{1}, Art.{\sspace}No.: CD001886. <\doi{10.1002/14651858.CD001886.pub3}>
}
\keyword{dataset}

\name{metahet}
\alias{metahet}

\title{
Meta-Analysis Heterogeneity Measures
}
\description{
Calculates various between-study heterogeneity measures in meta-analysis, including the conventional measures (e.g., \eqn{I^2}) and the alternative measures (e.g., \eqn{I_r^2}) which are robust to outlying studies; p-values of various tests are also calculated.
}
\usage{
metahet(y, s2, data, n.resam = 1000)
}

\arguments{
  \item{y}{
  a numeric vector specifying the observed effect sizes in the collected studies; they are assumed to be normally distributed.
}
  \item{s2}{
  a numeric vector specifying the within-study variances.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y} and \code{s2}, should be specified as their corresponding column names in \code{data}.
}
  \item{n.resam}{
  a positive integer specifying the number of resampling iterations for calculating p-values of test statistics and 95\% confidence interval of heterogeneity measures.
}
}
\details{
Suppose that a meta-analysis collects \eqn{n} studies. The observed effect size in study \eqn{i} is \eqn{y_i} and its within-study variance is \eqn{s^{2}_{i}}. Also, the inverse-variance weight is \eqn{w_i = 1 / s^{2}_{i}}. The fixed-effect estimate of overall effect size is \eqn{\bar{\mu} = \sum_{i = 1}^{n} w_i y_i / \sum_{i = 1}^{n} w_i}. The conventional test statistic for heterogeneity is
\deqn{Q = \sum_{i = 1}^{n} w_i (y_{i} - \bar{\mu})^2.}
Based on the \eqn{Q} statistic, the method-of-moments estimate of the between-study variance \eqn{\hat{\tau}_{DL}^2} is (DerSimonian and Laird, 1986)
\deqn{\hat{\tau}^2_{DL} = \max \left\{ 0, \frac{Q - (n - 1)}{\sum_{i = 1}^{n} w_{i} - \sum_{i = 1}^{n} w_{i}^{2} / \sum_{i = 1}^{n} w_{i}} \right\}.}
Also, the \eqn{H} and \eqn{I^2} statistics (Higgins and Thompson, 2002; Higgins et al., 2003) are widely used in practice because they do not depend on the number of collected studies \eqn{n} and the effect size scale; these two statistics are defined as
\deqn{H = \sqrt{Q/(n - 1)};}
\deqn{I^{2} = \frac{Q - (n - 1)}{Q}.}
Specifically, the \eqn{H} statistic reflects the ratio of the standard deviation of the underlying mean from a random-effects meta-analysis compared to the standard deviation from a fixed-effect meta-analysis; the \eqn{I^2} statistic describes the proportion of total variance across studies that is due to heterogeneity rather than sampling error.

Outliers are frequently present in meta-analyses, and they may have great impact on the above heterogeneity measures. Alternatively, to be more robust to outliers, the test statistic may be modified as (Lin et al., 2017):
\deqn{Q_{r} = \sum_{i = 1}^{n} \sqrt{w_i} |y_{i} - \bar{\mu}|.}
Based on the \eqn{Q_r} statistic, the method-of-moments estimate of the between-study variance \eqn{\hat{\tau}_r^2} is defined as the solution to
\deqn{Q_r \sqrt{\frac{\pi}{2}} = \sum_{i = 1}^{n} \left\{1 - \frac{w_{i}}{\sum_{j = 1}^{n} w_{j}} + \tau^{2} \left[ w_{i} - \frac{2 w_{i}^{2}}{\sum_{j = 1}^{n} w_{j}} + \frac{w_{i} \sum_{j = 1}^{n} w_{j}^{2}}{(\sum_{j = 1}^{n} w_{j})^2} \right]\right\}.}
If no positive solution exists to the equation above, set \eqn{\hat{\tau}_{r}^{2} = 0}. The counterparts of the \eqn{H} and \eqn{I^2} statistics are defined as
\deqn{H_{r} = Q_r \sqrt{\pi/[2 n (n - 1)]};}
\deqn{I_{r}^{2} = \frac{Q_{r}^{2} - 2 n (n - 1) / \pi}{Q_{r}^{2}}.}

To further improve the robustness of heterogeneity assessment, the weighted \emph{mean} in the \eqn{Q_r} statistic may be replaced by the weighted \emph{median} \eqn{\hat{\mu}_m}, which is the solution to \eqn{\sum_{i = 1}^{n} w_i [I (\theta \geq y_i) - 0.5] = 0} with respect to \eqn{\theta}. The new test statistic is
\deqn{Q_m = \sum_{i = 1}^{n} \sqrt{w_i} |y_{i} - \hat{\mu}_m|.}
Based on \eqn{Q_m}, the new estimator of the between-study variance \eqn{\hat{\tau}_m^2} is the solution to
\deqn{Q_m \sqrt{\pi/2} = \sum_{i = 1}^{n} \sqrt{(s_i^2 + \tau^2)/s_i^2}.}
The counterparts of the \eqn{H} and \eqn{I^2} statistics are
\deqn{H_m = \frac{Q_m}{n} \sqrt{\pi/2};}
\deqn{I_m^2 = \frac{Q_m^2 - 2 n^2/\pi}{Q_m^2}.}
}
\value{
This function returns a list containing p-values of various heterogeneity tests and various heterogeneity measures with 95\% confidence intervals. Specifically, the components include:
  \item{p.Q}{p-value of the \eqn{Q} statistic (using the resampling method).}
  \item{p.Q.theo}{p-value of the \eqn{Q} statistic using the \eqn{Q}'s theoretical chi-squared distribution.}
  \item{p.Qr}{p-value of the \eqn{Q_r} statistic (using the resampling method).}
  \item{p.Qm}{p-value of the \eqn{Q_m} statistic (using the resampling method).}
  \item{Q}{the \eqn{Q} statistic.}
  \item{ci.Q}{95\% CI of the \eqn{Q} statistic.}
  \item{tau2.DL}{DerSimonian--Laird estimate of the between-study variance.}
  \item{ci.tau2.DL}{95\% CI of the between-study variance based on the DerSimonian--Laird method.}
  \item{H}{the \eqn{H} statistic.}
  \item{ci.H}{95\% CI of the \eqn{H} statistic.}
  \item{I2}{the \eqn{I^2} statistic.}
  \item{ci.I2}{95\% CI of the \eqn{I^2} statistic.}
  \item{Qr}{the \eqn{Q_r} statistic.}
  \item{ci.Qr}{95\% CI of the \eqn{Q_r} statistic.}
  \item{tau2.r}{the between-study variance estimate based on the \eqn{Q_r} statistic.}
  \item{ci.tau2.r}{95\% CI of the between-study variance based on the \eqn{Q_r} statistic.}
  \item{Hr}{the \eqn{H_r} statistic.}
  \item{ci.Hr}{95\% CI of the \eqn{H_r} statistic.}
  \item{Ir2}{the \eqn{I_r^2} statistic.}
  \item{ci.Ir2}{95\% CI of the \eqn{I_r^2} statistic.}
  \item{Qm}{the \eqn{Q_m} statistic.}
  \item{ci.Qm}{95\% CI of the \eqn{Q_m} statistic.}
  \item{tau2.m}{the between-study variance estimate based on the \eqn{Q_m} statistic.}
  \item{ci.tau2.m}{95\% CI of the between-study variance based on the \eqn{Q_m} statistic}
  \item{Hm}{the \eqn{H_m} statistic.}
  \item{ci.Hm}{95\% CI of the \eqn{H_m} statistic.}
  \item{Im2}{the \eqn{I_m^2} statistic.}
  \item{ci.Im2}{95\% CI of the \eqn{I_m^2} statistic.}
}
\references{
DerSimonian R, Laird N (1986). "Meta-analysis in clinical trials." \emph{Controlled Clinical Trials}, \bold{7}(3), 177--188. <\doi{10.1016/0197-2456(86)90046-2}>

Higgins JPT, Thompson SG (2002). "Quantifying heterogeneity in a meta-analysis." \emph{Statistics in Medicine}, \bold{21}(11), 1539--1558. <\doi{10.1002/sim.1186}>

Higgins JPT, Thompson SG, Deeks JJ, Altman DG (2003). "Measuring inconsistency in meta-analyses." \emph{BMJ}, \bold{327}(7414), 557--560. <\doi{10.1136/bmj.327.7414.557}>

Lin L, Chu H, Hodges JS (2017). "Alternative measures of between-study heterogeneity in meta-analysis: reducing the impact of outlying studies." \emph{Biometrics}, \bold{73}(1), 156--166. <\doi{10.1111/biom.12543}>
}

\seealso{
\code{\link{metahet.hybrid}}
}
\examples{
data("dat.aex")
set.seed(1234)
metahet(y, s2, dat.aex, 100)
\donttest{metahet(y, s2, dat.aex, 1000)}

data("dat.hipfrac")
set.seed(1234)
metahet(y, s2, dat.hipfrac, 100)
\donttest{metahet(y, s2, dat.hipfrac, 1000)}
}
\keyword{heterogeneity}

\name{meta.or.smd}
\alias{meta.or.smd}
\title{
Meta-Analysis of Combining Standardized Mean Differences and Odds Ratios
}
\description{
Performs a Bayesian meta-analysis to synthesize standardized mean differences (SMDs) for a continuous outcome and odds ratios (ORs) for a binary outcome.
}
\usage{
meta.or.smd(y1, sd1, n1, y0, sd0, n0, r1, m1, r0, m0, id.bin, data,
            n.adapt = 1000, n.chains = 3, n.burnin = 5000, n.iter = 20000, n.thin = 2,
            seed = 1234)
}
\arguments{
  \item{y1}{
  a vector specifying the sample means in the treatment group for the continuous outcome. \code{NA} is allowed and indicates that data are not available for this outcome; the same applies to the other arguments, including \code{sd1}, \code{n1}, \code{y0}, \code{sd0}, \code{n0}, \code{r1}, \code{m1}, \code{r0}, and \code{m0}.
}
  \item{sd1}{
  a vector specifying the sample standard deviations in the treatment group for the continuous outcome.
}
  \item{n1}{
  a vector specifying the sample sizes in the treatment group for the continuous outcome.
}
  \item{y0}{
  a vector specifying the sample means in the control group for the continuous outcome.
}
  \item{sd0}{
  a vector specifying the sample standard deviations in the control group for the continuous outcome.
}
  \item{n0}{
  a vector specifying the sample sizes in the control group for the continuous outcome.
}
  \item{r1}{
  a vector specifying the event counts in the treatment group for the binary outcome.
}
  \item{m1}{
  a vector specifying the sample sizes in the treatment group for the binary outcome.
}
  \item{r0}{
  a vector specifying the event counts in the control group for the binary outcome.
}
  \item{m0}{
  a vector specifying the sample sizes in the control group for the binary outcome.
}
  \item{id.bin}{
  a vector indicating whether the outcome is binary (1) or continuous (0).
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y1}, \code{sd1}, \code{n1}, \code{y0}, \code{sd0}, \code{n0}, \code{r1}, \code{m1}, \code{r0}, \code{m0}, and \code{id.bin} should be specified as their corresponding column names in \code{data}.
}
  \item{n.adapt}{
  the number of iterations for adaptation in the Markov chain Monte Carlo (MCMC) algorithm. The default is 1,000. This argument and the following \code{n.chains}, \code{n.burnin}, \code{n.iter}, and \code{n.thin} are passed to the functions in the package \pkg{rjags}.
}
  \item{n.chains}{
  the number of MCMC chains. The default is 3.
}
  \item{n.burnin}{
  the number of iterations for burn-in period. The default is 5,000.
}
  \item{n.iter}{
  the total number of iterations in each MCMC chain after the burn-in period. The default is 20,000.
}
  \item{n.thin}{
  a positive integer specifying thinning rate. The default is 2.
}
  \item{seed}{
  an integer for specifying the seed of the random number generation for reproducibility during the MCMC algorithm for performing the Bayesian meta-analysis model.
}
}
\details{
The Bayesian meta-analysis model implemented by this function is detailed in Section 2.5 of Jing et al.\sspace(2023).
}
\value{
"This function returns a list of Bayesian estimates, including posterior medians and 95\% credible intervals (comprising the 2.5\% and 97.5\% posterior quantiles) for the overall SMD (\code{d}), the between-study standard deviation (\code{tau}), and the individual studies' SMDs (\code{theta}).
}
\references{
Jing Y, Murad MH, Lin L (2023). "A Bayesian model for combining standardized mean differences and odds ratios in the same meta-analysis." \emph{Journal of Biopharmaceutical Statistics}, \bold{33}(2), 167--190. <\doi{10.1080/10543406.2022.2105345}>
}
\author{
Yaqi Jing, Lifeng Lin 
}

\examples{
\donttest{
data("dat.dep")
out <- meta.or.smd(y1, sd1, n1, y0, sd0, n0, r1, m1, r0, m0, id.bin, data = dat.dep)
out
}
}
\keyword{multivariate meta-analysis}

\name{meta.biv}
\alias{meta.biv}
\title{
Bivariate Method for Meta-Analysis.
}
\description{
Performs a meta-analysis with a binary outcome using a bivariate generalized linear mixed model (GLMM) described in Chu et al.\sspace(2012).
}
\usage{
meta.biv(sid, tid, e, n, data, link = "logit", alpha = 0.05,
         b.iter = 1000, seed = 1234, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{sid}{
  a vector specifying the study IDs.
}
  \item{tid}{
  a vector of 0/1 specifying the treatment/exposure IDs (0: control/non-exposure; 1: treatment/exposure).
}
  \item{e}{
  a numeric vector specifying the event counts.
}
  \item{n}{
  a numeric vector specifying the sample sizes.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{sid}, \code{tid}, \code{e}, and \code{n}, should be specified as their corresponding column names in \code{data}.
}
  \item{link}{
  a character string specifying the link function used in the GLMM, which can be either \code{"logit"} (the default) or \code{"probit"}.
}
  \item{alpha}{
  a numeric value specifying the statistical significance level.
}
  \item{b.iter}{
  an integer specifying the number of bootstrap iterations, which are used to produce confidence intervals of marginal results.
}
  \item{seed}{
  an integer for specifying the seed of the random number generation for reproducibility during the bootstrap resampling.
}
  \item{\dots}{
  other arguments that can be passed to the function \code{\link[lme4]{glmer}} in the package \pkg{lme4}.
}
}
\details{
Suppose a meta-analysis with a binary outcome contains \eqn{N} studies. Let \eqn{n_{i0}} and \eqn{n_{i1}} be the sample sizes in the control/non-exposure and treatment/exposure groups in study \eqn{i}, respectively, and let \eqn{e_{i0}} and \eqn{e_{i1}} be the event counts (\eqn{i = 1, \ldots, N}). The event counts are assumed to independently follow binomial distributions:
\deqn{e_{i0} \sim Bin(n_{i0}, p_{i0});}
\deqn{e_{i1} \sim Bin(n_{i1}, p_{i1}),}
where \eqn{p_{i0}} and \eqn{p_{i1}} represent the true event probabilities. They are modeled jointly as follows:
\deqn{g(p_{i0}) = \mu_0 + \nu_{i0};}
\deqn{g(p_{i1}) = \mu_1 + \nu_{i1};}
\deqn{(\nu_{i0}, \nu_{i1})^\prime \sim N ((0, 0)^\prime, \mathbf{\Sigma}).}
Here, \eqn{g(\cdot)} denotes the link function that transforms the event probabilities to linear forms. The fixed effects \eqn{\mu_0} and \eqn{\mu_1} represent the overall event probabilities on the transformed scale. The study-specific parameters \eqn{\nu_{i0}} and \eqn{\nu_{i1}} are random effects, which are assumed to follow the bivariate normal distribution with zero means and variance-covariance matrix \eqn{\mathbf{\Sigma}}. The diagonal elements of \eqn{\mathbf{\Sigma}} are \eqn{\sigma_0^2} and \eqn{\sigma_1^2} (between-study variances due to heterogeneity), and the off-diagonal elements are \eqn{\rho \sigma_0 \sigma_1}, where \eqn{\rho} is the correlation coefficient.

When using the logit link, \eqn{\mu_1 - \mu_0} represents the log odds ratio (Van Houwelingen et al., 1993; Stijnen et al., 2010; Jackson et al., 2018); \eqn{\exp(\mu_1 - \mu_0)} may be referred to as the conditional odds ratio (Agresti, 2013). Alternatively, we can obtain the marginal event probabilities (Chu et al., 2012):
\deqn{p_k = E[p_{ik}] \approx \left[1 + \exp\left(-\mu_k/\sqrt{1 + C^2 \sigma_k^2}\right)\right]^{-1}}
for \eqn{k} = 0 and 1, where \eqn{C = 16 \sqrt{3} / (15 \pi)}. The marginal odds ratio, relative risk, and risk difference are subsequently obtained as \eqn{[p_1/(1 - p_1)]/[p_0/(1 - p_0)]}, \eqn{p_1/p_0}, and \eqn{p_1 - p_0}, respectively.

When using the probit link, the model does not yield the conditional odds ratio. The marginal probabilities have closed-form solutions:
\deqn{p_k = E[p_{ik}] = \Phi\left(\mu_k/\sqrt{1 + \sigma_k^2}\right)}
for \eqn{k} = 0 and 1, where \eqn{\Phi(\cdot)} is the cumulative distribution function of the standard normal distribution. They further lead to the marginal odds ratio, relative risk, and risk difference.

}
\value{
This function returns a list containing the point and interval estimates of the marginal event rates (\code{p0.m}, \code{p0.m.ci}, \code{p1.m}, and \code{p1.m.ci}), odds ratio (\code{OR.m} and \code{OR.m.ci}), relative risk (\code{RR.m} and \code{RR.m.ci}), risk difference (\code{RD.m} and \code{RD.m.ci}), and correlation coefficient between the two treatment/exposure groups (\code{rho} and \code{rho.ci}). These interval estimates are obtained using the bootstrap resampling. During the bootstrap resampling, computational warnings or errors may occur for implementing the bivariate GLMM in some resampled meta-analyses. This function returns the counts of warnings and errors (\code{b.w.e}). The resampled meta-analyses that lead to warnings and errors are not used for producing the bootstrap confidence intervals; the bootstrap iterations stop after obtaining \code{b.iter} resampled meta-analyses without warnings and errors. If the logit link is used (\code{link} = \code{"logit"}), it also returns the point and interval estimates of the conditional odds ratio (\code{OR.c} and \code{OR.c.ci}), which are more frequently reported in the current literature than the marginal odds ratios. Unlike the marginal results that use the bootstrap resampling to produce their confidence intervals, the Wald-type confidence interval is calculated for the log conditional odds ratio; it is then transformed to the odds ratio scale.
}
\references{
Agresti A (2013). \emph{Categorical Data Analysis}. Third edition. John Wiley & Sons, Hoboken, NJ.

Chu H, Nie L, Chen Y, Huang Y, Sun W (2012). "Bivariate random effects models for meta-analysis of comparative studies with binary outcomes: methods for the absolute risk difference and relative risk." \emph{Statistical Methods in Medical Research}, \bold{21}(6), 621--633. <\doi{10.1177/0962280210393712}>

Jackson D, Law M, Stijnen T, Viechtbauer W, White IR (2018). "A comparison of seven random-effects models for meta-analyses that estimate the summary odds ratio." \emph{Statistics in Medicine}, \bold{37}(7), 1059--1085. <\doi{10.1002/sim.7588}>

Stijnen T, Hamza TH, Ozdemir P (2010). "Random effects meta-analysis of event outcome in the framework of the generalized linear mixed model with applications in sparse data." \emph{Statistics in Medicine}, \bold{29}(29), 3046--3067. <\doi{10.1002/sim.4040}>

Van Houwelingen HC, Zwinderman KH, Stijnen T (1993). "A bivariate approach to meta-analysis." \emph{Statistics in Medicine}, \bold{12}(24), 2273--2284. <\doi{10.1002/sim.4780122405}>
}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{maprop.glmm}}, \code{\link{meta.dt}}
}
\examples{
\donttest{
data("dat.bellamy")
out.bellamy.logit <- meta.biv(sid, tid, e, n, data = dat.bellamy,
  link = "logit", b.iter = 1000)
out.bellamy.logit
out.bellamy.probit <- meta.biv(sid, tid, e, n, data = dat.bellamy,
  link = "probit", b.iter = 1000)
out.bellamy.probit
}
}
\keyword{generalized linear mixed model}

\name{dat.plourde}
\alias{dat.plourde}
\docType{data}
\title{
A Meta-Analysis for Comparing the Fluoroscopy Time in Percutaneous Coronary Intervention Between Radial and Femoral Accesses
}
\description{
This dataset serves as an example of meta-analysis of mean differences.
}
\usage{data("dat.plourde")}
\format{
  A data frame with 19 studies with the following 5 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of mean differences.}
    \item{\code{s2}}{sample variances of mean differences.}
    \item{\code{n1}}{sample sizes in treatment group 1 (radial).}
    \item{\code{n2}}{sample sizes in treatment group 2 (femoral).}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Plourde G, Pancholy SB, Nolan J, Jolly S, Rao SV, Amhed I, Bangalore S, Patel T, Dahm JB, Bertrand OF (2015). "Radiation exposure in relation to the arterial access site used for diagnostic coronary angiography and percutaneous coronary intervention: a systematic review and meta-analysis." \emph{Lancet}, \bold{386}(10009), 2192--2203. <\doi{10.1016/S0140-6736(15)00305-0}>
}
\keyword{dataset}

\name{dat.bohren}
\alias{dat.bohren}
\docType{data}
\title{
A Meta-Analysis on the Effects of Continuous, One-To-One Intrapartum Support on Spontaneous Vaginal Births, Compared With Usual Care
}
\description{
This meta-analysis serves as an example to illustrate the usage of the function \code{\link{meta.pen}}.
}
\usage{data("dat.ha")}
\format{
  A data frame containing 21 studies.
  \describe{
    \item{\code{n1}}{the sample size in the treatment group in each study.}
    \item{\code{n2}}{the sample size in the control group in each study.}
    \item{\code{r1}}{the event count in the treatment group in each study.}
    \item{\code{r2}}{the event count in the control group in each study.}
    \item{\code{y}}{the observed effect size (log odds ratio) for each study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Bohren MA, Hofmeyr GJ, Sakala C, Fukuzawa RK, Cuthbert A (2017). "Continuous support for women during childbirth." \emph{Cochrane Database of Systematic Reviews}, \bold{7}. Art.{\sspace}No.: CD003766. <\doi{10.1002/14651858.CD003766.pub6}>
}
\keyword{dataset}

\name{mvma.hybrid}
\alias{mvma.hybrid}

\title{
Hybrid Model for Random-Effects Multivariate Meta-Analysis
}
\description{
Performs a multivariate meta-analysis using the hybrid random-effects model when the within-study correlations are unknown.
}
\usage{
mvma.hybrid(ys, vars, data, method = "reml", tol = 1e-10)
}

\arguments{
  \item{ys}{
  an n x p numeric matrix containing the observed effect sizes. The n rows represent studies, and the p columns represent the multivariate endpoints. \code{NA} is allowed for missing endpoints.
}
  \item{vars}{
  an n x p numeric matrix containing the observed within-study variances. The n rows represent studies, and the p columns represent the multivariate endpoints. \code{NA} is allowed for missing endpoints.
}
  \item{data}{
  an optional data frame containing the multivariate meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{ys} and \code{vars}, should be specified as their corresponding column names in \code{data}.
}
  \item{method}{
  a character string specifying the method for estimating the overall effect sizes. It should be \code{"ml"} (random-effects model using the maximum likelihood method) or \code{"reml"} (random-effects model using the restricted maximum likelihood method, the default).
}
  \item{tol}{
  a small number specifying the convergence tolerance for the estimates by maximizing (restricted) likelihood. The default is \code{1e-10}.
}
}
\details{
Suppose \eqn{n} studies are collected in a multivariate meta-analysis on a total of \eqn{p} endpoints. Denote the \eqn{p}-dimensional vector of effect sizes as \eqn{\boldsymbol{y}_i}, and their within-study variances form a diagonal matrix \eqn{\mathbf{D}_i}. However, the within-study correlations are unknown. Then, the random-effects hybrid model is as follows (Riley et al., 2008; Lin and Chu, 2018):
\deqn{\boldsymbol{y}_i \sim N (\boldsymbol{\mu}, (\mathbf{D}_i + \mathbf{T})^{1/2} \mathbf{R} (\mathbf{D}_i + \mathbf{T})^{1/2}),}
where \eqn{\boldsymbol{\mu}} represents the overall effect sizes across studies, \eqn{\mathbf{T} = diag(\tau_1^2, \ldots, \tau_p^2)} consists of the between-study variances, and \eqn{\mathbf{R}} is the marginal correlation matrix. Although the within-study correlations are unknown, this model accounts for both within- and between-study correlations by using the marginal correlation matrix.
}
\value{
This function returns a list containing the following elements:
\item{mu.est}{The estimated overall effect sizes of the p endpoints.}
\item{tau2.est}{The estimated between-study variances of the p endpoints.}
\item{mar.R}{The estimated marginal correlation matrix.}
\item{mu.cov}{The covariance matrix of the estimated overall effect sizes.}
\item{method}{The method used to produce the estimates.}
}
\references{
Lin L, Chu H (2018), "Bayesian multivariate meta-analysis of multiple factors." \emph{Research Synthesis Methods}, \bold{9}(2), 261--272. <\doi{10.1002/jrsm.1293}>

Riley RD, Thompson JR, Abrams KR (2008), "An alternative model for bivariate random-effects meta-analysis when the within-study correlations are unknown." \emph{Biostatistics}, \bold{9}(1), 172--186. <\doi{10.1093/biostatistics/kxm023}>
}

\note{
The algorithm for maximizing (restricted) likelihood may not converge when the dimension of endpoints is too high or the data are too sparse.
}

\seealso{
\code{\link{mvma}}, \code{\link{mvma.bayesian}}, \code{\link{mvma.hybrid.bayesian}}
}
\examples{
\donttest{
data("dat.fib")
y <- dat.fib$y
sd <- dat.fib$sd
mvma.hybrid(y = y, vars = sd^2)
}
}
\keyword{multivariate meta-analysis}

\name{dat.hipfrac}
\alias{dat.hipfrac}
\docType{data}
\title{
A Meta-Analysis on the Magnitude and Duration of Excess Mortality After Hip Fracture Among Older Men
}
\description{
This meta-analysis serves as an example to illustrate function usage in the package \pkg{altmeta}.
}
\usage{data("dat.hipfrac")}
\format{
  A data frame containing 17 studies with the observed effect sizes and their within-study variances.
  \describe{
    \item{\code{y}}{the observed effect size for each collected study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Haentjens P, Magaziner J, Colon-Emeric CS, Vanderschueren D, Milisen K, Velkeniers B, Boonen S (2010). "Meta-analysis: excess mortality after hip fracture among older women and men". \emph{Annals of Internal Medicine}, \bold{152}(6), 380--390. <\doi{10.7326/0003-4819-152-6-201003160-00008}>
}
\keyword{dataset}

\name{maprop.glmm}
\alias{maprop.glmm}
\title{
Meta-Analysis of Proportions Using Generalized Linear Mixed Models
}
\description{
Performs a meta-analysis of proportions using generalized linear mixed models (GLMMs) with various link functions.
}
\usage{
maprop.glmm(e, n, data, link = "logit", alpha = 0.05,
            pop.avg = TRUE, int.approx = 10000, b.iter = 1000,
            seed = 1234, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{e}{
  a numeric vector specifying the event counts in the collected studies.
}
  \item{n}{
  a numeric vector specifying the sample sizes in the collected studies.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{e} and \code{n}, should be specified as their corresponding column names in \code{data}.
}
  \item{link}{
  a character string specifying the link function used in the GLMM, which can be one of \code{"log"} (log link), \code{"logit"} (logit link, the default), \code{"probit"} (probit link), \code{"cauchit"} (cauchit link), and \code{"cloglog"} (complementary log-log link).
}
  \item{alpha}{
  a numeric value specifying the statistical significance level.
}
  \item{pop.avg}{
  a logical value indicating whether the population-averaged proportion and its confidence interval are to be produced. This quantity is the marginal mean of study-specific proportions, while the commonly-reported overall proportion usually represents the median (or interpreted as a conditional measure); see more details about this quantity in Section 13.2.3 in Agresti (2013), Chu et al.{\sspace}(2012), Lin and Chu (2020), and Zeger et al.{\sspace}(1988). If \code{pop.avg} = \code{TRUE} (the default), the bootstrap resampling is used to produce the confidence interval of the population-averaged proportion; the confidence interval of the commonly-reported median proportion will be also produced, in addition to its conventional confidence interval (by back-transforming the Wald-type confidence interval derived on the scale specified by \code{link}).
}
  \item{int.approx}{
  an integer specifying the number of independent standard normal samples for numerically approximating the integration involved in the calculation of the population-averaged proportion; see details in Lin and Chu (2020). It is only used when \code{pop.avg} = \code{TRUE} and \code{link} is not \code{"probit"}. The probit link leads to a closed form of the population-averaged proportion, so it does not need the numerical approximation; for other links, the population-averaged proportion does not have a closed form.
}
  \item{b.iter}{
  an integer specifying the number of bootstrap iterations; it is only used when \code{pop.avg} = \code{TRUE}.
}
  \item{seed}{
  an integer for specifying the seed of the random number generation for reproducibility during the bootstrap resampling (and numerical approximation for the population-averaged proportion); it is only used when \code{pop.avg} = \code{TRUE}.
}
  \item{...}{
  other arguments that can be passed to the function \code{\link[lme4]{glmer}} in the package \pkg{lme4}.
}
}
%%\details{
%%  ~~ If necessary, more details than the description above ~~
%%}
\value{
This function returns a list containing the point and interval estimates of the overall proportion. Specifically, \code{prop.c.est} is the commonly-reported median (or conditional) proportion, and \code{prop.c.ci} is its confidence interval. It also returns information about AIC, BIC, log likelihood, deviance, and residual degrees-of-freedom. If \code{pop.avg} = \code{TRUE}, the following additional elements will be also in the produced list: \code{prop.c.ci.b} is the bootstrap confidence interval of the commonly-reported median (conditional) proportion, \code{prop.m.est} is the point estimate of the population-averaged (marginal) proportion, \code{prop.m.ci.b} is the bootstrap confidence interval of the population-averaged (marginal) proportion, and \code{b.w.e} is a vector of two numeric values, indicating the counts of warnings and errors occurred during the bootstrap iterations.
}
\references{
Agresti A (2013). \emph{Categorical Data Analysis}. Third edition. John Wiley & Sons, Hoboken, NJ.

Bakbergenuly I, Kulinskaya E (2018). "Meta-analysis of binary outcomes via generalized linear mixed models: a simulation study." \emph{BMC Medical Research Methodology}, \bold{18}, 70. <\doi{10.1186/s12874-018-0531-9}>

Chu H, Nie L, Chen Y, Huang Y, Sun W (2012). "Bivariate random effects models for meta-analysis of comparative studies with binary outcomes: methods for the absolute risk difference and relative risk." \emph{Statistical Methods in Medical Research}, \bold{21}(6), 621--633. <\doi{10.1177/0962280210393712}>

Hamza TH, van Houwelingen HC, Stijnen T (2008). "The binomial distribution of meta-analysis was preferred to model within-study variability." \emph{Journal of Clinical Epidemiology}, \bold{61}(1), 41--51. <\doi{10.1016/j.jclinepi.2007.03.016}>

Lin L, Chu H (2020). "Meta-analysis of proportions using generalized linear mixed models." \emph{Epidemiology}, \bold{31}(5), 713--717. <\doi{10.1097/ede.0000000000001232}>

Stijnen T, Hamza TH, Ozdemir P (2010). "Random effects meta-analysis of event outcome in the framework of the generalized linear mixed model with applications in sparse data." \emph{Statistics in Medicine}, \bold{29}(29), 3046--3067. <\doi{10.1002/sim.4040}>

Zeger SL, Liang K-Y, Albert PS (1988). "Models for longitudinal data: a generalized estimating equation approach." \emph{Biometrics}, \bold{44}(4), 1049--1060. <\doi{10.2307/2531734}>
}
%%\author{
%%  ~~who you are~~
%%}
\note{
This function implements the GLMM for the meta-analysis of proportions via the function \code{\link[lme4]{glmer}} in the package \pkg{lme4}. It is possible that the algorithm of the GLMM estimation may not converge for some bootstrapped meta-analyses when \code{pop.avg} = \code{TRUE}, and the function \code{\link[lme4]{glmer}} may report warnings or errors about the convergence issue. The bootstrap iterations are continued until \code{b.iter} replicates without any warnings or errors are obtained; those replicates with any warnings or errors are discarded.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{maprop.twostep}}
}
\examples{
# chorioamnionitis data
data("dat.chor")
# GLMM with the logit link with only 10 bootstrap iterations
out.chor.glmm.logit <- maprop.glmm(e, n, data = dat.chor,
  link = "logit", b.iter = 10, seed = 1234)
out.chor.glmm.logit
# not calculating the population-averaged (marginal) proportion,
#  without bootstrap resampling
out.chor.glmm.logit <- maprop.glmm(e, n, data = dat.chor,
  link = "logit", pop.avg = FALSE)
out.chor.glmm.logit
\donttest{
# increases the number of bootstrap iterations to 1000,
#  taking longer time
out.chor.glmm.logit <- maprop.glmm(e, n, data = dat.chor,
  link = "logit", b.iter = 1000, seed = 1234)
out.chor.glmm.logit

# GLMM with the log link
out.chor.glmm.log <- maprop.glmm(e, n, data = dat.chor,
  link = "log", b.iter = 10, seed = 1234)
out.chor.glmm.log
# GLMM with the probit link
out.chor.glmm.probit <- maprop.glmm(e, n, data = dat.chor,
  link = "probit", b.iter = 10, seed = 1234)
out.chor.glmm.probit
# GLMM with the cauchit link
out.chor.glmm.cauchit <- maprop.glmm(e, n, data = dat.chor,
  link = "cauchit", b.iter = 10, seed = 1234)
out.chor.glmm.cauchit
# GLMM with the cloglog link
out.chor.glmm.cloglog <- maprop.glmm(e, n, data = dat.chor,
  link = "cloglog", b.iter = 10, seed = 1234)
out.chor.glmm.cloglog


# depression data
data("dat.beck17")
out.beck17.glmm.log <- maprop.glmm(e, n, data = dat.beck17,
  link = "log", b.iter = 10, seed = 1234)
out.beck17.glmm.log
out.beck17.glmm.logit <- maprop.glmm(e, n, data = dat.beck17,
  link = "logit", b.iter = 10, seed = 1234)
out.beck17.glmm.logit
out.beck17.glmm.probit <- maprop.glmm(e, n, data = dat.beck17,
  link = "probit", b.iter = 10, seed = 1234)
out.beck17.glmm.probit
out.beck17.glmm.cauchit <- maprop.glmm(e, n, data = dat.beck17,
  link = "cauchit", b.iter = 10, seed = 1234)
out.beck17.glmm.cauchit
out.beck17.glmm.cloglog<- maprop.glmm(e, n, data = dat.beck17,
  link = "cloglog", b.iter = 10, seed = 1234)
out.beck17.glmm.cloglog
}
}
\keyword{generalized linear mixed model}
\keyword{proportion}

\name{mvma.bayesian}
\alias{mvma.bayesian}

\title{
Bayesian Random-Effects Multivariate Meta-Analysis
}
\description{
Performs a Bayesian random-effects model for multivariate meta-analysis when the within-study correlations are known.
}
\usage{
mvma.bayesian(ys, covs, data, n.adapt = 1000, n.chains = 3,
              n.burnin = 10000, n.iter = 10000, n.thin = 1,
              data.name = NULL, traceplot = FALSE, coda = FALSE)
}

\arguments{
  \item{ys}{
  an n x p numeric matrix containing the observed effect sizes. The n rows represent studies, and the p columns represent the multivariate endpoints. \code{NA} is allowed for missing endpoints.
}
  \item{covs}{
  a numeric list with length n. Each element is the p x p within-study covariance matrix. \code{NA} is allowed for missing endpoints in the covariance matrix.
}
  \item{data}{
  an optional data frame containing the multivariate meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{ys} and \code{covs}, should be specified as their corresponding column names in \code{data}.
}
  \item{n.adapt}{
  the number of iterations for adaptation in the Markov chain Monte Carlo (MCMC) algorithm. The default is 1,000. This argument and the following \code{n.chains}, \code{n.burnin}, \code{n.iter}, and \code{n.thin} are passed to the functions in the package \pkg{rjags}.
}
  \item{n.chains}{
  the number of MCMC chains. The default is 3.
}
  \item{n.burnin}{
  the number of iterations for burn-in period. The default is 10,000.
}
  \item{n.iter}{
  the total number of iterations in each MCMC chain after the burn-in period. The default is 10,000.
}
  \item{n.thin}{
  a positive integer specifying thinning rate. The default is 1.
}
  \item{data.name}{
  a character string specifying the data name. This is used in the names of the generated files that contain results. The default is \code{NULL}.
}
  \item{traceplot}{
  a logical value indicating whether to save trace plots for the overall effect sizes and between-study standard deviations. The default is \code{FALSE}.
}
  \item{coda}{
  a logical value indicating whether to output MCMC posterior samples. The default is \code{FALSE}.
}
}
\details{
Suppose \eqn{n} studies are collected in a multivariate meta-analysis on a total of \eqn{p} endpoints. Denote the \eqn{p}-dimensional vector of effect sizes as \eqn{\boldsymbol{y}_i}, and the within-study covariance matrix \eqn{\mathbf{S}_i} is assumed to be known. Then, the random-effects model is as follows:
\deqn{\boldsymbol{y}_i \sim N (\boldsymbol{\mu}_i, \mathbf{S}_i);}
\deqn{\boldsymbol{\mu}_i \sim N (\boldsymbol{\mu}, \mathbf{T}).}
Here, \eqn{\boldsymbol{\mu}_i} represents the true underlying effect sizes in study \eqn{i}, \eqn{\boldsymbol{\mu}} represents the overall effect sizes across studies, and \eqn{\mathbf{T}} is the between-study covariance matrix due to heterogeneity.

The vague priors \eqn{N (0, 10^3)} are specified for the fixed effects \eqn{\boldsymbol{\mu}}. Also, this function uses the separation strategy to specify vague priors for the variance and correlation components in \eqn{\mathbf{T}} (Pinheiro and Bates, 1996); this technique is considered less sensitive to hyperparameters compared to specifying the inverse-Wishart prior (Lu and Ades, 2009; Wei and Higgins, 2013). Specifically, write the between-study covariance matrix as \eqn{\mathbf{T} = \mathbf{D}^{1/2} \mathbf{R} \mathbf{D}^{1/2}}, where the diagonal matrix \eqn{\mathbf{D} = diag(\mathbf{T}) = diag(\tau_1^2, \ldots, \tau_p^2)} contains the between-study variances, and \eqn{\mathbf{R}} is the correlation matrix. Uniform priors \eqn{U (0, 10)} are specified for \eqn{\tau_j}'s (\eqn{j = 1, \ldots, p}). Further, the correlation matrix can be written as \eqn{\mathbf{R} = \mathbf{L} \mathbf{L}^\prime}, where \eqn{\mathbf{L} = (L_{ij})} is a lower triangular matrix with nonnegative diagonal elements. Also, \eqn{L_{11} = 1} and for \eqn{i = 2, \ldots, p}, \eqn{L_{ij} = \cos \theta_{i2}} if \eqn{j = 1}; \eqn{L_{ij} = (\prod_{k = 2}^{j} \sin \theta_{ik}) \cos \theta_{i, j + 1}} if \eqn{j = 2, \ldots, i - 1}; and \eqn{L_{ij} = \prod_{k = 2}^{i} \sin \theta_{ik}} if \eqn{j = i}. Here, \eqn{\theta_{ij}}'s are angle parameters for \eqn{2 \leq j \leq i \leq p}, and \eqn{\theta_{ij} \in (0, \pi)}. Uniform priors are specified for the angle parameters: \eqn{\theta_{ij} \sim U (0, \pi)}.
}
\value{
This functions produces posterior estimates and Gelman and Rubin's potential scale reduction factor, and it generates several files that contain trace plots (if \code{traceplot} = \code{TRUE}) and MCMC posterior samples (if \code{coda} = \code{TRUE}) in users' working directory. In these results, \code{mu} represents the overall effect sizes, \code{tau} represents the between-study variances, \code{R} contains the elements of the correlation matrix, and \code{theta} represents the angle parameters (see "Details").
}
\references{
Gelman A, Rubin DB (1992). "Inference from iterative simulation using multiple sequences." \emph{Statistical Science}, \bold{7}(4), 457--472. <\doi{10.1214/ss/1177011136}>

Jackson D, Riley R, White IR (2011). "Multivariate meta-analysis: potential and promise." \emph{Statistics in Medicine}, \bold{30}(20), 2481--2498. <\doi{10.1002/sim.4172}>

Lu G, Ades AE (2009). "Modeling between-trial variance structure in mixed treatment comparisons." \emph{Biostatistics}, \bold{10}(4), 792--805. <\doi{10.1093/biostatistics/kxp032}>

Pinheiro JC, Bates DM (1996). "Unconstrained parametrizations for variance-covariance matrices." \emph{Statistics and Computing}, \bold{6}(3), 289--296. <\doi{10.1007/BF00140873}>

Wei Y, Higgins JPT (2013). "Bayesian multivariate meta-analysis with multiple outcomes." \emph{Statistics in Medicine}, \bold{32}(17), 2911--2934. <\doi{10.1002/sim.5745}>
}

\note{
This function only implements the MCMC algorithm for the random-effects multivariate model, but not the fixed-effects model. Generally, the fixed-effects model can be easily implemented using the function \code{\link{mvma}}. However, when using \code{\link{mvma}} to fit the random-effects model, a large number of parameters need to be estimated, and the algorithm for maximizing (restricted) likelihood may not converge well. The Bayesian method in this function provides an alternative.

If a warning "adaptation incomplete" appears, users may increase \code{n.adapt}.
}

\seealso{
\code{\link{mvma}}, \code{\link{mvma.hybrid}}, \code{\link{mvma.hybrid.bayesian}}
}
\examples{
\donttest{
data("dat.fib")
set.seed(12345)
## increase n.burnin and n.iter for better convergence of MCMC
out <- mvma.bayesian(ys = y, covs = S, data = dat.fib,
  n.adapt = 1000, n.chains = 3, n.burnin = 100, n.iter = 100,
  n.thin = 1, data.name = "Fibrinogen")
out
}
}
\keyword{multivariate meta-analysis}

\name{plot.metaoutliers}
\alias{plot.metaoutliers}

\title{
Standardized Residual Plot for Outliers Diagnostics
}
\description{
Draws a plot showing study-specific standardized residuals.
}
\usage{
\method{plot}{metaoutliers}(x, xtick.cex = 1, ytick.cex = 0.5, ...)
}

\arguments{
  \item{x}{
  an object created by the function \code{\link{metaoutliers}}.
}
  \item{xtick.cex}{
  a numeric value specifying the size of ticks on the x axis.
}
  \item{ytick.cex}{
  a numeric value specifying the size of ticks on the y axis.
}
  \item{...}{
  other arguments that can be passed to the function \code{\link[graphics]{plot.default}}.
}
}

\value{
None.
}

\seealso{
\code{\link{metaoutliers}}
}
\examples{
data("dat.aex")
attach(dat.aex)
out.aex <- metaoutliers(y, s2, model = "FE")
detach(dat.aex)
plot(out.aex)

data("dat.hipfrac")
attach(dat.hipfrac)
out.hipfrac <- metaoutliers(y, s2, model = "RE")
detach(dat.hipfrac)
plot(out.hipfrac)
}
\keyword{heterogeneity}
\keyword{plot}

\name{dat.ducharme}
\alias{dat.ducharme}
\docType{data}
\title{
A Meta-Analysis on the Effect of Long-Acting Inhaled Beta2-Agonists vs. Control for Chronic Asthma
}
\description{
This meta-analysis serves as an example of meta-analysis with binary outcomes.
}
\usage{data("dat.ducharme")}
\format{
  A data frame containing 33 studies with the following 4 variables within each study.
  \describe{
    \item{\code{n00}}{counts of non-events in treatment group 0 (placebo).}
    \item{\code{n01}}{counts of events in treatment group 0 (placebo).}
    \item{\code{n10}}{counts of non-events in treatment group 1 (beta2-agonists).}
    \item{\code{n11}}{counts of events in treatment group 1 (beta2-agonists).}
  }
}
\note{The original review collected 35 studies; two double-zero-counts studies are excluded from this dataset because their odds ratios are not estimable.}
\source{
Ducharme FM, Ni Chroinin M, Greenstone I, Lasserson TJ (2010). "Addition of long-acting beta2-agonists to inhaled corticosteroids versus same dose inhaled corticosteroids for chronic asthma in adults and children." \emph{Cochrane Database of Systematic Reviews}, \bold{5}, Art.{\sspace}No.: CD005535. <\doi{10.1002/14651858.CD005535.pub2}>
}
\keyword{dataset}

\name{mvma}
\alias{mvma}

\title{
Multivariate Meta-Analysis
}
\description{
Performs a multivariate meta-analysis when the within-study correlations are known.
}
\usage{
mvma(ys, covs, data, method = "reml", tol = 1e-10)
}

\arguments{
  \item{ys}{
  an n x p numeric matrix containing the observed effect sizes. The n rows represent studies, and the p columns represent the multivariate endpoints. \code{NA} is allowed for missing endpoints.
}
  \item{covs}{
  a numeric list with length n. Each element is the p x p within-study covariance matrix. \code{NA} is allowed for missing endpoints in the covariance matrix.
}
  \item{data}{
  an optional data frame containing the multivariate meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{ys} and \code{covs}, should be specified as their corresponding column names in \code{data}.
}
  \item{method}{
  a character string specifying the method for estimating the overall effect sizes. It should be \code{"fe"} (fixed-effects model), \code{"ml"} (random-effects model using the maximum likelihood method), or \code{"reml"} (random-effects model using the restricted maximum likelihood method, the default).
}
  \item{tol}{
  a small number specifying the convergence tolerance for the estimates by maximizing (restricted) likelihood. The default is \code{1e-10}.
}
}
\details{
Suppose \eqn{n} studies are collected in a multivariate meta-analysis on a total of \eqn{p} endpoints. Denote the \eqn{p}-dimensional vector of effect sizes as \eqn{\boldsymbol{y}_i}, and the within-study covariance matrix \eqn{\mathbf{S}_i} is assumed to be known. Then, the random-effects model is as follows:
\deqn{\boldsymbol{y}_i \sim N (\boldsymbol{\mu}_i, \mathbf{S}_i);}
\deqn{\boldsymbol{\mu}_i \sim N (\boldsymbol{\mu}, \mathbf{T}).}
Here, \eqn{\boldsymbol{\mu}_i} represents the true underlying effect sizes in study \eqn{i}, \eqn{\boldsymbol{\mu}} represents the overall effect sizes across studies, and \eqn{\mathbf{T}} is the between-study covariance matrix due to heterogeneity. By setting \eqn{\mathbf{T} = \mathbf{0}}, this model becomes the fixed-effects model.
}
\value{
This function returns a list containing the following elements:
\item{mu.est}{The estimated overall effect sizes of the p endpoints.}
\item{Tau.est}{The estimated between-study covariance matrix.}
\item{mu.cov}{The covariance matrix of the estimated overall effect sizes.}
\item{method}{The method used to produce the estimates.}
}
\references{
Jackson D, Riley R, White IR (2011). "Multivariate meta-analysis: potential and promise." \emph{Statistics in Medicine}, \bold{30}(20), 2481--2498. <\doi{10.1002/sim.4172}>
}

\seealso{
\code{\link{mvma.bayesian}}, \code{\link{mvma.hybrid}}, \code{\link{mvma.hybrid.bayesian}}
}
\examples{
data("dat.fib")
mvma(ys = y, covs = S, data = dat.fib, method = "fe")
\donttest{
mvma(ys = y, covs = S, data = dat.fib, method = "reml")
}
}
\keyword{multivariate meta-analysis}

\name{dat.chor}
\alias{dat.chor}
\docType{data}
\title{
A Meta-Analysis of Proportions on Chorioamnionitis
}
\description{
This dataset serves as an example of meta-analysis of proportions.
}
\usage{data("dat.chor")}
\format{
  A data frame with 21 studies with the following 2 variables within each study.
  \describe{
    \item{\code{e}}{event counts of horioamnionitis.}
    \item{\code{n}}{sample sizes.}
  }
}
\source{
Woodd SL, Montoya A, Barreix M, Pi L, Calvert C, Rehman AM, Chou D, Campbell OMR (2019). "Incidence of maternal peripartum infection: a systematic review and meta-analysis." \emph{PLOS Medicine}, \bold{16}(12), e1002984. <\doi{10.1371/journal.pmed.1002984}>
}
\keyword{dataset}

\name{dat.ha}
\alias{dat.ha}
\docType{data}
\title{
A Meta-Analysis on the Effect of Placebo Interventions for All Clinical Conditions Regarding Patient-Reported Outcomes
}
\description{
This meta-analysis serves as an example to illustrate function usage in the package \pkg{altmeta}.
}
\usage{data("dat.ha")}
\format{
  A data frame containing 109 studies with the observed effect sizes and their within-study variances.
  \describe{
    \item{\code{y}}{the observed effect size for each collected study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Hrobjartsson A, Gotzsche PC (2010). "Placebo interventions for all clinical conditions." \emph{Cochrane Database of Systematic Reviews}, \bold{1}. Art.{\sspace}No.: CD003974. <\doi{10.1002/14651858.CD003974.pub3}>
}
\keyword{dataset}

\name{dat.baker}
\alias{dat.baker}
\docType{data}
\title{
A Network Meta-Analysis on Effects of Pharmacologic Treatments for Chronic Obstructive Pulmonary Disease
}
\description{
This dataset serves as an example of network meta-analysis with binary outcomes.
}
\usage{data("dat.baker")}
\format{
  A dataset of network meta-analysis with binary outcomes, containing 38 studies and 5 treatments.
  \describe{
    \item{\code{sid}}{study IDs.}
    \item{\code{tid}}{treatment IDs.}
    \item{\code{r}}{event counts.}
    \item{\code{n}}{sample sizes.}
  }
}
\details{
Treatment IDs represent: 1) placebo; 2) inhaled corticosteroid; 3) inhaled corticosteroid + long-acting \eqn{\beta_2}-agonist; 4) long-acting \eqn{\beta_2}-agonist; and 5) tiotropium.
}
\source{
Baker WL, Baker EL, Coleman CI (2009). "Pharmacologic treatments for chronic obstructive pulmonary disease: a mixed-treatment comparison meta-analysis." \emph{Pharmacotherapy}, \bold{29}(8), 891--905. <\doi{10.1592/phco.29.8.891}>
}
\keyword{dataset}

\name{dat.whiting}
\alias{dat.whiting}
\docType{data}
\title{
A Meta-Analysis on Adverse Events for the Comparison Cannabinoid vs. Placebo
}
\description{
This dataset serves as an example of meta-analysis of (log) odds ratios.
}
\usage{data("dat.whiting")}
\format{
  A data frame with 29 studies with the following 9 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of log odds ratios.}
    \item{\code{s2}}{sample variances of log odds ratios.}
    \item{\code{n00}}{counts of non-events in treatment group 0 (placebo).}
    \item{\code{n01}}{counts of events in treatment group 0.}
    \item{\code{n10}}{counts of non-events in treatment group 1 (cannabinoid).}
    \item{\code{n11}}{counts of events in treatment group 1 (cannabinoid).}
    \item{\code{n0}}{sample sizes in treatment group 0.}
    \item{\code{n1}}{sample sizes in treatment group 1.}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Whiting PF, Wolff RF, Deshpande S, Di Nisio M, Duffy S, Hernandez AV, Keurentjes JC, Lang S, Misso K, Ryder S, Schmidlkofer S, Westwood M, Kleijnen J (2015). "Cannabinoids for medical use: a systematic review and meta-analysis." \emph{JAMA}, \bold{313}(24), 2456--2473. <\doi{10.1001/jama.2015.6358}>
}
\keyword{dataset}

\name{nma.icdf}
\alias{nma.icdf}
\title{
Evidence Inconsistency Degrees of Freedom in Bayesian Network Meta-Analysis
}
\description{
Calculates evidence inconsistency degrees of freedom (ICDF) in Bayesian network meta-analysis with binary outcomes.
}
\usage{
nma.icdf(sid, tid, r, n, data, type = c("la", "fe", "re"),
         n.adapt = 1000, n.chains = 3, n.burnin = 5000, n.iter = 20000,
         n.thin = 2, traceplot = FALSE, nma.name = NULL, seed = 1234)
}
\arguments{
  \item{sid}{
  a vector specifying the study IDs.
}
  \item{tid}{
  a vector specifying the treatment IDs.
}
  \item{r}{
  a numeric vector specifying the event counts.
}
  \item{n}{
  a numeric vector specifying the sample sizes.
}
  \item{data}{
  an optional data frame containing the network meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{sid}, \code{tid}, \code{r}, and \code{n}, should be specified as their corresponding column names in \code{data}.
}
  \item{type}{
  a character string or a vector of character strings specifying the ICDF measures. It can be chosen from \code{"la"} (the Lu--Ades measure), \code{"fe"} (based on fixed-effects models), and \code{"re"} (based on random-effects models).
}
  \item{n.adapt}{
  the number of iterations for adaptation in the Markov chain Monte Carlo (MCMC) algorithm. The default is 1,000. This argument and the following \code{n.chains}, \code{n.burnin}, \code{n.iter}, and \code{n.thin} are passed to the functions in the package \pkg{rjags}.
}
  \item{n.chains}{
  the number of MCMC chains. The default is 3.
}
  \item{n.burnin}{
  the number of iterations for burn-in period. The default is 5,000.
}
  \item{n.iter}{
  the total number of iterations in each MCMC chain after the burn-in period. The default is 20,000.
}
  \item{n.thin}{
  a positive integer specifying thinning rate. The default is 2.
}
  \item{traceplot}{
  a logical value indicating whether to generate trace plots of network meta-analysis models for calculating the ICDF measures. It is only used when the argument \code{type} includes \code{"fe"} and/or \code{"re"}.
}
  \item{nma.name}{
  a character string for specifying the name of the network meta-analysis, which will be used in the file names of the trace plots. It is only used when \code{traceplot} = \code{TRUE}.
}
  \item{seed}{
  an integer for specifying the seed of the random number generation for reproducibility during the MCMC algorithm for performing Bayesian network meta-analysis models.
}
}
\details{
Network meta-analysis frequenlty assumes that direct evidence is consistent with indirect evidence, but this consistency assumption may not hold in some cases. One may use the ICDF to assess the potential that a network meta-analysis might suffer from inconsistency. Suppose that a network meta-analysis compares a total of \eqn{K} treatments. When it contains two-arm studies only, Lu and Ades (2006) propose to measure the ICDF as
\deqn{ICDF^{LA} = T - K + 1,}
where \eqn{T} is the total number of treatment pairs that are directly compared in the treatment network. This measure is interpreted as the number of independent treatment loops. However, it may not be feasibly calculated when the network meta-analysis contains multi-arm studies. Multi-arm studies provide intrinsically consistent evidence and complicate the calculation of the ICDF.

Alternatively, the ICDF may be measured as the difference between the effective numbers of parameters of the consistency and inconsistency models for network meta-analysis. The consistency model assumes evidence consistency in all treatment loops, and the inconsistency model treats the overall effect sizes of all direct treatment comparisons as separate, unrelated parameters (Dias et al., 2013). The effective number of parameters is frequently used to assess the complexity of Bayesian hierarchical models (Spiegelhalter et al., 2002). The effect measure is the (log) odds ratio in the models used in this function. Let \eqn{p_D^{FE,con}} and \eqn{p_D^{FE,incon}} be the effective numbers of parameters of the consistency and inconsistency models under the fixed-effects setting, and \eqn{p_D^{RE,con}} and \eqn{p_D^{RE,incon}} be those under the random-effects setting. The ICDF measures under the fixed-effects and random-effects settings are
\deqn{ICDF^{FE} = p_D^{FE,incon} - p_D^{FE,con};}
\deqn{ICDF^{RE} = p_D^{RE,incon} - p_D^{RE,con},}
respectively. See more details in Lin (2020).
}
\value{
This function produces a list containing the following results: a table of the number of arms within studies and the corresponding counts of studies (\code{nstudy.trtarm}); the number of multi-arm studies (\code{nstudy.multi}); the set of treatments compared in each multi-arm study (\code{multi.trtarm}); the Lu--Ades ICDF measure (\code{icdf.la}); the ICDF measure based on the fixed-effects consistency and inconsistency models (\code{icdf.fe}); and the ICDF measure based on the random-effects consistency and inconsistency models (\code{icdf.re}). The Lu--Ades ICDF measure is \code{NA} (not available) in the presence of multi-arm studies, because multi-arm studies complicate the counting of independent treatment loops in generic network meta-analyses. When \code{traceplot} = \code{TRUE}, the trace plots will be saved in users' working directory.
}
\references{
Dias S, Welton NJ, Sutton AJ, Caldwell DM, Lu G, Ades AE (2013). "Evidence synthesis for decision making 4: inconsistency in networks of evidence based on randomized controlled trials." \emph{Medical Decision Making}, \bold{33}(5), 641--656. <\doi{10.1177/0272989X12455847}>

Lin L (2021). "Evidence inconsistency degrees of freedom in Bayesian network meta-analysis." \emph{Journal of Biopharmaceutical Statistics}, \bold{31}(3), 317--330. <\doi{10.1080/10543406.2020.1852247}>

Lu G, Ades AE (2006). "Assessing evidence inconsistency in mixed treatment comparisons." \emph{Journal of the American Statistical Association}, \bold{101}(474), 447--459. <\doi{10.1198/016214505000001302}>

Spiegelhalter DJ, Best NG, Carlin BP, Van Der Linde A (2002). "Bayesian measures of model complexity and fit." \emph{Journal of the Royal Statistical Society, Series B (Statistical Methodology)}, \bold{64}(4), 583--639. <\doi{10.1111/1467-9868.00353}>
}
%%\author{
%%  ~~who you are~~
%%}
%%\note{
%%  ~~further notes~~
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%%}
\examples{
\donttest{
data("dat.baker")
## increase n.burnin (e.g., to 50000) and n.iter (e.g., to 200000)
##  for better convergence of MCMC
out <- nma.icdf(sid, tid, r, n, data = dat.baker,
  type = c("la", "fe", "re"),
  n.adapt = 1000, n.chains = 3, n.burnin = 500, n.iter = 2000,
  n.thin = 2, traceplot = FALSE, seed = 1234)
out
}
}
\keyword{multivariate meta-analysis}

\name{metaoutliers}
\alias{metaoutliers}

\title{
Outlier Detection in Meta-Analysis
}
\description{
Calculates the standardized residual for each study in meta-analysis using the methods desribed in Chapter 12 in Hedges and Olkin (1985) and Viechtbauer and Cheung (2010). A study is considered as an outlier if its standardized residual is greater than 3 in absolute magnitude.
}
\usage{
metaoutliers(y, s2, data, model)
}

\arguments{
  \item{y}{
  a numeric vector specifying the observed effect sizes in the collected studies; they are assumed to be normally distributed.
}
  \item{s2}{
  a numeric vector specifying the within-study variances.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y} and \code{s2}, should be specified as their corresponding column names in \code{data}.
}
  \item{model}{
  a character string specified as either \code{"FE"} or \code{"RE"}. If \code{model} = \code{"FE"}, this function uses the outlier detection procedure for the fixed-effect meta-analysis desribed in Chapter 12 in Hedges and Olkin (1985); If \code{model} = \code{"RE"}, the procedure for the random-effects meta-analysis desribed in Viechtbauer and Cheung (2010) is used. See Details for the two approaches. If the argument \code{model} is not specified, this function sets \code{model} = \code{"FE"} if \eqn{I_r^2 < 30\%} and sets \code{model} = \code{"RE"} if \eqn{I_r^2 \geq 30\%}.
}
}
\details{
Suppose that a meta-analysis collects \eqn{n} studies. The observed effect size in study \eqn{i} is \eqn{y_i} and its within-study variance is \eqn{s^{2}_{i}}. Also, the inverse-variance weight is \eqn{w_i = 1 / s^{2}_{i}}.

Chapter 12 in Hedges and Olkin (1985) describes the outlier detection procedure for the fixed-effect meta-analysis (\code{model} = \code{"FE"}). Using the studies except study \eqn{i}, the pooled estimate of the overall effect size is \eqn{\bar{\mu}_{(-i)} = \sum_{j \neq i} w_j y_j / \sum_{j \neq i} w_j}. The residual of study \eqn{i} is \eqn{e_{i} = y_i - \bar{\mu}_{(-i)}}. The variance of \eqn{e_{i}} is \eqn{v_{i} = s_{i}^{2} + (\sum_{j \neq i} w_{j})^{-1}}, so the standardized residual of study \eqn{i} is \eqn{\epsilon_{i} = e_{i} / \sqrt{v_{i}}}.

Viechtbauer and Cheung (2010) describes the outlier detection procedure for the random-effects meta-analysis (\code{model} = \code{"RE"}). Using the studies except study \eqn{i}, let the method-of-moments estimate of the between-study variance be \eqn{\hat{\tau}_{(-i)}^{2}}. The pooled estimate of the overall effect size is \eqn{\bar{\mu}_{(-i)} = \sum_{j \neq i} \tilde{w}_{(-i)j} y_j / \sum_{j \neq i} \tilde{w}_{(-i)j}}, where \eqn{\tilde{w}_{(-i)j} = 1/(s_{j}^{2} + \hat{\tau}_{(-i)}^{2})}. The residual of study \eqn{i} is \eqn{e_{i} = y_i - \bar{\mu}_{(-i)}}, and its variance is \eqn{v_{i} = s_{i}^2 + \hat{\tau}_{(-i)}^{2} + (\sum_{j \neq i} \tilde{w}_{(-i)j})^{-1}}. Then, the standardized residual of study \eqn{i} is \eqn{\epsilon_{i} = e_{i} / \sqrt{v_{i}}}. 
}
\value{
This functions returns a list which contains standardized residuals and identified outliers. A study is considered as an outlier if its standardized residual is greater than 3 in absolute magnitude.
}
\references{
Hedges LV, Olkin I (1985). \emph{Statistical Method for Meta-Analysis}. Academic Press, Orlando, FL.

Viechtbauer W, Cheung MWL (2010). "Outlier and influence diagnostics for meta-analysis." \emph{Research Synthesis Methods}, \bold{1}(2), 112--125. <\doi{10.1002/jrsm.11}>
}

\examples{
data("dat.aex")
metaoutliers(y, s2, dat.aex, model = "FE")
metaoutliers(y, s2, dat.aex, model = "RE")

data("dat.hipfrac")
metaoutliers(y, s2, dat.hipfrac)
}
\keyword{heterogeneity}

\name{dat.smith}
\alias{dat.smith}
\docType{data}
\title{
Meta-Analysis on the Diagnostic Accuracy of Ultrasound for Detecting Partial Thickness Rotator Cuff Tears
}
\description{
This meta-analysis serves as an example of meta-analyses of diagnostic tests.
}
\usage{data("dat.smith")}
\format{
  A data frame with 30 studies with the following 4 variables; each row represents a study.
  \describe{
    \item{\code{tp}}{counts of true positives.}
    \item{\code{fp}}{counts of false positives.}
    \item{\code{fn}}{counts of false negatives.}
    \item{\code{tn}}{counts of true negatives.}
  }
}
\source{
Smith TO, Back T, Toms AP, Hing CB (2011). "Diagnostic accuracy of ultrasound for rotator cuff tears in adults: a systematic review and meta-analysis." \emph{Clinical Radiology}, \bold{66}(11), 1036--1048. <\doi{10.1016/j.crad.2011.05.007}>
}
\keyword{dataset}

\name{dat.poole}
\alias{dat.poole}
\docType{data}
\title{
A Meta-Analysis for Evaluating the Effect of Mucolytic on Bronchitis/Chronic Obstructive Pulmonary Disease
}
\description{
This meta-analysis serves as an example of meta-analysis with binary outcomes.
}
\usage{data("dat.poole")}
\format{
  A data frame containing 24 studies with the following 4 variables within each study.
  \describe{
    \item{\code{n00}}{counts of non-events in treatment group 0 (placebo).}
    \item{\code{n01}}{counts of events in treatment group 0 (placebo).}
    \item{\code{n10}}{counts of non-events in treatment group 1 (mucolytic).}
    \item{\code{n11}}{counts of events in treatment group 1 (mucolytic).}
  }
}
\source{
Poole P, Chong J, Cates CJ (2015). "Mucolytic agents versus placebo for chronic bronchitis or chronic obstructive pulmonary disease." \emph{Cochrane Database of Systematic Reviews}, \bold{7}, Art.{\sspace}No.: CD001287. <\doi{10.1002/14651858.CD001287.pub5}>
}
\keyword{dataset}

\name{dat.barlow}
\alias{dat.barlow}
\docType{data}
\title{
A Meta-Analysis on the Effect of Parent Training Programs vs. Control for Improving Parental Psychosocial Health Within 4 Weeks After Intervention
}
\description{
This dataset serves as an example of meta-analysis of standardized mean differences.
}
\usage{data("dat.barlow")}
\format{
  A data frame with 26 studies with the following 5 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of standardized mean differences.}
    \item{\code{s2}}{sample variances of standardized mean differences.}
    \item{\code{n1}}{sample sizes in treatment group 1 (parent training programs).}
    \item{\code{n2}}{sample sizes in treatment group 2 (control).}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Barlow J, Smailagic N, Huband N, Roloff V, Bennett C (2014). "Group-based parent training programmes for improving parental psychosocial health." \emph{Cochrane Database of Systematic Reviews}, \bold{5}, Art.{\sspace}No.: CD002020. <\doi{10.1002/14651858.CD002020.pub4}>
}
\keyword{dataset}

\name{dat.bellamy}
\alias{dat.bellamy}
\docType{data}
\title{
A Meta-Analysis on Type 2 Diabetes Mellitus After Gestational Diabetes
}
\description{
This meta-analysis serves as an example of meta-analysis with binary outcomes.
}
\usage{data("dat.bellamy")}
\format{
  A data frame containing 20 cohort studies with the following 4 variables.
  \describe{
    \item{\code{sid}}{study IDs.}
    \item{\code{tid}}{treatment/exposure IDs (0: non-exposure; 1: exposure).}
    \item{\code{e}}{event counts.}
    \item{\code{n}}{sample sizes.}
  }
}
\source{
Bellamy L, Casas JP, Hingorani AD, Williams D (2009). "Type 2 diabetes mellitus after gestational diabetes: a systematic review and meta-analysis." \emph{Lancet}, \bold{373}(9677), 1773--1779. <\doi{10.1016/S0140-6736(09)60731-5}>
}
\keyword{dataset}

\name{dat.scheidler}
\alias{dat.scheidler}
\docType{data}
\title{
Meta-Analysis on the Utility of Lymphangiography, Computed Tomography, and Magnetic Resonance Imaging for the Diagnosis of Lymph Node Metastasis
}
\description{
This meta-analysis serves as an example of meta-analyses of diagnostic tests.
}
\usage{data("dat.scheidler")}
\format{
  A data frame with 44 studies with the following 5 variables; each row represents a study.
  \describe{
    \item{\code{dt}}{types of diagnostic tests; CT: computed tomography; LAG: lymphangiography; and MRI: magnetic resonance imaging.}
    \item{\code{tp}}{counts of true positives.}
    \item{\code{fp}}{counts of false positives.}
    \item{\code{fn}}{counts of false negatives.}
    \item{\code{tn}}{counts of true negatives.}
  }
}
\source{
Scheidler J, Hricak H, Yu KK, Subak L, Segal MR (1997). "Radiological evaluation of lymph node metastases in patients with cervical cancer: a meta-analysis." \emph{JAMA}, \bold{278}(13), 1096--1101. <\doi{10.1001/jama.1997.03550130070040}>
}
\keyword{dataset}

\name{dat.annane}
\alias{dat.annane}
\docType{data}
\title{
A Meta-Analysis for Comparing the Effect of Steroids vs. Control in the Length of Intensive Care Unit (ICU) Stay
}
\description{
This dataset serves as an example of meta-analysis of mean differences.
}
\usage{data("dat.annane")}
\format{
  A data frame with 12 studies with the following 5 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of mean differences.}
    \item{\code{s2}}{sample variances of mean differences.}
    \item{\code{n1}}{sample sizes in treatment group 1 (steroids).}
    \item{\code{n2}}{sample sizes in treatment group 2 (control).}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Annane D, Bellissant E, Bollaert PE, Briegel J, Keh D, Kupfer Y (2015). "Corticosteroids for treating sepsis." \emph{Cochrane Database of Systematic Reviews}, \bold{12}, Art.{\sspace}No.: CD002243. <\doi{10.1002/14651858.CD002243.pub3}>
}
\keyword{dataset}

\name{dat.beck17}
\alias{dat.beck17}
\docType{data}
\title{
A Meta-Analysis of Prevalence of Depression or Depressive Symptoms Among Medical Students
}
\description{
This dataset serves as an example of meta-analysis of proportions.
}
\usage{data("dat.beck17")}
\format{
  A data frame with 6 studies with the following 2 variables within each study.
  \describe{
    \item{\code{e}}{event counts of samples with depression or depressive symptoms.}
    \item{\code{n}}{sample sizes.}
  }
}
\details{
The original article by Rotenstein et al.{\sspace}(2016) stratified all extracted studies based on various screening instruments and cutoff scores. This dataset focuses on the meta-analysis of 6 studies with Beck Depression Inventory Score \eqn{\geq}17.
}
\source{
Rotenstein LS, Ramos MA, Torre M, Segal JB, Peluso MJ, Guille C, Sen S, Mata DA (2016). "Prevalence of depression, depressive symptoms, and suicidal ideation among medical students: a systematic review and meta-analysis." \emph{JAMA}, \bold{316}(21), 2214--2236. <\doi{10.1001/jama.2016.17324}>
}
\keyword{dataset}

\name{dat.paige}
\alias{dat.paige}
\docType{data}
\title{
A Meta-Analysis on the Effectiveness of Spinal Manipulative Therapies (Other Than Sham)
}
\description{
This dataset serves as an example of meta-analysis of standardized mean differences.
}
\usage{data("dat.paige")}
\format{
  A data frame with 6 studies with the following 5 variables within each study.
  \describe{
    \item{\code{y}}{point esimates of standardized mean differences.}
    \item{\code{s2}}{sample variances of standardized mean differences.}
    \item{\code{n1}}{sample sizes in treatment group 1 (spinal manipulation).}
    \item{\code{n2}}{sample sizes in treatment group 2 (comparator).}
    \item{\code{n}}{total sample sizes.}
  }
}
\source{
Paige NM, Miake-Lye IM, Booth MS, Beroes JM, Mardian AS, Dougherty P, Branson R, Tang B, Morton SC, Shekelle PG (2017). "Association of spinal manipulative therapy with clinical benefit and harm for acute low back pain: systematic review and meta-analysis." \emph{JAMA}, \bold{317}(14), 1451--1460. <\doi{10.1001/jama.2017.3086}>
}
\keyword{dataset}

\name{meta.pen}
\alias{meta.pen}
\title{
A penalization approach to random-effects meta-analysis
}
\description{
Performs the penalization methods introduced in Wang et al.{\sspace}(2022) to achieve a compromise between the common-effect and random-effects model.
}
\usage{
meta.pen(y, s2, data, tuning.para = "tau", upp = 1, n.cand = 100, tol = 1e-10)
}
\arguments{
  \item{y}{
  a numeric vector or the corresponding column name in the argument data, specifying the observed effect sizes in the collected studies.
}
  \item{s2}{
  a numeric vector or the corresponding column name in the argument data, specifying the within-study variances.
}
  \item{data}{
  an optional data frame containing the meta-analysis dataset. If \code{data} is specified, the previous arguments, \code{y} and \code{s2}, should be specified as their corresponding column names in \code{data}.
}
  \item{tuning.para}{
  a character string specifying the type of tuning parameter used in the penalization method. It should be one of "lambda" (use lambda as a tuning parameter) or "tau" (use the standard deviation as a tuning parameter). The default is "tau".
}
  \item{upp}{
  a positive scalar used to control the upper bound of the range for the tuning parameter. Specifically, [0, T*\code{upp}] is used as the range of a tuning parameter. T is the upper threshold value of a tuning parameter. The default value of \code{upp} is 1. 
}
  \item{n.cand}{
  the total number of candidate values of the tuning parameter within the specified range. The default is 100. 
}
  \item{tol}{
  the desired accuracy (convergence tolerance). The default is 1e-10.
}
}
\details{
Suppose a meta-analysis collects \eqn{n} independent studies. Let \eqn{\mu_{i}} be the true effect size in study \eqn{i} (\eqn{i} = 1, \ldots, \eqn{n}). Each study reports an estimate of the effect size and its sample variance, denoted by \eqn{y_{i}} and \eqn{s_{i}^{2}}, respectively. These data are commonly modeled as \eqn{y_{i}} from \eqn{N(\mu_{i},s_{i}^{2})}. If study-specific true effect sizes \eqn{\mu_{i}} are assumed i.i.d.{\sspace}from \eqn{N(\mu, \tau^{2})}, this is the random-effects (RE) model, where \eqn{\mu} is the overall effect size and \eqn{\tau^{2}} is the between-study variance. If \eqn{\tau^{2}=0} and thus \eqn{\mu_{i}=\mu} for all studies, this implies that studies are homogeneous and the RE model is reduced to the common-effect (CE) model.

Marginally, the RE model yields \eqn{y_{i} \sim N(\mu,s_{i}^2+\tau^{2})}, and its log-likelihood is 
\deqn{l(\mu, \tau^{2}) = -\frac{1}{2} \sum_{i=1}^{n} \left[\log(s_{i}^{2} + \tau^{2}) + \frac{(y_{i} - \mu)^2}{s_{i}^2 + \tau^2}\right] + C,}
where \eqn{C} is a constant. In the past two decades, penalization methods have been rapidly developed for variable selection in high-dimensional data analysis to control model complexity and reduce the variance of parameter estimates. Borrowing the idea from the penalization methods, we employ a penalty term on the between-study variance \eqn{\tau^2} when the heterogeneity is overestimated. The penalty term increases with \eqn{\tau^2}. Specifically, we consider the following optimization problem:
\deqn{(\hat{\mu}(\lambda), \hat{\tau}^2(\lambda)) = \min_{\mu, \tau^2 \geq 0} \left\{\sum_{i=1}^{n} \left[ \log(s_{i}^{2} + \tau^{2}) + \frac{(y_{i} - \mu)^2}{s_{i}^2 + \tau^2}\right] + \lambda \tau^2\right\},}
where \eqn{\lambda \geq 0} is a tuning parameter that controls the penalty strength. Using the technique of profile likelihood by taking the target function's derivative in the above equation with respect to \eqn{\mu} for a given \eqn{\tau^2}. The bivariate optimization problem is reduced to a univariate minimization problem. When \eqn{\lambda=0}, the minimization problem is equivalent to minimizing the log-likelihood without penalty, so the penalized-likelihood method is identical to the conventional RE model. By contrast, it can be shown that a sufficiently large \eqn{\lambda} produces the estimated between-study variance as 0, leading to the conventional CE model.

As different tuning parameters lead to different estimates of \eqn{\hat{\mu}(\lambda)} and \eqn{\hat{\tau}^2(\lambda)}, it is important to select the optimal \eqn{\lambda} among a set of candidate values. We perform the cross-validation process and construct a loss function of \eqn{\lambda} to measure the performance of specific \eqn{\lambda} values. The \eqn{\lambda} corresponding to the smallest loss is considered optimal. The threshold, denoted by \eqn{\lambda_{\max}}, based on the penalty function \eqn{p(\tau^2)=\tau^2} can be calculated. For all \eqn{\lambda > \lambda_{\max}}, the estimated between-study variance is 0. Consequently, we select a certain number of candidate values (e.g., 100) from the range \eqn{[0,\lambda_{\max}]} for the tuning parameter. For a set of tuning parameters, the leave-one-study-out (i.e., \eqn{n}-fold) cross-validation is used to construct the loss function. Specifically, we use the following loss function for the penalization method by tuning \eqn{\lambda}:
\deqn{\hat{L}(\lambda) = \left[\frac{1}{n} \sum_{i=1}^{n} \frac{(y_{i} - \hat{\mu}_{(-i)}(\lambda))^2}{s_{i}^2 + \hat{\tau}_{RE(-i)}^2 + \frac{\sum_{j \ne i}(s_{j}^2 + \hat{\tau}_{RE(-i)}^2) / (s_{j}^2 + \hat{\tau}^2_{(-i)}(\lambda))^2}{(\sum_{j \ne i} 1 / (s_{j}^2 + \hat{\tau}^2_{(-i)}(\lambda)))^2}}\right]^{1/2},}
where the subscript \eqn{(-i)} indicates that study \eqn{i} is removed, \eqn{\hat{\tau}^2_{(-i)}(\lambda)} is the estimated between-study variance for a given \eqn{\lambda}, \eqn{\hat{\tau}_{RE(-i)}^2} is the corresponding between-study variance estimate of the RE model, and 
\deqn{\hat{\mu}_{(-i)}(\lambda) = \frac{\sum_{j \ne i} y_{j} / (s_{j}^2 + \hat{\tau}^2_{(-i)}(\lambda))}{\sum_{j \ne i} 1 / (s_{j}^2 + \hat{\tau}^2_{(-i)}(\lambda))}.}

The above procedure focuses on tuning the parameter, \eqn{\lambda}, to control the penalty strength for the between-study variance. Alternatively, for the purpose of shrinking the potentially overestimated heterogeneity, we can directly treat the between-study standard deviation (SD), \eqn{\tau}, as the tuning parameter. A set of candidate values of \eqn{\tau} are considered, and the value that produces the minimum loss function is selected. Compared with tuning \eqn{\lambda} from the perspective of penalized likelihood, tuning \eqn{\tau} is more straightforward and intuitive from the practical perspective. The candidate values of \eqn{\tau} can be naturally chosen from \eqn{[0,\hat{\tau}_{RE}]}, with the lower and upper bounds corresponding to the CE and RE models, respectively. Denoted the candidate SDs as \eqn{\tau_{t}}, the loss function with respect to \eqn{\tau_{t}} is similarly defined as
\deqn{\hat{L}(\tau_{t}) = \left[\frac{1}{n} \sum_{i=1}^{n} \frac{(y_i - \hat{\mu}_{(-i)}(\tau_{t}))^2}{s_{i}^2 + \hat{\tau}_{RE(-i)} + \frac{\sum_{j \ne i}(s_{j}^2 + \hat{\tau}^2_{RE(-i)}) / (s_{j}^2 + \tau_{t}^2)^2}{[\sum_{j \ne i} 1 / (s_{j}^2 + \tau_{t}^2)]^2}}\right]^{1/2},}
where the overall effect size estimate (excluding study \eqn{i}) is
\deqn{\hat{\mu}_{(-i)}(\tau_{t}) = \frac{\sum_{j \ne i} y_j / (s_{j}^2 + \tau_{t}^2)}{\sum_{j \ne i} 1 / (s_{j}^2 + \tau_{t}^2)}.}
}
\value{
This function returns a list containing estimates of the overall effect size and their 95\% confidence intervals. Specifically, the components include:
  \item{n}{the number of studies in the meta-analysis.}
  \item{I2}{the \eqn{I^2} statistic for quantifying heterogeneity.}
  \item{tau2.re}{the maximum likelihood estimate of the between-study variance.}
  \item{mu.fe}{the estimated overall effect size of the \code{CE} model.}
  \item{se.fe}{the standard deviation of the overall effect size estimate of the \code{CE} model.}
  \item{mu.re}{the estimated overall effect size of the \code{RE} model.}
  \item{se.re}{the standard deviation of the overall effect size estimate of the \code{RE} model.}
  \item{loss}{the values of the loss function for candidate tuning parameters.}
  \item{tau.cand}{the candidate values of the tuning parameter \eqn{\tau}.}
  \item{lambda.cand}{the candidate values of the tuning parameter \eqn{\lambda}.}
  \item{tau.opt}{the estimated between-study standard deviation of the penalization method.}
  \item{mu.opt}{the estimated overall effect size of the penalization method.}
  \item{se.opt}{the standard error estimate of the estimated overall effect size of the penalization method.}
}
\references{
Wang Y, Lin L, Thompson CG, Chu H (2022). "A penalization approach to random-effects meta-analysis." \emph{Statistics in Medicine}, \bold{41}(3), 500--516. <\doi{10.1002/sim.9261}>
}
\author{
Yipeng Wang \email{yipeng.wang1@ufl.edu}
}
\examples{
data("dat.bohren") ## log odds ratio
## perform the penaliztion method by tuning tau
out11 <- meta.pen(y, s2, dat.bohren)
## plot the loss function and candidate taus
plot(out11$tau.cand, out11$loss, xlab = NA, ylab = NA, 
  lwd = 1.5, type = "l", cex.lab = 0.8, cex.axis = 0.8)
title(xlab = expression(paste(tau[t])),
  ylab = expression(paste("Loss function ", ~hat(L)(tau[t]))),
  cex.lab = 0.8, line = 2)
idx <- which(out11$loss == min(out11$loss))
abline(v = out11$tau.cand[idx], col = "gray", lwd = 1.5, lty = 2)
     
## perform the penaliztion method by tuning lambda     
out12 <- meta.pen(y, s2, dat.bohren, tuning.para = "lambda")
## plot the loss function and candidate lambdas
plot(log(out12$lambda.cand + 1), out12$loss, xlab = NA, ylab = NA, 
  lwd=1.5, type = "l", cex.lab = 0.8, cex.axis = 0.8)
title(xlab = expression(log(lambda + 1)),
  ylab = expression(paste("Loss function ", ~hat(L)(lambda))),
  cex.lab = 0.8, line = 2)
idx <- which(out12$loss == min(out12$loss))
abline(v = log(out12$lambda.cand[idx] + 1), col = "gray", lwd = 1.5, lty = 2)

\donttest{
data("dat.bjelakovic") ## log odds ratio
## perform the penaliztion method by tuning tau
out21 <- meta.pen(y, s2, dat.bjelakovic)
## plot the loss function and candidate taus
plot(out21$tau.cand, out21$loss, xlab = NA, ylab = NA, 
  lwd=1.5, type = "l", cex.lab = 0.8, cex.axis = 0.8)
title(xlab = expression(paste(tau[t])),
  ylab = expression(paste("Loss function ", ~hat(L)(tau[t]))),
  cex.lab = 0.8, line = 2)
idx <- which(out21$loss == min(out21$loss))
abline(v = out21$tau.cand[idx], col = "gray", lwd = 1.5, lty = 2)

out22 <- meta.pen(y, s2, dat.bjelakovic, tuning.para = "lambda")

data("dat.carless") ## log odds ratio
## perform the penaliztion method by tuning tau
out31 <- meta.pen(y, s2, dat.carless)
## plot the loss function and candidate taus
plot(out31$tau.cand, out31$loss, xlab = NA, ylab = NA, 
  lwd=1.5, type = "l", cex.lab = 0.8, cex.axis = 0.8)
title(xlab = expression(paste(tau[t])),
  ylab = expression(paste("Loss function ", ~hat(L)(tau[t]))),
  cex.lab = 0.8, line = 2)
idx <- which(out31$loss == min(out31$loss))
abline(v = out31$tau.cand[idx], col = "gray", lwd = 1.5, lty = 2)

out32 <- meta.pen(y, s2, dat.carless, tuning.para = "lambda")

data("dat.adams") ## mean difference
out41 <- meta.pen(y, s2, dat.adams)
## plot the loss function and candidate taus
plot(out41$tau.cand, out41$loss, xlab = NA, ylab = NA, 
  lwd=1.5, type = "l", cex.lab = 0.8, cex.axis = 0.8)
title(xlab = expression(paste(tau[t])),
  ylab = expression(paste("Loss function ", ~hat(L)(tau[t]))),
  cex.lab = 0.8, line = 2)
idx <- which(out41$loss == min(out41$loss))
abline(v = out41$tau.cand[idx], col = "gray", lwd = 1.5, lty = 2)

out42 <- meta.pen(y, s2, dat.adams, tuning.para = "lambda")
}
}
\keyword{heterogeneity}

\name{dat.carless}
\alias{dat.carless}
\docType{data}
\title{
A Meta-Analysis on the Efficacy of Platelet-Rich-Plasmapheresis in Reducing Peri-Operative Allogeneic Red Blood Cell Transfusion in Cardiac Surgery
}
\description{
This meta-analysis serves as an example to illustrate the usage of the function \code{\link{meta.pen}}.
}
\usage{data("dat.ha")}
\format{
  A data frame containing 20 studies.
  \describe{
    \item{\code{n1}}{the sample size in the treatment group in each study.}
    \item{\code{n2}}{the sample size in the control group in each study.}
    \item{\code{r1}}{the event count in the treatment group in each study.}
    \item{\code{r2}}{the event count in the control group in each study.}
    \item{\code{y}}{the observed effect size (log odds ratio) for each study in the meta-analysis.}
    \item{\code{s2}}{the within-study variance for each study.}
  }
}
\source{
Carless PA, Rubens FD, Anthony DM, O'Connell D, Henry DA (2011). "Platelet-rich-plasmapheresis for minimising peri-operative allogeneic blood transfusion." \emph{Cochrane Database of Systematic Reviews}, \bold{3}. Art.{\sspace}No.: CD004172. <\doi{10.1002/14651858.CD004172.pub2}>
}
\keyword{dataset}
